#include <cmath>
#include <fstream>
#include <iomanip>
#include <string>

#include <string.h>
#include <stdio.h>

#include "NavierStokes.h"
#include "Thompson2D.h"
#include "kinCond1D.h"
#include "streamFunction2D.h"

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Filter.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Result.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;
using namespace keyFlow ;



int main(void) {
  
  char buf[256];
  
  Double visc1=1.0;
  Double visc2=1.0;
  Double capillarity=100.0;
  Double strainRate=1.0;
  Double bubbleRadius=1.0;
  Double bubbleVolume=(2.0/3.0)*3.1415926535
    *bubbleRadius*bubbleRadius*bubbleRadius;
  
  // Reading and renumbering of the mesh:
  
  Mesh MESH;
  MESH.read("brol.mesh","emc2Mesh");
  string sortType="frontal";
  if (sortType=="position")
  {
    Domain Dom=MESH.getDefDomain();
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
		  DERIV=false,COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",
                 COMPONENT(0)=Double(1.0),COMPONENT(1)=Double(1.0e-4)));
    Array ADist=accumulation(ACoord,ABrol,1);
    MESH=renumberDefDomainArray(MESH,ADist);
  }
  else if (sortType=="frontal") {
    Domain Bnd=MESH.getDomain(6);
    MESH=renumberDefDomainFrontal(MESH,Bnd);
  }
  else if (sortType=="none") {
    // Nothing to do!
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }

  // Domains:

  Domain Dom,Dom1,Dom2,Bnd1,Bnd2,Bnd3,Bnd4,Bnd5,Bnd6,Bnd7;
  Dom=MESH.getDefDomain();
  Dom.setName("main domain");
  Dom1=MESH.getDomain(0);
  Dom1.setName("inside the bubble");
  Dom2=MESH.getDomain(1);
  Dom2.setName("outside the bubble");
  Bnd1=MESH.getDomain(2);
  Bnd1.setName("boundary 1");
  Bnd2=MESH.getDomain(3);
  Bnd2.setName("boundary 2");
  Bnd3=MESH.getDomain(4);
  Bnd3.setName("boundary 3");
  Bnd4=MESH.getDomain(5);
  Bnd4.setName("boundary 4");
  Bnd5=MESH.getDomain(6);
  Bnd5.setName("boundary 5");
  Bnd6=MESH.getDomain(7);
  Bnd6.setName("boundary 6");
  Bnd7=MESH.getDomain(8);
  Bnd7.setName("interface");
  
  Domain Corners1,Corners2;
  Corners1=Bnd3.getBoundary();
  Corners2=Bnd4.getBoundary();
  Domain cornerUR=Corners1*Corners2;
  cornerUR.setName("upper right corner");
  Corners1=Bnd2.getBoundary();
  Corners2=Bnd7.getBoundary();
  Domain attachPoint=Corners1*Corners2;
  attachPoint.setName("attachment point of free surface");
  Domain leftBnd=Bnd5+Bnd6;
  leftBnd.setName("left boundary");
  Domain lowerBnd=Bnd1+Bnd2;
  lowerBnd.setName("lower boundary");
  Domain extBnd=Bnd3+Bnd4;
  extBnd.setName("external boundary");
  Domain intBnd=leftBnd+lowerBnd;
  intBnd.setName("internal boundary");
  
  // Coordinate systems:
  string coordInterp="quadratic";
  Field coord; 
  Array SPHERE;
  {
    Field tmpCoord=MESH.getCoordinateField(Dom);
    coord=Field((DOM=Dom,INTERPOLATION=coordInterp,NBRCOMPONENTS=2,
                 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
                 NAME="coordinates")); 
    coord.interpolate(tmpCoord,Dom,tmpCoord);
    
    // Initial sphere:
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,DERIV=false,
                  COMPONENT(0)=tmpCoord.component(0),
                  COMPONENT(1)=tmpCoord.component(1),
                  NAME="ACoord"));
    Array AX((TYPE="arrayField",ORDER=0,COMPONENT()=tmpCoord.component(0),
              DERIV=false,NAME="AX"));
    Array AY((TYPE="arrayField",ORDER=0,COMPONENT()=tmpCoord.component(1),
              DERIV=false,NAME="AY"));
    Array AR((TYPE="array",ORDER=0,COMPONENT()=bubbleRadius,
              NAME="bubbleRadius"));
    SPHERE=sqrt(AR*AR*pow(AX*AX+AY*AY,-1.0))*ACoord;
  }
  coord.setName("coordinates");
  
  Field coord2((DOM=Dom,INTERPOLATION=coordInterp,NBRCOMPONENTS=2,
                COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
                NAME="coord2")); 
  Array ACoord2((TYPE="arrayField",ORDER=1,DIM=2,DERIV=false,
                 COMPONENT(0)=coord2.component(0),
                 COMPONENT(1)=coord2.component(1),
                 NAME="ACoord2"));
  
  // Other Fields:
  
  Field pressure1((DOM=Dom1,INTERPOLATION="linear",NBRCOMPONENTS=1,
                   MINREF=1.0,
		   COMPONENTNAME(0)="p1",NAME="pressure1"));
  Field pressure2((DOM=Dom2,INTERPOLATION="linear",NBRCOMPONENTS=1,
                   MINREF=1.0,
		   COMPONENTNAME(0)="p2",NAME="pressure2"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  Field psi((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
	     COMPONENTNAME(0)="psi",NAME="psi"));
  Field tangent((DOM=Bnd7,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
	         COMPONENTNAME(0)="Vs",NAME="tangentVelo"));
      
  // Remeshing:
  
  Thompson2D Prob1((param("type","2D2D - coordinates - "),
		    param("domain",Dom),
		    param("coord",coord),
		    param("integRule","order 5"),
		    param("name","2D Thompson equations")));
  Prob1.setLayer(-100);
  
  kinCond1D Prob3((param("type","3D1D - Coord V - "),
		   param("domain",Bnd7),
		   param("coord",coord),
		   param("velo",velocities),
		   param("axisym",true),
		   param("surfaceTension",capillarity),
                   param("fixEnd",true),
		   param("coupledNormals",true),
		   param("integRule","order 5"),
		   param("name","All for free surface") ));
  
  // Various quantities from Prob1 and Prob3:
  
  Array X=Prob1.X;
  Array Y=Prob1.Y;
  Array testX=Prob1.testX;
  Array testY=Prob1.testY;
  Array refX=Prob1.refX;
  Array refY=Prob1.refY;
  
  Array ACoordXY=Prob1.coordinates;
  Array ATestCoordXY=Prob1.testCoordinates;
  TensorCoordSys TCSXY=Prob1.tcs;
  
  GlobLoc GL0D((COORD=ACoordXY,GLTYPE="0D"));
  GlobLoc GL1D=Prob3.gl;
  GlobLoc GL2D=Prob1.gl;
  TensorCoordSys TCS=Prob3.tcs;
  
  // Boundary conditions for Thompson remeshing:
  
  Constr Prob1BC1((NAME="Thompson lower boundary",TYPE="usual",DOM=lowerBnd,
		   TEST=testY,EQUATION=Y-refY,GLOBLOC=GL1D));
  Constr Prob1BC2((NAME="Thompson right boundary",TYPE="usual",DOM=Bnd3,
		   TEST=testX,EQUATION=X-refX,GLOBLOC=GL1D));
  Constr Prob1BC3((NAME="Thompson upper boundary",TYPE="usual",DOM=Bnd4,
		   TEST=testY,EQUATION=Y-refY,GLOBLOC=GL1D));
  Constr Prob1BC4((NAME="Thompson left boundary",TYPE="usual",DOM=leftBnd,
		   TEST=testX,EQUATION=X-refX,GLOBLOC=GL1D));
		   
  Constr Prob1BC31((NAME="Spherical free surface",TYPE="usual",DOM=Bnd7,
		    TEST=ATestCoordXY,EQUATION=ACoordXY-SPHERE,GLOBLOC=GL1D));
  Constr Prob1BC41((NAME="Spherical coordinates",TYPE="usual",DOM=Dom,
		    TEST=ATestCoordXY,EQUATION=ACoordXY-ACoord2,GLOBLOC=GL2D));
  Constr Prob1BC51((NAME="Spherical coordinates",TYPE="usual",DOM=extBnd,
		    TEST=ATestCoordXY,EQUATION=ACoordXY-ACoord2,GLOBLOC=GL1D));
		    
  Constr Prob1BC21((NAME="Thompson Dom 1",TYPE="usual",DOM=Dom,
		    TEST=testX,EQUATION=X-refX,GLOBLOC=GL2D));
  Constr Prob1BC22((NAME="Thompson Dom 1",TYPE="usual",DOM=Dom,
		    TEST=testY,EQUATION=Y-refY,GLOBLOC=GL2D));
    
  // Navier-Stokes equations:
  
  NavierStokes Prob2((param("type","3D2D - V P - F GPn"),
		        param("domain",Dom1),
		        param("coord",coord),
		        param("coordMobil",true),
		        param("velo",velocities),
		        param("pressure",pressure1),
		        param("axisym",true),
		        param("integRule","order 5"),
		        param("name","2D Stokes equations Dom1"),
		        param("rho",0.0),
		        param("viscosity",visc1) ));
  
  NavierStokes Prob5((param("type","3D2D - V P - F GPn"),
		        param("domain",Dom2),
		        param("coord",coord),
		        param("coordMobil",true),
		        param("velo",velocities),
		        param("pressure",pressure2),
		        param("axisym",true),
		        param("integRule","order 5"),
		        param("name","2D Stokes equations Dom2"),
		        param("rho",0.0),
		        param("viscosity",visc2) ));
  
  TensorCoordSys TCS2=Prob2.tcs;
  GlobLoc GL3D2D=Prob2.gl;
  
  // Boundary conditions for NS equations:
    
  Array Avelo=Prob2.velocities;
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
  
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=Avelo.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=Avelo.component(1)));
  Array testU((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=testMvt.component(0)));
  Array testV((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=testMvt.component(1)));

  Constr Prob2BC1((NAME="U=0 on left boundary",TYPE="usual",DOM=leftBnd,
		   TEST=testU,EQUATION=U,GLOBLOC=GL1D));
  Constr Prob2BC2((NAME="V=0 on lower boundary",TYPE="usual",DOM=lowerBnd,
		   TEST=testV,EQUATION=V,GLOBLOC=GL1D));
  Constr Prob2BC3((NAME="U extensional Flow",TYPE="usual",DOM=extBnd,
		   TEST=testU,EQUATION=U+(0.5*strainRate)*X,GLOBLOC=GL1D));
  Constr Prob2BC4((NAME="V extensional Flow",TYPE="usual",DOM=extBnd,
		   TEST=testV,EQUATION=V-strainRate*Y,GLOBLOC=GL1D));
  Constr Prob2BC101((NAME="U=0 on Corners",TYPE="usual",DOM=Corners2,
		   TEST=testU,EQUATION=U,GLOBLOC=GL0D));
  Constr Prob2BC102((NAME="V=0 on Corners",TYPE="usual",DOM=Corners2,
		   TEST=testV,EQUATION=V,GLOBLOC=GL0D));
  
  Array Atangent((TYPE="arrayField",ORDER=0,NAME="Atangent",
		  TENSORCOORDSYS=TCS2,
		  COMPONENT()=tangent.component(0)));
  Array tmp1=-1.0*X*Atangent;
  Array tmp2=Y*Atangent;
  Array AtangentVelo((TYPE="arrayComp",ORDER=1,DIM=3,NAME="vecteur velo tangent to the circle",
		      COMPONENT(0)=tmp2.component(),COMPONENT(1)=tmp1.component(),
		      TENSORCOORDSYS=TCS2));
  Constr Prob2BC11((NAME="Vn=0 on circle",TYPE="usual",DOM=Bnd7,TEST=testMvt,
		    EQUATION=Avelo-AtangentVelo,GLOBLOC=GL1D));
  
  Array Apressure1=Prob2.pressure;
  Array Apressure2=Prob5.pressure;
  Array testDiv1((TYPE="arrayTest",TEST=Apressure1,
		 NAME="test for continuity equations"));
  Array testDiv2((TYPE="arrayTest",TEST=Apressure2,
		  NAME="test for continuity equations"));
  Array A100=Array((TYPE="array",ORDER=0,COMPONENT()=100.0,NAME="A100"));
  Constr Prob5BC1((TYPE="usual",NAME="P=0 on a corner",DOM=cornerUR,
		   TEST=testDiv2,EQUATION=Apressure2-A100,GLOBLOC=GL0D,
		   TENSORCOORDSYS=TCS));
  Constr Prob2BC12((TYPE="usual",NAME="P=0 on a corner",DOM=attachPoint,
		    TEST=testDiv1,EQUATION=Apressure1,GLOBLOC=GL0D,
		    TENSORCOORDSYS=TCS));

  // Volume equation :
  
  Integ INTG((GLOBLOC=GL3D2D,TENSORCOORDSYS=TCS,RULE=3));
  Field attachPos((DOM=Dom,INTERPOLATION="constantOnDomain",NBRCOMPONENTS=1,
                   MINREF=1.0,
	           COMPONENTNAME(0)="attachPos",NAME="attachPos"));
  Array AattachPos((TYPE="arrayField",ORDER=0,NAME="AattachPos",DERIV=true,
                    TENSORCOORDSYS=TCS,COMPONENT()=attachPos.component(0)));
  Array AVolume((TYPE="array",ORDER=0,COMPONENT()=bubbleVolume,
		 NAME="imposed bubble volume"));
  Array testAttachPos((TYPE="arrayTest",TEST=AattachPos,
		       NAME="Test for attachment position equation"));  
  Array Prob6=integrate(testAttachPos,Dom1,INTG)-testAttachPos*AVolume;
  Prob6.setName("equation for the attachment point");
  
  Array testDisplX=Prob1.testX;
  Array displX=Prob1.X;
  Constr Prob1BC5((NAME="Free surface attachement BC",TYPE="usual",DOM=attachPoint,
		   TEST=testDisplX,EQUATION=displX-AattachPos,GLOBLOC=GL0D));
  
  // Equations for stream function:

  streamFunction2D Prob4((param("type","3D2D - psi - V"),
			  param("axisym",true),
		          param("psi",psi),
		          param("coord",coord),
		          param("velo",velocities),
		          param("domain",Dom),
		          param("integRule","order 5"),
		          param("name","stream function equations") ));
  
  // Boundary conditions for stream function:
  
  Array Astream=Prob4.psi;
  Array testStream=test(Astream);
  Constr Prob4BC1((NAME="Psi=0 on boundary 1",TYPE="usual",DOM=intBnd,
		   TEST=testStream,EQUATION=Astream,GLOBLOC=GL1D,
		   TENSORCOORDSYS=TCSXY));
  Constr Prob4BC2((NAME="Psi=0 on boundary 1",TYPE="usual",DOM=Bnd7,
		   TEST=testStream,EQUATION=Astream,GLOBLOC=GL1D,
		   TENSORCOORDSYS=TCSXY));
  		      
  // Solvers:		   
		   
  Solver initMesh((PROBLEM=Prob1,
                   CONSTR=Prob1BC1,CONSTR=Prob1BC2,CONSTR=Prob1BC3,
		   CONSTR=Prob1BC4,CONSTR=Prob1BC31,
		   CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=10));
  Solver initFlow((PROBLEM=Prob1,
                   CONSTR=Prob1BC41,
		   PROBLEM=Prob2,
		   CONSTR=Prob2BC1,CONSTR=Prob2BC2,CONSTR=Prob2BC3,
		   CONSTR=Prob2BC4,CONSTR=Prob2BC11,CONSTR=Prob2BC12,
		   PROBLEM=Prob5,
		   CONSTR=Prob5BC1,
		   CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=10));
  Solver flow((PROBLEM=Prob1,
	       CONSTR=Prob1BC1,CONSTR=Prob1BC2,CONSTR=Prob1BC3,
	       CONSTR=Prob1BC4,CONSTR=Prob1BC5,
	       CONSTR=Prob1BC51,
	       PROBLEM=Prob2,
	       CONSTR=Prob2BC1,CONSTR=Prob2BC2,CONSTR=Prob2BC3,CONSTR=Prob2BC4,
	       PROBLEM=Prob3,
	       PROBLEM=Prob5,
	       CONSTR=Prob5BC1,
	       PROBLEM=Prob6,
	       CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=6));
//flow.setZero(1.0e-60);
  Solver STREAM((PROBLEM=Prob4,CONSTR=Prob4BC1,CONSTR=Prob4BC2,
		 CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
  
  Plot output1((DOM=Dom1,COORD=ACoordXY,
                RESULT=Avelo,RESULT=Apressure1,RESULT=Astream));
  Plot output2((DOM=Dom2,COORD=ACoordXY,
                RESULT=Avelo,RESULT=Apressure2,RESULT=Astream));
		
  Result Rvelo((NAME="velocities",ARRAY=Avelo,DOM=Dom,
                INTERPOLATION="linear"));  
  Result Rpressure1((NAME="pressure1",ARRAY=Apressure1,DOM=Dom1,
                     INTERPOLATION="linear"));       
  Result Rpressure2((NAME="pressure2",ARRAY=Apressure2,DOM=Dom2,
                     INTERPOLATION="linear"));       
  Result Rstream((NAME="stream",ARRAY=Astream,DOM=Dom,
                  INTERPOLATION="linear")); 
  Plot outputDX((DOM=Dom,COORD=ACoordXY,TYPE="openDX",
                 RESULT=Rvelo,RESULT=Rpressure1,
	         RESULT=Rpressure2,RESULT=Rstream));
  Plot outputGMSH((DOM=Dom,COORD=ACoordXY,TYPE="gmsh",
                   RESULT=Rvelo,RESULT=Rpressure1,
	           RESULT=Rpressure2,RESULT=Rstream));

  // Resolution:
  
  initMesh.NewtonRaphson();
  coord2.interpolate(ACoordXY,Dom,ACoordXY); 
    
  initFlow.NewtonRaphson();
  
//  flow.NewtonRaphson();
  STREAM.NewtonRaphson();
  
  output1.exec("bubble_dom1");
  output2.exec("bubble_dom2");
  outputDX.exec("bubble");
  outputGMSH.exec("bubble");
  
  string filename;
  Double factor=1.5;
  int i=1;
  while (factor>1.001) {
    cerr  << "step " << i << ": " 
          << "NEW VALUE OF SURFACE TENSION: " << capillarity << endl ;
    Prob3.surfaceTension.component()=capillarity;
    flow.NewtonRaphson();
    
    if (flow.getLastState()==1) {
      STREAM.NewtonRaphson();
    
      sprintf(buf,"%d",i);
      filename="bubble_dom1_";
      filename+=buf;
      output1.exec(filename);
    
      sprintf(buf,"%d",i);
      filename="bubble_dom2_";
      filename+=buf;
      output2.exec(filename);
      
      sprintf(buf,"%d",i);
      filename="bubble_";
      filename+=buf;
      outputDX.exec(filename);
      
      sprintf(buf,"%d",i);
      filename="bubble_";
      filename+=buf;
      outputGMSH.exec(filename);
      
      capillarity/=factor;
      i++;
    }
    else {
      capillarity*=factor;
      factor=sqrt(factor);
      capillarity/=factor;
    }
  }
  
  

}
  
  
  
  
  
  
  
  
