#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "NavierStokes.h"
#include "Thompson2D.h"
#include "kinCondXY.h"
#include "streamFunction2D.h"

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Filter.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;
using namespace keyFlow ;



int main(void) {
  Double viscosity=100.0;
  
  // Reading and renumbering of the mesh:
  
  Mesh MESH;
  MESH.read("refined.msh","gmsh2dMesh");
  string sortType="frontal";
  if (sortType=="position")
  {
    Domain Dom=MESH.getDefDomain();
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
		  DERIV=false,COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",
                 COMPONENT(0)=Double(1.0e-4),COMPONENT(1)=Double(1.0)));
    Array ADist=accumulation(ACoord,ABrol,1);
    MESH=renumberDefDomainArray(MESH,ADist);
  }
  else if (sortType=="frontal") {
    Domain Bnd=MESH.getDomainFromIntId(18);
    MESH=renumberDefDomainFrontal(MESH,Bnd);
  }
  else if (sortType=="none") {
    // Nothing to do!
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }

  // Domains:

  Domain Dom,Dom12,Dom13,Bnd14,Bnd15,Bnd16,Bnd17,Bnd18,Bnd19,Bnd20;
  Dom=MESH.getDefDomain();
  Dom.setName("main domain");
  Dom12=MESH.getDomainFromIntId(12);
  Dom12.setName("Dom12");
  Dom13=MESH.getDomainFromIntId(13);
  Dom13.setName("Dom13");
  Bnd14=MESH.getDomainFromIntId(14);
  Bnd14.setName("Bnd14");
  Bnd15=MESH.getDomainFromIntId(15);
  Bnd15.setName("Bnd15");
  Bnd16=MESH.getDomainFromIntId(16);
  Bnd16.setName("Bnd16");
  Bnd17=MESH.getDomainFromIntId(17);
  Bnd17.setName("Bnd17");
  Bnd18=MESH.getDomainFromIntId(18);
  Bnd18.setName("Bnd18");
  Bnd19=MESH.getDomainFromIntId(19);
  Bnd19.setName("Bnd19");
  Bnd20=MESH.getDomainFromIntId(20);
  Bnd20.setName("Bnd20");
  
  // Coordinate systems:
  
  Field coord;
  {
    Field tmpCoord=MESH.getCoordinateField(Dom);
    coord=Field((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
                 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
                 NAME="coordinates")); 
    coord.interpolate(tmpCoord,Dom,tmpCoord);
  }
  
  // Other Fields:
  
  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		  COMPONENTNAME(0)="p",NAME="pressure"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  Field psi((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
	     COMPONENTNAME(0)="psi",NAME="psi"));
      
  // Remeshing:
  
  Thompson2D Prob1((param("type","2D2D - coordinates - "),
		    param("domain",Dom13),
		    param("coord",coord),
		    param("integRule","order 5"),
		    param("name","2D Thompson equations")));
  Prob1.setLayer(-100);
  
  kinCondXY Prob2((param("type","3D1D - Coord V - "),
		   param("domain",Bnd17),
		   param("coord",coord),
		   param("velo",velocities),
		   param("axisym",true),
		   param("surfaceTension",0.0),
		   param("integRule","order 5"),
		   param("name","All for free surface") ));

  // Various quantities from Prob1 and Prob3:
  
  Array X=Prob1.X;
  Array Y=Prob1.Y;
  Array testX=Prob1.testX;
  Array testY=Prob1.testY;
  Array refX=Prob1.refX;
  Array refY=Prob1.refY;
  
  Array ACoordXY=Prob1.coordinates;
  Array ACoordXYND=((TYPE="arrayField",ORDER=1,DIM=2,NAME="ACoordXYND",
  		     DERIV=false,COMPONENT(0)=coord.component(0),
  		     COMPONENT(1)=coord.component(1)));
  TensorCoordSys TCSXY=Prob1.tcs;
  
  GlobLoc GL2D((COORD=ACoordXYND,GLTYPE="2D2D"));
  GlobLoc GL1D((COORD=ACoordXYND,GLTYPE="2D1D"));
  
  // Boundary conditions for Thompson remshing:
  
  Constr Prob1BC13((NAME="Prob1BC13",TYPE="usual",DOM=Dom13,
		    TEST=testY,EQUATION=Y-refY,GLOBLOC=GL2D));
  Constr Prob1BC19((NAME="Prob1BC19",TYPE="usual",DOM=Bnd19,
		    TEST=testX,EQUATION=X-refX,GLOBLOC=GL1D));
  Constr Prob1BC15((NAME="Prob1BC15",TYPE="usual",DOM=Bnd15,
		    TEST=testX,EQUATION=X-refX,GLOBLOC=GL1D));
  Constr Prob1BC13b((NAME="Prob1BC13",TYPE="usual",DOM=Bnd19,
		    TEST=testY,EQUATION=Y-refY,GLOBLOC=GL1D));
  Constr Prob1BC13c((NAME="Prob1BC13",TYPE="usual",DOM=Bnd20,
		    TEST=testY,EQUATION=Y-refY,GLOBLOC=GL1D));
		    
  Constr Prob1BC17((NAME="Prob1BC17",TYPE="usual",DOM=Bnd17,
		    TEST=testX,EQUATION=X-refX,GLOBLOC=GL1D));
		    
  
  // Navier-Stokes equations:
  
  NavierStokes Prob3((param("type","3D2D - V P - F GPn"),
		      param("domain",Dom13),
		      param("coord",coord),
		      param("coordMobil",true),
		      param("velo",velocities),
		      param("pressure",pressure),
		      param("axisym",true),
		      param("integRule","order 5"),
		      param("name","2D Stokes equations Dom13"),
		      param("rho",0.0),
		      param("viscosity",viscosity) ));

  NavierStokes Prob4((param("type","3D2D - V P - F GPn"),
		      param("domain",Dom12),
		      param("coord",coord),
		      param("velo",velocities),
		      param("pressure",pressure),
		      param("axisym",true),
		      param("integRule","order 5"),
		      param("name","2D Stokes equations Dom12"),
		      param("rho",0.0),
		      param("viscosity",viscosity) ));
		      
  Field gpn((DOM=Bnd18,INTERPOLATION="constantOnDomain",NBRCOMPONENTS=1,
             COMPONENTNAME(0)="gpn",NAME="normal gradient field"));
  Array Q((TYPE="array",ORDER=0,COMPONENT()=-1.0,NAME="Q"));
  NavierStokes Prob5((param("type","3D1D - V P GPn - F Q"),
	              param("domain",Bnd18),
	              param("coord",coord),
	              param("velo",velocities),
	              param("pressure",pressure),
	              param("gradPn",gpn),
	              param("flowRate",Q),
		      param("axisym",true),
	              param("integRule","order 5"),
	              param("name","NS Inflow Bnd18"),
	              param("viscosity",viscosity),
	              param("rho",0.0) ));
  Prob5.setLayer(100);
  
  // Boundary conditions for NS equations:
    
  Array Avelo=Prob2.velocities;
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));    
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=Avelo.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=Avelo.component(1)));
  Array testU((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=testMvt.component(0)));
  Array testV((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=testMvt.component(1)));

  Constr Prob4BC14((NAME="U=0 on Bnd14",TYPE="usual",DOM=Bnd14,
		    TEST=testU,EQUATION=U,GLOBLOC=GL1D));
  Constr Prob3BC15u((NAME="U=0 on Bnd15",TYPE="usual",DOM=Bnd15,
		     TEST=testU,EQUATION=U,GLOBLOC=GL1D));
  Constr Prob4BC16u((NAME="U=0 on Bnd16",TYPE="usual",DOM=Bnd16,
		     TEST=testU,EQUATION=U,GLOBLOC=GL1D));
  Constr Prob4BC16v((NAME="V=0 on Bnd16",TYPE="usual",DOM=Bnd16,
		     TEST=testV,EQUATION=V,GLOBLOC=GL1D));
  Constr Prob3BC17u((NAME="U=0 on Bnd17",TYPE="usual",DOM=Bnd17,
		     TEST=testU,EQUATION=U,GLOBLOC=GL1D));
		     
  Array VUp((TYPE="array",ORDER=0,NAME="VUp",COMPONENT()=0.5));
  Constr Prob3BC20v((NAME="Pull-Up",TYPE="usual",DOM=Bnd20,
		     TEST=testV,EQUATION=V-VUp,GLOBLOC=GL1D));  
  
  // Equations for stream function:
  
  streamFunction2D Prob6((param("type","3D2D - psi - V"),
			  param("axisym",true),
		          param("psi",psi),
		          param("coord",coord),
		          param("velo",velocities),
		          param("domain",Dom),
		          param("integRule","order 5"),
		          param("name","stream function equations") ));
  
  // Boundary conditions for stream function:
  
  Array Astream=Prob6.psi;
  Array testStream=test(Astream);
  Field streamBC((DOM=Dom,INTERPOLATION="constantOnDomain",
		  NBRCOMPONENTS=1,COMPONENTNAME(0)="psi",
		  NAME="value of stream function for BC"));
  Array AstreamBC((TYPE="arrayField",ORDER=0,NAME="streamBC",
		   TENSORCOORDSYS=TCSXY,COMPONENT()=streamBC.component(0)));  
  Array APsiZero((TYPE="array",ORDER=0,NAME="psi = 0",TENSORCOORDSYS=TCSXY));
  
  Constr Prob6BC14((NAME="Psi=0 on Bnd14",TYPE="usual",DOM=Bnd14,
		    TEST=testStream,EQUATION=Astream,GLOBLOC=GL1D,
		    TENSORCOORDSYS=TCSXY));
  Constr Prob6BC15((NAME="Psi=0 on Bnd15",TYPE="usual",DOM=Bnd15,
		    TEST=testStream,EQUATION=Astream,GLOBLOC=GL1D,
		    TENSORCOORDSYS=TCSXY));
  Constr Prob6BC16((NAME="Psi=0 on on Bnd16",TYPE="usual",DOM=Bnd16,
		    TEST=testStream,EQUATION=Astream-AstreamBC,
		    GLOBLOC=GL1D,TENSORCOORDSYS=TCSXY));
  Constr Prob6BC17((NAME="Psi=0 on on Bnd17",TYPE="usual",DOM=Bnd17,
		    TEST=testStream,EQUATION=Astream-AstreamBC,
		    GLOBLOC=GL1D,TENSORCOORDSYS=TCSXY));
  
  // Resolution:
  
  Solver FLOWinit((PROBLEM=Prob1,
                   CONSTR=Prob1BC13,CONSTR=Prob1BC17,
	           CONSTR=Prob1BC19,CONSTR=Prob1BC15,
	           PROBLEM=Prob3,PROBLEM=Prob4,PROBLEM=Prob5,
	           CONSTR=Prob3BC17u,
		   CONSTR=Prob4BC14,CONSTR=Prob3BC15u,CONSTR=Prob4BC16u,
	           CONSTR=Prob4BC16v,
  	           CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
  Solver FLOW((PROBLEM=Prob1,
               CONSTR=Prob1BC13,
	       CONSTR=Prob1BC19,CONSTR=Prob1BC15,
	       PROBLEM=Prob2,
	       PROBLEM=Prob3,PROBLEM=Prob4,PROBLEM=Prob5,
	       CONSTR=Prob4BC14,CONSTR=Prob3BC15u,CONSTR=Prob4BC16u,
	       CONSTR=Prob4BC16v,CONSTR=Prob3BC20v,
  	       CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=10));
  Solver STREAM((PROBLEM=Prob6,
                 CONSTR=Prob6BC14,CONSTR=Prob6BC15,
                 CONSTR=Prob6BC16,CONSTR=Prob6BC17,
		 CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
  
  Plot output((DOM=Dom,COORD=ACoordXY,RESULT=Avelo,RESULT=Prob3.pressure,
	       RESULT=Astream,TYPE="gmsh"));
  
  // Initial solution:
  
  FLOWinit.NewtonRaphson();

  double pullV=0.25;
  char buf[256];
  string filename;
  for (int i=0;i<20;i++) {
    VUp.component()=pullV;
    
    cerr << "pullV(" << i << ")=" << pullV << endl ;
  
    sprintf(buf,"%d",i);
    filename="dieSwell_";
    filename+=buf;
    
    FLOW.NewtonRaphson();
    STREAM.NewtonRaphson();
  
    output.exec(filename);
    
    pullV*=1.5;
  }
}
  
  
  
  
  
  
  
  
