
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;



int main(void) {
  char buf[256];
  int i;
  
  // Lecture et renumrotation du maillage :

  Mesh MESH;
  MESH.read("square.msh","rsMesh");
  string sortType="frontal";
  if (sortType=="position")
  {
    Domain Dom=MESH.getDomain(0);
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ de coordonnes",
		  DERIV=false,COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",COMPONENT(0)=1.0,
		 COMPONENT(1)=1.0e-4));
    Array ADist=accumulation(ACoord,ABrol,1);
    MESH=renumberDefDomainArray(MESH,ADist);
  }
  else if (sortType=="frontal") {
    Domain Bnd1=MESH.getDomain(1);
    MESH=renumberDefDomainFrontal(MESH,-Bnd1);
  }
  else if (sortType=="none") {
    // Rien  faire !
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }
  MESH.write("square.msh2","rsMesh");
  
  // Diffrents domaines :
  
  Domain Dom,Bnd,Bnd1,Bnd2,Bnd3,Bnd4,Corners1,Corners2,Corner;
  Dom=MESH.getDomain(0);
  Dom.setName("main domain");
  Bnd1=MESH.getDomain(1);
  Bnd1.setName("left boundary");
  Bnd2=MESH.getDomain(2);
  Bnd2.setName("lower boundary");
  Bnd3=MESH.getDomain(3);
  Bnd3.setName("right boundary");
  Bnd4=MESH.getDomain(4);
  Bnd4.setName("upper boundary");
  Corners1=Bnd1.getBoundary();
  Corners2=Bnd4.getBoundary();
  Corner=Corners1*Corners2;
  Bnd=Bnd1+Bnd2+Bnd3+Bnd4;

  // Tout ce qui concerne les coordonnes et intgrations :
  
  Field coord=MESH.getCoordinateField(Dom);
  coord.setName("coordinates");

  Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ de coordonnes",
		DERIV=false,COMPONENT(0)=coord.component(0),
		COMPONENT(1)=coord.component(1)));
  
  TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
  
  GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
  GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
  GlobLoc GL0((COORD=ACoord,GLTYPE="0D"));
  
  Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
  Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));  
  
  GlobLoc::setDefaultGL(GL);

  // Tout ce qui concerne les quations de Navier-Stokes et d'nergie :

  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		  COMPONENTNAME(0)="p",NAME="pressure"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
                    NAME="velocities"));
  Field temperature((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		     COMPONENTNAME(0)="T",NAME="temperature"));
  Array Avelo((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",
	       TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	       COMPONENT(1)=velocities.component(1)));
  Array Apressure((TYPE="arrayField",ORDER=0,NAME="pression",
		   TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
  Array Atemp((TYPE="arrayField",ORDER=0,NAME="temprature",
	       TENSORCOORDSYS=TCS,
	       COMPONENT()=temperature.component(0)));
  
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
  Array testDiv((TYPE="arrayTest",TEST=Apressure,
		 NAME="Equations de continuit"));
  Array testNRG((TYPE="arrayTest",TEST=Atemp,NAME="Equations d'nergie"));
  
  Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=1.0,
	      COMPONENT(1,1)=1.0,NAME="Unit tensor",TENSORCOORDSYS=TCS));
  Array rho_0((TYPE="array",ORDER=0,NAME="density",TENSORCOORDSYS=TCS,
	       COMPONENT()=1000.0));
  Array rho=rho_0-0.001*rho_0*Atemp;
  
  Array ForceY=-9.81*rho;  
  Array Force((TYPE="arrayComp",ORDER=1,DIM=2,NAME="force volumique",
	       COMPONENT(1)=ForceY.component()));
  
  // Navier-Stokes equations :

  Array gradVelo=grad(Avelo);
  gradVelo.setName("Champ gradients de vitesses");
  Array Convect=rho*contractedMultiplication(Avelo,0,gradVelo,1);
  Convect.setName("Termes convectifs");
  Array gradMvt=grad(testMvt);
  gradMvt.setName("gradMvt");
  
  Array divV=contract(gradVelo);
  divV.setName("divergence des vitesses");
  
  Array D=0.5*(gradVelo+transpose(gradVelo));
  D.setName("Taux de dformation");
  Array T=D-Apressure*Unit; // viscosity = 0.5
  T.setName("Tenseur des tensions de Cauchy");
  Array NavierStokes=accumulation(gradMvt,T)+accumulation(testMvt,Convect)
    +testDiv*divV-accumulation(testMvt,Force);
  NavierStokes.setName("Equations de Navier-Stokes");
  
  Array Prob1=integrate(NavierStokes,Dom,INTG);
  
  // Vitesse nulle sur les parois :
  
  Array AveloZero((TYPE="array",ORDER=1,DIM=2,NAME="vitesses_inflow",
		   TENSORCOORDSYS=TCS));
  Constr Prob1CF1((NAME="V=0 on boundary",TYPE="usual",DOM=Bnd,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
   
  Array ApressureZero((TYPE="array",ORDER=0,NAME="pressureZero",
		       TENSORCOORDSYS=TCS));
  Constr Prob1CF2((NAME="P=0 on a corner",TYPE="usual",DOM=Corner,
		   TEST=testDiv,EQUATION=Apressure-ApressureZero,GLOBLOC=GL0,
		   TENSORCOORDSYS=TCS));

  // Equation d'nergie et conditions frontires :
  
  Array gradTemp=grad(Atemp);
  gradTemp.setName("gradient de temprature");
  Array gradNRG=grad(testNRG);
  gradNRG.setName("gradNRG");
  Array Energie=0.020*accumulation(gradNRG,gradTemp)
    +testNRG*accumulation(Avelo,gradTemp);
  Array Prob3=integrate(Energie,Dom,INTG);
  Array AtempLeft((TYPE="array",ORDER=0,NAME="T left",TENSORCOORDSYS=TCS,
		   COMPONENT()=1.0));
  Array AtempRight((TYPE="array",ORDER=0,NAME="T right",TENSORCOORDSYS=TCS,
		    COMPONENT()=0.0));
  Constr Prob3CF1((NAME="temp on left boundary",TYPE="usual",DOM=Bnd1,
		   TEST=testNRG,EQUATION=Atemp-AtempLeft,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob3CF2((NAME="temp on right boundary",TYPE="usual",DOM=Bnd3,
		   TEST=testNRG,EQUATION=Atemp-AtempRight,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Array Prob4=integrate(NavierStokes+Energie,Dom,INTG);
  
  // Calcul de la fonction de Courant :

  Field courant((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		 COMPONENTNAME(0)="psi",NAME="courant"));
  Array Acourant((TYPE="arrayField",ORDER=0,NAME="courant",TENSORCOORDSYS=TCS,
		  COMPONENT()=courant.component(0)));
  Array Avelo2((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",DERIV=false,
	        TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	        COMPONENT(1)=velocities.component(1)));
     // (meme chose que Avelo, mais on ne drive pas par rapport aux valeurs
     //  nodales.)
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",
	   COMPONENT()=Avelo2.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",
	   COMPONENT()=Avelo2.component(1)));
  Array mV=-1.0*V;
  Array tmpVelo((TYPE="arrayComp",ORDER=1,DIM=2,NAME="special",
	         TENSORCOORDSYS=TCS,COMPONENT(0)=mV.component(),
	         COMPONENT(1)=U.component()));
  Array gradPsi=grad(Acourant);
  Array testCourant((TYPE="arrayTest",TEST=Acourant,
		     NAME="Equations de courant"));  
  Array gradCourant=grad(testCourant);
  Array eqCourant=gradPsi-tmpVelo;
  Array equationCourant=accumulation(gradCourant,eqCourant);
  Array Prob2=integrate(equationCourant,Dom,INTG);

  // Courant boundary conditions :
  
  Array APsiZero((TYPE="array",ORDER=0,NAME="psi = 0",TENSORCOORDSYS=TCS));
  Constr Prob2CF1((NAME="Psi=0 on boundary",TYPE="usual",DOM=Bnd,
		   TEST=testCourant,EQUATION=Acourant,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
    
  // Rsolution :
  
  for (i=0;i<=10;i++) {
    double Tmax=10*i;
    AtempLeft.component()=Tmax;
    
    cerr << "New continuation step with Tmax = " << Tmax << endl ;
    
    Solver NS((PROBLEM=Prob4,CONSTR=Prob1CF1,
               CONSTR=Prob3CF1,CONSTR=Prob3CF2,
	       DELAY=5,AUTOFIX=100.0,
               CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=6));
//    Solver NS((PROBLEM=Prob4,CONSTR=Prob1CF1,CONSTR=Prob1CF2,
//               CONSTR=Prob3CF1,CONSTR=Prob3CF2,
//               CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=6));
//    Solver NS((PROBLEM=Prob1,CONSTR=Prob1CF1,CONSTR=Prob1CF2,
//               PROBLEM=Prob3,CONSTR=Prob3CF1,CONSTR=Prob3CF2,
//               CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=6));
    NS.NewtonRaphson();
    
    Solver STREAM((PROBLEM=Prob2,CONSTR=Prob2CF1,CONVERGENCE=1.0e-4,
      DIVERGENCE=1.0e4,NITMAX=1));
    STREAM.NewtonRaphson();
    
    Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure,
		 RESULT=Acourant,RESULT=Atemp));
    
    sprintf(buf,"%d",i);
    string filename="square";
    filename+=buf;
    output.exec(filename);
  }
}


