
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <math.h>
#include <stdio.h>

#include "BirdCarreau.h"

using namespace std ;
using namespace keyFlow ;



BirdCarreau::BirdCarreau(Double argEta0,Double argLambda0,
		         Double argExpo) : function1() {
  eta0=argEta0;
  lambda0=argLambda0;
  expo=argExpo;
  
  orderDeriv=0;
}



BirdCarreau::BirdCarreau(const BirdCarreau &in) : function1() {
  eta0=in.eta0;
  lambda0=in.lambda0;
  expo=in.expo;
  
  orderDeriv=in.orderDeriv;
}



BirdCarreau::~BirdCarreau(void) {
}



Double BirdCarreau::value(Double arg) const {
  Double tmp1=1.0+lambda0*lambda0*arg*arg;
  Double lambdaSquare=lambda0*lambda0;
  
  if (orderDeriv==0)
    return(eta0*pow(tmp1,0.5*(expo-1.0)));
  else if (orderDeriv==1)
    return(eta0*(expo-1.0)*lambdaSquare*pow(tmp1,0.5*(expo-3.0))*arg);
  else if (orderDeriv==2)
    return(eta0*(expo-1.0)*lambdaSquare*pow(tmp1,0.5*(expo-3.0))
	   +eta0*(expo-1.0)*(expo-3.0)*lambdaSquare*lambdaSquare
	     *pow(tmp1,0.5*(expo-5.0))*arg*arg);
  else
    return(0.0);
}



Double BirdCarreau::deriv(Double arg) const {
  Double tmp1=1.0+lambda0*lambda0*arg*arg;
  Double lambdaSquare=lambda0*lambda0;
  
  if (orderDeriv==0)
    return(eta0*(expo-1.0)*lambdaSquare*pow(tmp1,0.5*(expo-3.0))*arg);
  else if (orderDeriv==1)
    return(eta0*(expo-1.0)*lambdaSquare*pow(tmp1,0.5*(expo-3.0))
	   +eta0*(expo-1.0)*(expo-3.0)*lambdaSquare*lambdaSquare
	     *pow(tmp1,0.5*(expo-5.0))*arg*arg);
  else
    return(0.0);
}



void BirdCarreau::setArgEta0(Double x) {
  eta0=x;
}



Double BirdCarreau::getArgEta0(void) const {
  return(eta0);
}



void BirdCarreau::setArgLambda0(Double x) {
  lambda0=x;
}



Double BirdCarreau::getArgLambda0(void) const {
  return(lambda0);
}



void BirdCarreau::setArgExpo(Double x) {
  expo=x;
}



Double BirdCarreau::getArgExpo(void) const {
  return(expo);
}
      
      
      
function1* BirdCarreau::makeCopy(void) const {
   return(new BirdCarreau(*this));
}



function1* BirdCarreau::makeFunctionDeriv(void) const {
  BirdCarreau *tmp=new BirdCarreau(*this);
  tmp->orderDeriv++;
  return(tmp);
}



string BirdCarreau::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;

  sprintf(cbuf,"BirdCarreau");
  sbuf=cbuf;
  for (int i=0;i<orderDeriv;i++) 
    sbuf+="'";
  sprintf(cbuf,"(eta0=%g,lambda0=%g,expo=%g,arrayArg)",
	  eta0,lambda0,expo);
  sbuf+=cbuf;
  
  return sbuf;
}  
  
