#ifndef ARRAY_COMP_H
#define ARRAY_COMP_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>
#include <string>

#include "array_base.h"

namespace keyFE2 {
  
  class Array ;
  class field ;
  class globLoc ;
  class object ;
  class printTree ;
  class timeDep ;
  
  class arrayComp : public virtual array {
    
    friend class Array ;
    friend class arrayOp1 ;
    friend class arrayOp2 ;
    friend class arrayOpN ;
    
    protected :
      
      globLoc *GL ;
      
      array **arrayList ;
      Int *compCorresp ;
    
      array **dependence ;
      Int dependenceSize ;
      Int dependenceCapacity ;
    
    public : 
      
      arrayComp(void) ;
      arrayComp(array &in) ;
      arrayComp(arrayComp &in) ;
      ~arrayComp(void) ;
    
      array* makeCloneForArrayTime(timeDep &TD) ;
    
      void setOrder(Int nbr) ;
      void setDim(Int index,Int nbrComponents) ;
      
      void adaptToNbrVal(void) ;
      void makeDependence(void) ;
      
      void operator=(const Double &d) ;
      void operator=(array&) ;
      
      void makeAtPoint(object *element,Int direct,Double *localCoord,
		       bool makeValue=true,bool makeDeriv=true) ;
      
      array* makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
		       std::string derivType) const ;
      array* makeExplicitDeriv(Int nbrScal,scal **scalList) const ;
      array* makeCoordDeriv(globLoc *GL) const ;
      array* makeCorlocDeriv(Int nbrDP) const ;
      
      void setNewPoint(bool b) ;
      void accessNbrInit(void) ;
      
      void printValues(Int dec=0) const ;
      printTree getPrintTree(bool values) const ;
      
      std::set<field*>* buildActiveFieldList(void) ;
      std::set<parents*> getListOfType(std::string typeName) ;
      
      void setGlobLoc(const globLoc *gl ) ;
      globLoc* getGlobLoc(void) const ;
    
  } ;
  
}

#endif


