
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <stdio.h>

#include "arrayComp.h"
#include "arrayFunctionsBuild.h"
#include "arrayOp1.h"
#include "arrayOp2.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/tensorCoordSys_base.h"
#include "err/err.h"
#include "function/functions.h"
#include "print/printTree.h"
#include "scal/scal.h"
#include "timeDep/timeDep_base.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;



arrayOp1::arrayOp1(void) : array() {
  type=0;
  arrayArg=NULL;
  iParam1=-1;
  iParam2=-1;
  fParam=NULL;
  IListNbr=0;
  IList=NULL;
}



arrayOp1::arrayOp1(const array &Ain) : array(Ain) {
  type=0;
  arrayArg=NULL;
  iParam1=-1;
  iParam2=-1;
  fParam=NULL;
  IListNbr=0;
  IList=NULL;
}



arrayOp1::arrayOp1(const arrayOp1 &in) : array(in) {
  Int i;
  
  type=in.type;
  if (in.arrayArg!=NULL) {
    arrayArg=in.arrayArg;
    arrayArg->addParent(&arrayArg);
  }
  else
    arrayArg=NULL;
  iParam1=in.iParam1;
  iParam2=in.iParam2;
  fParam=NULL;
  if (in.fParam!=NULL)
    setFunction(in.fParam->makeCopy());
  if (in.IListNbr!=0) {
    IListNbr=in.IListNbr;
    if (IListNbr>0)
      IList=new Int[IListNbr];
    else
      IList=NULL;
    for (i=0;i<IListNbr;i++)
      IList[i]=in.IList[i];
  }
  else {
    IListNbr=0;
    IList=NULL;
  }
}



arrayOp1::~arrayOp1(void) {
  type=0;
  delinkAndDelete(arrayArg,&arrayArg);
  arrayArg=NULL;
  iParam1=-1;
  iParam2=-1;
  delinkAndDelete(fParam,&fParam);
  fParam=NULL;
  delete[] IList;
  IListNbr=0;
  IList=NULL;
}



array* arrayOp1::makeCloneForArrayTime(timeDep &TD) {
  try {
    arrayOp1 *newArray=new arrayOp1(*this);
    newArray->setArrayArg(arrayArg->makeCloneForArrayTime(TD));
    return(newArray);
  }
  catchAll("arrayOp1::makeCloneForArrayTime","")
}



void arrayOp1::setType(Int tp) {
  type=tp;
}



void arrayOp1::setType(string typeName) {
  if (typeName=="none")
    type=0;
  else if (typeName=="transposition")
    type=1;
  else if (typeName=="contraction")
    type=2;
  else if (typeName=="inversion")
    type=3;
  else if (typeName=="function")
    type=4;
  else if (typeName=="indicesPermutation")
    type=5;
  else if (typeName=="discretizedTransposition")
    type=6;
  else
    errThrow("arrayOp1::setType",
	     string("   unknown type name : ")+typeName+".");
}



Int arrayOp1::getType(void) const {
  return(type);
}



string arrayOp1::getTypeName(void) const {
  switch(type) {
  case 0 :
    return("none");
  case 1 :
    return("transposition");
  case 2 :
    return("contraction");
  case 3 :
    return("inversion");
  case 4 :
    return("function");
  case 5 :
    return("indicesPermutation");
  case 6 :
    return("discretizedTransposition");
  }
  return("unknown");
}



void arrayOp1::setArrayArg(array *arg) {
  try {
    delinkAndDelete(arrayArg,&arrayArg);
    arrayArg=arg;
    if (arrayArg!=NULL)
      arrayArg->addParent(&arrayArg);
  }
  catchAll("arrayOp1::setArrayArg","")
}


array* arrayOp1::getArrayArg(void) const {
  return(arrayArg);
}



void arrayOp1::setIParam1(Int nbr) {
  iParam1=nbr;
}



void arrayOp1::setIParam2(Int nbr) {
  iParam2=nbr;
}



Int arrayOp1::getIParam1(void) {
  return(iParam1);
}



Int arrayOp1::getIParam2(void) {
  return(iParam2);
}



void arrayOp1::setFunction(function1 *f) {
  try {
    delinkAndDelete(fParam,&fParam);
    fParam=f;
    if (fParam!=NULL)
      fParam->addParent(&fParam);
  }
  catchAll("arrayOp1::setFunction","")
}



function1* arrayOp1::getFunction(void) const {
  return(fParam);
}



void arrayOp1::setIListNbr(Int nbr) {
  Int i;
  
  delete[] IList;
  if (nbr>0)
    IList=new Int[nbr];
  else
    IList=NULL;
  for (i=0;i<nbr;i++)
    IList[i]=0;
  IListNbr=nbr;
}



void arrayOp1::setIList(Int index,Int value) const {
  if (index>order)
    errThrow("arrayOp1::setIList",
	     string("index too large (")+strFromInt(index)+").");
  if (value>order)
    errThrow("arrayOp1::setIList",
	     string("value too large (")+strFromInt(value)+").");
  
  IList[index]=value;
}



Int arrayOp1::getIList(Int index) const {
  if (index>order)
    errThrow("arrayOp1::getIList",
	     string("index too large (")+strFromInt(index)+").");
  
  return(IList[index]);
}



Int* arrayOp1::getIList(void) const {
  return(IList);
}



void arrayOp1::swapComponents(Int index1,Int index2) const {
  Int iTmp;
  
  if (getTypeName()!="indicesPermutation")
    errThrow("arrayOp1::swapComponents",
	     string("   invalid arrayOp1 type : ")+getTypeName()+".");
  if (index1>order)
    errThrow("arrayOp1::swapComponents",
	     string("first index too large (")+strFromInt(index1)+").");
  if (index2>order)
    errThrow("arrayOp1::swapComponents",
	     string("second index too large (")+strFromInt(index2)+").");
  
  iTmp=IList[index1];
  IList[index1]=IList[index2];
  IList[index2]=iTmp;
}



void arrayOp1::makeAtPoint(object *element,Int direct,Double *localCoord,
			   bool makeValue,bool makeDeriv) {
  
  if (!getNewPoint())
    return;
  newPoint::setNewPoint(false);
  
  try {
    
    arrayArg->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
    
    switch (type) {
      
      // transposition :
    case 1 :
      transpositionExec(arrayArg,iParam1,iParam2);
      break;
      
      // contraction :
    case 2 :
      contractionExec(arrayArg,iParam1,iParam2);
      break;
      
      // inversion :
    case 3 :
      inversionExec(arrayArg);
      break;
      
      // function : 
    case 4 : 
      functionExec(arrayArg,fParam);
      break;
      
      // componentsIndices :
    case 5 :
      indicesPermutationExec(arrayArg,IList);
      break;
      
      // transposition des composantes scalaires :
    case 6 :
      discretizedTranspositionExec(arrayArg);
      break;
      
      // Erreur unknown type :
    default :
      
      errThrow("arrayOp1::makeAtPoint",
	       string("   unknown type : ")+strFromInt(type)+".");
      
    } // switch(type)
    
    updateBoolZeroValue();  
  }
  catchAll("arrayOp1::makeAtPoint","")
}



array* arrayOp1::makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
			   string derivType) const {
  try {
    Int i,j;
    string typeName;
    array *newArray=NULL;
    
    if (!(derivType=="explicit"||derivType=="coord"||derivType=="corloc"))
      errThrow("arrayOp1::makeDeriv",
	       string("   unknown derivation type : ")+derivType+".");
    
    if (derivType=="coord") {
      if (GL!=NULL) {
	nbrScal=GL->getCoordComponentsNbr();
	scalList=GL->getCoordComponents();
      }
      else
	errThrow("arrayOp1::makeDeriv",
		 "   no coord derivative can be calculated if GL==NULL");
    }
    
    typeName=getTypeName();
    
    if (typeName=="transposition") {
      array *arg=arrayArg->makeDeriv(nbrScal,scalList,GL,derivType);
      newArray=transpositionBuild(*arg,iParam1,iParam2);
    } // if (typeName=="transposition")
    else if (typeName=="contraction") {
      array *arg=arrayArg->makeDeriv(nbrScal,scalList,GL,derivType);
      newArray=contractionBuild(*arg,iParam1,iParam2);
    } // if (typeName=="contraction")
    else if (typeName=="inversion") {
      errThrow("arrayOp1::makeDeriv",
	       "   derivatives not implemented for inversion.");
    }
    else if (typeName=="function") {
      if (typeid(*fParam)==typeid(DConst)) {
	array *tmpArray=new array(*this);
	tmpArray->setOrder(order+1);
	for (i=0;i<tmpArray->order;i++)
	  tmpArray->setDim(i,dim[i]);
	tmpArray->setDim(order,nbrScal);
	for (i=0;i<tmpArray->nbrVal;i++)
	  tmpArray->componentValues[i]=0.0;
	newArray=tmpArray;
      }
      else if (typeid(*fParam)==typeid(DMult)) {
	arrayOp1 *tmpArray=new arrayOp1(*this);
	array* arg=tmpArray->arrayArg->makeDeriv(nbrScal,scalList,GL,
						 derivType);
	tmpArray->setArrayArg(arg);
	tmpArray->setOrder(arg->order);
	for (i=0;i<tmpArray->order;i++)
	  tmpArray->setDim(i,arg->dim[i]);
	newArray=tmpArray;
      }
      else {
	// Autres fonctions drivables :
	arrayOp1 *arg1=new arrayOp1(*this);
	arg1->setFunction(fParam->makeFunctionDeriv());
	array* arg2=arrayArg->makeDeriv(nbrScal,scalList,GL,derivType);
	if (order==0)
	  newArray=exteriorMultiplicationBuild(arg1,arg2);
	else
	  newArray=specialContractBuild(arg1,arg2,arg1->getOrder());
      }
    } // if (typeName=="function")
    else if (typeName=="indicesPermutation") {
      array *arg=arrayArg->makeDeriv(nbrScal,scalList,GL,derivType);
      arrayOp1 *tmpArray=new arrayOp1(*this);
      tmpArray->setOrder(order+1);
      delete[] tmpArray->IList;
      if (order+1>0)
	tmpArray->IList=new Int[order+1];
      else
	tmpArray->IList=NULL;
      tmpArray->IListNbr=order+1;
      for (i=0;i<order;i++) {
	tmpArray->setDim(i,dim[i]);
	tmpArray->IList[i]=IList[i];
      }
      tmpArray->setDim(order,nbrScal);
      tmpArray->IList[order]=order;
      tmpArray->setArrayArg(arg);
      newArray=tmpArray;
    } // if (typeName=="indicesPermutation")
    else if (typeName=="discretizedTransposition") {
      array *arg=arrayArg->makeDeriv(nbrScal,scalList,GL,derivType);
      newArray=discretizedTranspositionBuild(*arg);
    } // if (typeName=="discretizedTransposition")
    else
      errThrow("arrayOp1::makeDeriv",
	       string("   unknown operator or function type : ")+typeName+".");
    
    if (derivType=="explicit"||derivType=="coord") {
      Int nbrIdentical=0;
      for (i=0;i<nbrVal;i++) {
	for (j=0;j<nbrScal;j++) {
	  if (componentValues+i==scalList[j])
	    nbrIdentical++;
	}
      }
      if (nbrIdentical>0) {
	cerr << "Warning from arrayOp1::makeDeriv :" << endl
	  << "   this part of the routine has never been checked."
	    << endl ;
	if (nbrIdentical==nbrVal) {
	  delete[] newArray;
	  newArray=new array;
	  newArray->setOrder(order+1);
	  for (i=0;i<order;i++)
	    newArray->setDim(i,dim[i]);
	  newArray->setDim(order,nbrScal);
	  for (i=0;i<nbrVal;i++) {
	    for (j=0;j<nbrScal;j++) {
	      if (componentValues+i==scalList[j])
		newArray->componentValues[i*nbrScal+j]=1.0;
	      else
		newArray->componentValues[i*nbrScal+j]=0.0;
	    }
	  }
	} // if (nbrIdentical==nbrVal)
	else {
	  arrayComp *tmpArray=new arrayComp;
	  tmpArray->setOrder(newArray->getOrder());
	  for (i=0;i<newArray->getOrder();i++)
	    tmpArray->setDim(i,newArray->getDim(i));
	  for (i=0;i<tmpArray->nbrVal;i++) {
	    tmpArray->arrayList[i]=newArray;
	    newArray->addParent(tmpArray->arrayList+i);
	    tmpArray->compCorresp[i]=i;
	  }
	  bool componentDeriv;
	  Int newIndex;
	  for (i=0;i<nbrVal;i++) {
	    componentDeriv=false;
	    for (j=0;j<nbrScal;j++)
	      if (componentValues+i==scalList[j])
		componentDeriv=true;
	    if (componentDeriv) {
	      for (j=0;j<nbrScal;j++) {
		newIndex=i*nbrScal+j;
		delinkAndDelete(tmpArray->arrayList[newIndex],
				tmpArray->arrayList+newIndex);
		tmpArray->arrayList[newIndex]=NULL;
		tmpArray->compCorresp[newIndex]=-1;
		if (componentValues+i==scalList[j])
		  tmpArray->componentValues[newIndex]=1.0;
		else
		  tmpArray->componentValues[newIndex]=0.0;
	      } // for (j=0;j<nbrScal;j++)
	    } // if (componentDeriv)
	  } // for (i=0;i<nbrVal;i++)
	  tmpArray->makeDependence();
	  newArray=tmpArray;
	} // if (nbrIdentical==nbrVal) ... else
      } // if (nbrIdentical>0)
    }
    
    if (newArray!=NULL) // Pour viter un avertissement lors de la compilation,
      return(newArray); // on est oblig d'avoir recours  cet artifice.
    else                // (NB. l'avertissement ne se produisait qu' la
      return(NULL);     //  compilation optimise).
  }
  catchAll("arrayOp1::makeDeriv","")
}



array* arrayOp1::makeExplicitDeriv(Int nbrScal,scal **scalList) const {
  try {
    return(makeDeriv(nbrScal,scalList,NULL,"explicit"));
  }
  catchAll("arrayOp1::makeExplicitDeriv","")  
}



array* arrayOp1::makeCoordDeriv(globLoc *GL) const {
  try {
    return(makeDeriv(0,NULL,GL,"coord"));
  }
  catchAll("arrayOp1::makeCoordDeriv","")  
}



array* arrayOp1::makeCorlocDeriv(Int nbrDP) const {
  try {
    return(makeDeriv(nbrDP,NULL,NULL,"corloc"));  
  }
  catchAll("arrayOp1::makeCorlocDeriv","")  
}



void arrayOp1::setNewPoint(bool b) {
  if (arrayArg!=NULL)
    arrayArg->setNewPoint(b);
  newPoint::setNewPoint(b);
}



void arrayOp1::accessNbrInit(void) {
  if (arrayArg!=NULL)
    arrayArg->accessNbrInit();
  newPoint::accessNbrInit();
}



printTree arrayOp1::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt,ptArg;
    Int i;
    
    typeName=getTypeName();
    if (typeName=="none") {
      sprintf(cbuf,"arrayOp1 with no type");
      sbuf=cbuf;
    }
    else if (typeName=="transposition") {
      sprintf(cbuf,"arrayOp1=transpose(arrayArg,%d,%d)",iParam1,iParam2);
      sbuf=cbuf;
    }
    else if (typeName=="contraction") {
      sprintf(cbuf,"arrayOp1=contract(arrayArg,%d,%d)",iParam1,iParam2);
      sbuf=cbuf;
    }
    else if (typeName=="inversion") {
      sprintf(cbuf,"arrayOp1=inverse(arrayArg)");
      sbuf=cbuf;
    }
    else if (typeName=="function") {
      sbuf="arrayOp1="+fParam->strPrint();
    }
    else if (typeName=="indicesPermutation") {
      sprintf(cbuf,"arrayOp1=permutation(arrayArg");
      sbuf+=cbuf;
      for (i=0;i<IListNbr;i++) {
	sprintf(cbuf,",%2d",IList[i]);
	sbuf+=cbuf;
      }
      sbuf+=")";
    }
    else if (typeName=="discretizedTransposition") {
      sprintf(cbuf,"arrayOp1=discretizedTransposition(arrayArg)");
      sbuf=cbuf;
    }
    sprintf(cbuf," %p",static_cast<const void*>(this));
    sbuf+=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    if (values)
      pt+=getPrintTreeValues();
    
    sbuf="   ";
    pt+=sbuf;
    
    pt.prepareForArguments();
    
    ptArg=arrayArg->getFirstPrintTree("arrayArg",values);
    ptArg.prepareForAddLast();
    pt+=ptArg;
    
    return pt;
  }
  catchAll("arrayOp1::getPrintTree","")
}



set<field*>* arrayOp1::buildActiveFieldList(void) {
  try {
    FL=*(arrayArg->buildActiveFieldList());
    return(&FL);
  }
  catchAll("arrayOp1::buildActiveFieldList","")
}



set<parents*> arrayOp1::getListOfType(string typeName) {
  try {
    set<parents*> S,tmpS;
    set<parents*>::iterator it;
    
    if (isOfType(typeName))
      S.insert(dynamic_cast<parents*>(this));
    tmpS=arrayArg->getListOfType(typeName);
    for (it=tmpS.begin();it!=tmpS.end();it++)
      S.insert(*it);
    
    return(S);
  }
  catchAll("arrayOp1::getListOfType","")
}



