#ifndef ARRAY_OP1_H
#define ARRAY_OP1_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>
#include <string>

#include "array_base.h"

namespace keyFE2 {
  
  class function1 ;
  class globLoc ;
  class printTree ;
  class stepList ;
  class timeDep ;
  
  class arrayOp1 : public virtual array {
    
    friend class arrayOp2 ;
    
    protected :
      
      Int type ;
    
      array *arrayArg ;
    
      Int iParam1 ;
      Int iParam2 ;
    
      function1 *fParam ;
    
      Int IListNbr ;
      Int *IList ;
    
    public : 
      
      arrayOp1(void) ;
      arrayOp1(const array &in) ;
      arrayOp1(const arrayOp1 &in) ;
      ~arrayOp1(void) ;
    
      array* makeCloneForArrayTime(timeDep &TD) ;
      
      void setType(Int) ;
      void setType(std::string) ;
      Int getType(void) const ;
      std::string getTypeName(void) const ;
      
      void setArrayArg(array*) ;
      array* getArrayArg(void) const ;
      
      void setIParam1(Int) ;
      void setIParam2(Int) ;
      Int getIParam1(void) ;
      Int getIParam2(void) ;
      
      void setFunction(function1 *f) ;
      function1 *getFunction(void) const ;
      
      void setIListNbr(Int nbr) ;
      void setIList(Int index,Int value) const ;
      Int getIList(Int index) const ;
      Int* getIList(void) const ;
      
      void swapComponents(Int index1,Int index2) const ;
      
      void makeAtPoint(object *element,Int direct,Double *localCoord,
		       bool makeValue=true,bool makeDeriv=true) ;
      
      array* makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
		       std::string derivType) const ;
      array* makeExplicitDeriv(Int nbrScal,scal **scalList) const ;
      array* makeCoordDeriv(globLoc *GL) const ;
      array* makeCorlocDeriv(Int nbrDP) const ;
      
      void setNewPoint(bool) ;
      void accessNbrInit(void) ;
      
      printTree getPrintTree(bool values) const ;
      
      std::set<field*>* buildActiveFieldList(void) ;
      std::set<parents*> getListOfType(std::string typeName) ;
      
  } ;
  
}

#endif
