
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <stdio.h>
#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <string>

#include "arrayOp2.h"
#include "arrayComp.h"
#include "arrayFunctionsBuild.h"
#include "arrayOp1.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/tensorCoordSys_base.h"
#include "err/err.h"
#include "print/printTree.h"
#include "scal/scal.h"
#include "step/step.h"
#include "step/stepList.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;



arrayOp2::arrayOp2(void) : array() {
  type=0;
  arrayArg1=NULL;
  arrayArg2=NULL;
  iParam1=0;
  iParam2=0;
}



arrayOp2::arrayOp2(const array &Ain) : array(Ain) {
  type=0;
  arrayArg1=NULL;
  arrayArg2=NULL;
  iParam1=0;
  iParam2=0;
}



arrayOp2::arrayOp2(const arrayOp2 &in) : array(in) {
  try {
    type=in.type;
    if (in.arrayArg1!=NULL) {
      arrayArg1=in.arrayArg1;
      arrayArg1->addParent(&arrayArg1);
    }
    else
      arrayArg1=NULL;
    if (in.arrayArg2!=NULL) {
      arrayArg2=in.arrayArg2;
      arrayArg2->addParent(&arrayArg2);
    }
    else
      arrayArg2=NULL;
    iParam1=in.iParam1;
    iParam2=in.iParam2;
  }
  catchAll("arrayOp2::arrayOp2(const arrayOp2 &in)","")
}



arrayOp2::~arrayOp2(void) {
  try {
    type=0;
    delinkAndDelete(arrayArg1,&arrayArg1);
    arrayArg1=NULL;
    delinkAndDelete(arrayArg2,&arrayArg2);
    arrayArg2=NULL;
    iParam1=0;
    iParam2=0;
  }
  catchAll("arrayOp2::~arrayOp2","")
}



array* arrayOp2::makeCloneForArrayTime(timeDep &TD) {
  try {
    arrayOp2 *newArray=new arrayOp2(*this);
    newArray->setArrayArg1(arrayArg1->makeCloneForArrayTime(TD));
    newArray->setArrayArg2(arrayArg2->makeCloneForArrayTime(TD));
    return(newArray);
  }
  catchAll("arrayOp2::makeCloneForArrayTime","")
}



void arrayOp2::setType(Int tp) {
  type=tp;
}



void arrayOp2::setType(string typeName) {
  if (typeName=="none")
    type=0;
  else if (typeName=="specialContract")
    type=1;
  else if (typeName=="addition")
    type=2;
  else if (typeName=="soustraction")
    type=3;
  else if (typeName=="exteriorMultiplication")
    type=4;
  else if (typeName=="contractedMultiplication")
    type=5;
  else if (typeName=="accumulation")
    type=6;
  else if (typeName=="substitution")
    type=7;
  else if (typeName=="vectProduct")
    type=8;
  else
    errThrow("arrayOp2::setType",string("   unknown type name : ")+typeName
	     +".\n   (non implemented for arrayOp2.)") ;
}



Int arrayOp2::getType(void) const {
  return(type);
}



string arrayOp2::getTypeName(void) const {
  switch(type) {
  case 0 :
    return("none");
  case 1 :
    return("specialContract");
  case 2 :
    return("addition");
  case 3 :
    return("soustraction");
  case 4 :
    return("exteriorMultiplication");
  case 5 :
    return("contractedMultiplication");
  case 6 :
    return("accumulation");
  case 7 :
    return("substitution");
  case 8 :
    return("vectProduct");
  }
  return("unknown");
}



void arrayOp2::setArrayArg1(array *arg) {
  try {
    delinkAndDelete(arrayArg1,&arrayArg1);
    arrayArg1=arg;
    arrayArg1->addParent(&arrayArg1);
  }
  catchAll("arrayOp2::setArrayArg1","")
}



array* arrayOp2::getArrayArg1(void) const {
  return(arrayArg1);
}



void arrayOp2::setArrayArg2(array *arg) {
  try {
    delinkAndDelete(arrayArg2,&arrayArg2);
    arrayArg2=arg;
    arrayArg2->addParent(&arrayArg2);
  }
  catchAll("arrayOp2::setArrayArg2","")
}



array* arrayOp2::getArrayArg2(void) const {
  return(arrayArg2);
}



void arrayOp2::setIParam1(Int nbr) {
  iParam1=nbr;
}



Int arrayOp2::getIParam1(void) const {
  return(iParam1);
}



void arrayOp2::setIParam2(Int nbr) {
  iParam2=nbr;
}



Int arrayOp2::getIParam2(void) const {
  return(iParam2);
}



void arrayOp2::makeAtPoint(object *element,Int direct,Double *localCoord,
			   bool makeValue,bool makeDeriv) {
  
  if (!getNewPoint())
    return;
  newPoint::setNewPoint(false);
  
  try {
    
    arrayArg1->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
    arrayArg2->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
    
    switch(type) {
      
      // specialContract :
    case 1 :
      specialContractExec(arrayArg1,arrayArg2,iParam1);
      break;
      
      // Addition :
    case 2 :
      additionExec(arrayArg1,arrayArg2);
      break;
      
      // Soustraction :
    case 3 :
      soustractionExec(arrayArg1,arrayArg2);
      break;
      
      // Exterior multiplication :
    case 4 :
      exteriorMultiplicationExec(arrayArg1,arrayArg2);
      break;
      
      // Contracted multiplication :
    case 5 :
      contractedMultiplicationExec(arrayArg1,arrayArg2,iParam1,iParam2);
      break;
      
      // Accumulation :
    case 6 :
      accumulationExec(arrayArg1,arrayArg2,iParam1);
      break;
      
      // Substitution :
    case 7 :
      substitutionExec(arrayArg1,arrayArg2,iParam1,iParam2);
      break;
      
      // Vectorial Product :
    case 8 :
      vectProductExec(arrayArg1,arrayArg2);
      break;
      
      // Unknown operation type :
    default :
      errThrow("arrayOp2::makeAtPoint",
	       string("   unknown dyadic operator type : ")+strFromInt(type)
	       +".");
    }
    
    updateBoolZeroValue();
    
    return;
  }
  catchAll("arrayOp2::makeAtPoint","")
}



array* arrayOp2::makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
			   string derivType) const {
  try {
    Int i,j;
    string typeName;
    array *newArray;
    
    if (!(derivType=="explicit"||derivType=="coord"||derivType=="corloc"))
      errThrow("arrayOp2::makeDeriv",
	       string("   unknown derivation type : ")+derivType+".");
    
    if (derivType=="coord") {
      if (GL!=NULL) {
	nbrScal=GL->getCoordComponentsNbr();
	scalList=GL->getCoordComponents();
      }
      else
	errThrow("arrayOp2::makeDeriv",
		 "   no coord derivative can be calculated if GL==NULL");
    }
    
    typeName=getTypeName();
    
    if (typeName=="specialContract") {
      arrayOp2 *firstProduct=NULL,*secondProduct=NULL,*sumArray=NULL;
      arrayOp1 *permutationArray=NULL;
      Int i,index,order1,order2;
      
      firstProduct=new arrayOp2(*this);
      firstProduct->setArrayArg1(arrayArg1->makeDeriv(nbrScal,scalList,GL,
						      derivType));
      firstProduct->setArrayArg2(arrayArg2);
      order1=firstProduct->arrayArg1->order;
      order2=firstProduct->arrayArg2->order;
      firstProduct->setOrder(order1+order2-iParam1);
      for (i=0;i<order1;i++)
	firstProduct->setDim(i,firstProduct->arrayArg1->dim[i]);
      for (i=iParam1;i<order2;i++)
	firstProduct->setDim(i+order1-iParam1,firstProduct->arrayArg2->dim[i]);
      
      permutationArray=indicesPermutationBuild(*firstProduct);
      index=permutationArray->IList[order1-1];
      for (i=order1-1;i<firstProduct->order-1;i++)
	permutationArray->IList[i]=permutationArray->IList[i+1];
      permutationArray->IList[firstProduct->order-1]=index;
      
      secondProduct=new arrayOp2(*this);
      secondProduct->setArrayArg2(arrayArg2->makeDeriv(nbrScal,scalList,GL,
						       derivType));
      order1=secondProduct->arrayArg1->order;
      order2=secondProduct->arrayArg2->order;
      secondProduct->setOrder(order1+order2-iParam1);
      for (i=0;i<order1;i++)
	firstProduct->setDim(i,firstProduct->arrayArg1->dim[i]);
      for (i=iParam1;i<order2;i++)
	firstProduct->setDim(i+order1-iParam1,firstProduct->arrayArg2->dim[i]);
      
      sumArray=additionBuild(permutationArray,secondProduct);
      
      newArray=sumArray;
    } // if (typeName=="specialContract")
    
    else if (typeName=="addition"||typeName=="soustraction") {
      arrayOp2 *tmpArray=NULL;
      array *arg1=arrayArg1->makeDeriv(nbrScal,scalList,GL,derivType);
      array *arg2=arrayArg2->makeDeriv(nbrScal,scalList,GL,derivType);
      
      if (typeName=="addition")
	tmpArray=additionBuild(arg1,arg2);
      else if (typeName=="soustraction")
	tmpArray=soustractionBuild(arg1,arg2);
      
      newArray=tmpArray;
    } // if (typeName=="addition"||typeName=="soustraction")
    
    else if (typeName=="exteriorMultiplication") {
      arrayOp2 *firstProduct=NULL,*secondProduct=NULL,*sumArray=NULL;
      array *firstOp=NULL;
      Int i,index,order1,order2;
      
      firstProduct=new arrayOp2(*this);
      firstProduct->setArrayArg1(arrayArg1->makeDeriv(nbrScal,scalList,GL,
						      derivType));
      firstProduct->setArrayArg2(arrayArg2);
      order1=firstProduct->arrayArg1->order;
      order2=firstProduct->arrayArg2->order;
      firstProduct->setOrder(order1+order2);
      for (i=0;i<order1;i++)
	firstProduct->setDim(i,firstProduct->arrayArg1->dim[i]);
      for (i=0;i<order2;i++)
	firstProduct->setDim(order1+i,firstProduct->arrayArg2->dim[i]);
      
      if (arrayArg2->order!=0) {
	arrayOp1 *permutationArray=NULL;
	permutationArray=indicesPermutationBuild(*firstProduct);
	for (i=order1-1;i<firstProduct->order-1;i++)
	  permutationArray->IList[i]=i+1;
	permutationArray->IList[order1+order2-1]=order1-1;
	firstOp=permutationArray;
      }
      else
	firstOp=firstProduct;
      
      secondProduct=new arrayOp2(*this);
      secondProduct->setArrayArg2(arrayArg2->makeDeriv(nbrScal,scalList,GL,
						       derivType));
      order1=secondProduct->arrayArg1->order;
      order2=secondProduct->arrayArg2->order;
      secondProduct->setOrder(order1+order2);
      index=0;
      for (i=0;i<order1;i++) {
	secondProduct->setDim(index,secondProduct->arrayArg1->dim[i]);
	index++;
      }
      for (i=0;i<order2;i++) {
	secondProduct->setDim(index,secondProduct->arrayArg2->dim[i]);
	index++;
      }
      
      sumArray=additionBuild(firstOp,secondProduct);
      
      newArray=sumArray;
    } // if (typeName=="exteriorMultiplication")
    
    else if (typeName=="contractedMultiplication") {
      arrayOp2 *firstProduct=NULL,*secondProduct=NULL,*sumArray=NULL;
      arrayOp1 *permutationArray=NULL;
      Int i,index,order1,order2;
      
      firstProduct=new arrayOp2(*this);
      firstProduct->setArrayArg1(arrayArg1->makeDeriv(nbrScal,scalList,GL,
						      derivType));
      firstProduct->setArrayArg2(arrayArg2);
      order1=firstProduct->arrayArg1->order;
      order2=firstProduct->arrayArg2->order;
      firstProduct->setOrder(order1+order2-2);
      index=0;
      for (i=0;i<order1;i++)
	if (i!=iParam1) {
	firstProduct->setDim(index,firstProduct->arrayArg1->dim[i]);
	index++;
      }
      for (i=0;i<order2;i++)
	if (i!=iParam2) {
	firstProduct->setDim(index,firstProduct->arrayArg2->dim[i]);
	index++;
      }
  
      permutationArray=indicesPermutationBuild(*firstProduct);
      for (i=order1-2;i>=0&&i<firstProduct->order-1;i++)
	permutationArray->IList[i]=i+1;
      permutationArray->IList[order1+order2-3]=order1-2;

      secondProduct=new arrayOp2(*this);
      secondProduct->setArrayArg2(arrayArg2->makeDeriv(nbrScal,scalList,GL,
						       derivType));
      order1=secondProduct->arrayArg1->order;
      order2=secondProduct->arrayArg2->order;
      secondProduct->setOrder(order1+order2-2);
      index=0;
      for (i=0;i<order1;i++)
	if (i!=iParam1) {
	secondProduct->setDim(index,secondProduct->arrayArg1->dim[i]);
	index++;
      }
      for (i=0;i<order2;i++)
	if (i!=iParam2) {
	secondProduct->setDim(index,secondProduct->arrayArg2->dim[i]);
	index++;
      }
      
      sumArray=additionBuild(permutationArray,secondProduct);
      
      newArray=sumArray;
    } // if (typeName=="contractedMultiplication")
    
    else if (typeName=="accumulation") {
      array *firstOp,*secondOp;
      Int i,order1,order2;
      
      {
	array *tmp1=arrayArg1->makeDeriv(nbrScal,scalList,GL,derivType);
	arrayOp2 *accu=accumulationBuild(tmp1,arrayArg2,iParam1);
	order1=accu->arrayArg1->order;
	order2=accu->arrayArg2->order;
	if (order2!=0) {
	  arrayOp1 *permutationArray=indicesPermutationBuild(*accu);
	  for (i=order1-iParam1-1;i<accu->order-1;i++)
	    permutationArray->IList[i]=i+1;
	  permutationArray->IList[accu->order-1]=order1-iParam1-1;
	  firstOp=permutationArray;
	}
	else
	  firstOp=accu;      
      }
      
      {
	array *tmp2=arrayArg2->makeDeriv(nbrScal,scalList,GL,derivType);
	secondOp=accumulationBuild(arrayArg1,tmp2,iParam1);
      }
      
      newArray=additionBuild(firstOp,secondOp);
    } // if (typeName=="accumulation")
    
    else if (typeName=="substitution") {
      errThrow("arrayOp2::makeDeriv","   substitution not implemented yet.");
    }
    
    else if (typeName=="vectProduct") {
      errThrow("arrayOp2::makeDeriv","   vectorial product not implemented yet.");
    }
    
    else
      errThrow("arrayOp2::makeDeriv",
	       string("   unknown dyadic operator type : ")+typeName+".");
    
    if (derivType=="explicit"||derivType=="coord") {
      Int nbrIdentical=0;
      for (i=0;i<nbrVal;i++) {
	for (j=0;j<nbrScal;j++) {
	  if (componentValues+i==scalList[j])
	    nbrIdentical++;
	}
      }
      if (nbrIdentical>0) {
	cerr << "Warning from arrayOp2::makeDeriv :" << endl
	  << "   this part of the routine has never been checked."
	    << endl ;
	if (nbrIdentical==nbrVal) {
	  delete newArray;
	  newArray=new array;
	  newArray->setOrder(order+1);
	  for (i=0;i<order;i++)
	    newArray->setDim(i,dim[i]);
	  newArray->setDim(order,nbrScal);
	  for (i=0;i<nbrVal;i++) {
	    for (j=0;j<nbrScal;j++) {
	      if (componentValues+i==scalList[j])
		newArray->componentValues[i*nbrScal+j]=1.0;
	      else
		newArray->componentValues[i*nbrScal+j]=0.0;
	    }
	  }
	} // if (nbrIdentical==nbrVal)
	else {
	  arrayComp *tmpArray=new arrayComp;
	  tmpArray->setOrder(newArray->getOrder());
	  for (i=0;i<newArray->getOrder();i++)
	    tmpArray->setDim(i,newArray->getDim(i));
	  for (i=0;i<tmpArray->nbrVal;i++) {
	    tmpArray->arrayList[i]=newArray;
	    newArray->addParent(tmpArray->arrayList+i);
	    tmpArray->compCorresp[i]=i;
	  }
	  bool componentDeriv;
	  Int newIndex;
	  for (i=0;i<nbrVal;i++) {
	    componentDeriv=false;
	    for (j=0;j<nbrScal;j++)
	      if (componentValues+i==scalList[j])
		componentDeriv=true;
	    if (componentDeriv) {
	      for (j=0;j<nbrScal;j++) {
		newIndex=i*nbrScal+j;
		delinkAndDelete(tmpArray->arrayList[newIndex],
				tmpArray->arrayList+newIndex);
		tmpArray->arrayList[newIndex]=NULL;
		tmpArray->compCorresp[newIndex]=-1;
		if (componentValues+i==scalList[j])
		  tmpArray->componentValues[newIndex]=1.0;
		else
		  tmpArray->componentValues[newIndex]=0.0;
	      } // for (j=0;j<nbrScal;j++)
	    } // if (componentDeriv)
	  } // for (i=0;i<nbrVal;i++)
	  tmpArray->makeDependence();
	  newArray=tmpArray;
	} // if (nbrIdentical==nbrVal) ... else
      } // if (nbrIdentical>0)
    }
    
    return(newArray);
  }
  catchAll("arrayOp2::makeDeriv","")
}



array* arrayOp2::makeExplicitDeriv(Int nbrScal,scal **scalList) const {
  try {
    return(makeDeriv(nbrScal,scalList,NULL,"explicit"));
  }
  catchAll("arrayOp2::makeDeriv","")
}



array* arrayOp2::makeCoordDeriv(globLoc *GL) const {
  try {
    return(makeDeriv(0,NULL,GL,"coord"));
  }
  catchAll("arrayOp2::makeDeriv","")
}



array* arrayOp2::makeCorlocDeriv(Int nbrDP) const {
  try {
    return(makeDeriv(nbrDP,NULL,NULL,"corloc"));  
  }
  catchAll("arrayOp2::makeDeriv","")
}



void arrayOp2::setNewPoint(bool b) {
  if (arrayArg1!=NULL)
    arrayArg1->setNewPoint(b);
  if (arrayArg2!=NULL)
    arrayArg2->setNewPoint(b);
  newPoint::setNewPoint(b);
}



void arrayOp2::accessNbrInit() {
  if (arrayArg1!=NULL)
    arrayArg1->accessNbrInit();
  if (arrayArg1!=NULL)
    arrayArg2->accessNbrInit();
  newPoint::accessNbrInit();
}



printTree arrayOp2::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt,ptArg1,ptArg2;
    
    typeName=getTypeName();
    if (typeName=="none")
      sprintf(cbuf,"arrayOp2 with no type");
    else if (typeName=="specialContract")
      sprintf(cbuf,"arrayOp2=specialContract(arrayArg1,arrayArg2,%d)",iParam1);
    else if (typeName=="addition")
      sprintf(cbuf,"arrayOp2=arrayArg1+arrayArg2");
    else if (typeName=="soustraction")
      sprintf(cbuf,"arrayOp2=arrayArg1-arrayArg2");
    else if (typeName=="exteriorMultiplication")
      sprintf(cbuf,"arrayOp2=arrayArg1*arrayArg2");
    else if (typeName=="contractedMultiplication")
      sprintf(cbuf,"arrayOp2=contractedMultiplication(arrayArg1,%d,arrayArg2,%d)",
	      iParam1,iParam2);
    else if (typeName=="accumulation")
      sprintf(cbuf,"arrayOp2=accumulation(arrayArg1,arrayArg2,%d)",iParam1);
    else if (typeName=="substitution")
      sprintf(cbuf,"arrayOp2=substitution(arrayArg1,arrayArg2,%d)",iParam1);
    else if (typeName=="vectProduct")
      sprintf(cbuf,"arrayOp2=vectProduct(arrayArg1,arrayArg2)");
    
    sbuf=cbuf;
    sprintf(cbuf," %p",static_cast<const void*>(this));
    sbuf+=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    if (values)
      pt+=getPrintTreeValues();
    
    sbuf="   ";
    pt+=sbuf;
    
    pt.prepareForArguments();
    
    if (arrayArg1!=NULL)
      ptArg1=arrayArg1->getFirstPrintTree("arrayArg1",values);
    else
      ptArg1="arrayArg1 = NULL";
    ptArg1.prepareForAdd();
    pt+=ptArg1;
    
    if (arrayArg2!=NULL)
      ptArg2=arrayArg2->getFirstPrintTree("arrayArg2",values);
    else
      ptArg2="arrayArg2 = NULL";
    ptArg2.prepareForAddLast();
    pt+=ptArg2;
    
    return(pt);
  }
  catchAll("arrayOp2::getPrintTree","")
}



stepList arrayOp2::getStepList(void) {
  try {
    static stepList SL;
    SL.clear();
    if (order!=0)
      errThrow("arrayOp2::getStepList",
	       "   the array has more than one component.");
    
    if (type==2) {
      SL=arrayArg1->getStepList()+arrayArg2->getStepList();
    }    
    if (type==3) {
      static stepList SL2;
      SL2=arrayArg2->getStepList();
      SL2.multFactor(-1.0);
      SL=arrayArg1->getStepList()+SL2;
    }    
    else {
      step tmpStep(this);
      SL.pushBack(tmpStep);
    }
    return(SL);
  }
  catchAll("arrayOp2::getStepList","")
}


set<field*>* arrayOp2::buildActiveFieldList(void) {
  try {
    set<field*> tmpSF;
    set<field*>::iterator SFit;
    
    FL=*(arrayArg1->buildActiveFieldList());
    tmpSF=(*(arrayArg2->buildActiveFieldList()));
    for (SFit=tmpSF.begin();SFit!=tmpSF.end();SFit++)
      FL.insert(*SFit);
    return(&FL);
  }
  catchAll("arrayOp2::buildActiveFieldList","")
}



set<parents*> arrayOp2::getListOfType(string typeName) {
  try {
    set<parents*> S,tmpS;
    set<parents*>::iterator it;
    
    if (isOfType(typeName))
      S.insert(dynamic_cast<parents*>(this));
    tmpS=arrayArg1->getListOfType(typeName);
    for (it=tmpS.begin();it!=tmpS.end();it++)
      S.insert(*it);
    tmpS=arrayArg2->getListOfType(typeName);
    for (it=tmpS.begin();it!=tmpS.end();it++)
      S.insert(*it);
    
    return(S);
  }
  catchAll("arrayOp2::getListOfType","")
}



