
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <stdio.h>

#include "arrayOpN.h"
#include "arrayComp.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/tensorCoordSys_base.h"
#include "err/err.h"
#include "function/DUpwind.h"
#include "object/object_base.h"
#include "print/printTree.h"
#include "scal/scal.h"
#include "step/step.h"
#include "step/stepList.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;



arrayOpN::arrayOpN(void) : array() {
  type=0;
  arrayArgsNbr=0;
  arrayArgs=NULL;
  iParamsNbr=0;
  iParams=NULL;
  dParamsNbr=0;
  dParams=NULL;
}



arrayOpN::arrayOpN(const array &Ain) : array(Ain) {
  type=0;
  arrayArgsNbr=0;
  arrayArgs=NULL;
  iParamsNbr=0;
  iParams=NULL;
  dParamsNbr=0;
  dParams=NULL;
}



arrayOpN::arrayOpN(const arrayOpN &in) : array(in) {
  try {
    Int i;
    
    type=in.type;
    
    arrayArgsNbr=in.arrayArgsNbr;
    if (arrayArgsNbr!=0) {
      arrayArgs=new array* [arrayArgsNbr];
      for (i=0;i<arrayArgsNbr;i++) {
	arrayArgs[i]=in.arrayArgs[i];
	arrayArgs[i]->addParent(arrayArgs+i);
      }
    }
    else
      arrayArgs=NULL;
    
    iParamsNbr=in.iParamsNbr;
    if (iParamsNbr>0)
      iParams=new Int[iParamsNbr];
    else
      iParams=NULL;
    for (i=0;i<iParamsNbr;i++)
      iParams[i]=in.iParams[i];
    
    dParamsNbr=in.dParamsNbr;
    if (dParamsNbr>0)
      dParams=new Double[dParamsNbr];
    else
      dParams=NULL;
    for (i=0;i<dParamsNbr;i++)
      dParams[i]=in.dParams[i];
  }
  catchAll("arrayOpN::arrayOpN(const arrayOpN &in)","")
}



arrayOpN::~arrayOpN(void) {
  try {
    Int i;
    
    type=0;
    
    for (i=0;i<arrayArgsNbr;i++)
      delinkAndDelete(arrayArgs[i],arrayArgs+i);
    delete[] arrayArgs;
    arrayArgs=NULL;
    arrayArgsNbr=0;
    
    delete[] iParams;
    iParams=NULL;
    iParamsNbr=0;
    
    delete[] dParams;
    dParams=NULL;
    dParamsNbr=0;
  }
  catchAll("arrayOpN::~arrayOpN","")
}



array* arrayOpN::makeCloneForArrayTime(timeDep &TD) {
  try {
    Int i;
    arrayOpN *newArray=new arrayOpN(*this);
    for (i=0;i<arrayArgsNbr;i++)
      newArray->setArrayArg(i,arrayArgs[i]->makeCloneForArrayTime(TD));
    return(newArray);
  }
  catchAll("arrayOpN::makeCloneForArrayTime","");
}



void arrayOpN::setType(Int tp) {
  type=tp;
}



void arrayOpN::setType(string typeName) {
  if (typeName=="none")
    type=0;
  else if (typeName=="addition")
    type=1;
  else if (typeName=="combili")
    type=2;
  else if (typeName=="physicalDeriv")
    type=3;
  else if (typeName=="upwind")
    type=4;
  else
    errThrow("arrayOpN::setType",string("   unknown type name : ")
	     +typeName+".\n   (non implemented for arraOpN.)");
}



Int arrayOpN::getType(void) const {
  return(type);
}



string arrayOpN::getTypeName(void) const {
  switch(type) {
  case 0 :
    return("none");
  case 1 :
    return("addition");
  case 2 :
    return("combili");
  case 3 :
    return("physicalDeriv");
  case 4 :
    return("upwind");
  }
  return("unknown");
}



void arrayOpN::setArrayArgsNbr(Int nbr) {
  try {
    Int i;
    
    if (nbr==arrayArgsNbr)
      return;
    
    array** tmpList=new array* [nbr];
    for (i=0;i<nbr;i++)
      tmpList[i]=NULL;
    
    for (i=0;i<min(nbr,arrayArgsNbr);i++) {
      tmpList[i]=arrayArgs[i];
      tmpList[i]->addParent(tmpList+i);
    }
    
    for (i=0;i<arrayArgsNbr;i++)
      delinkAndDelete(arrayArgs[i],arrayArgs+i);
    delete[] arrayArgs;
    
    arrayArgs=tmpList;
    arrayArgsNbr=nbr;
  }
  catchAll("arrayOpN::setArrayArgsNbr","")
}



Int arrayOpN::getArrayArgsNbr(void) const {
  return(arrayArgsNbr);
}



void arrayOpN::setArrayArg(Int index,array *arg) {
  try {
    if (index<0||index>=arrayArgsNbr)
      errThrow("arrayOpN::setArrayArg",string("   invalid index : ")
	       +strFromInt(index)+".");
    delinkAndDelete(arrayArgs[index],arrayArgs+index);
    arrayArgs[index]=arg;
    if (arg!=NULL)
      arrayArgs[index]->addParent(arrayArgs+index);
  }
  catchAll("arrayOpN::setArrayArg","")
}



array* arrayOpN::getArrayArg(Int index) const {
  if (index<0||index>=arrayArgsNbr)
    errThrow("arrayOpN::getArrayArg",string("   invalid index : ")
	     +strFromInt(index)+".");
  return(arrayArgs[index]);
}



void arrayOpN::setIParamsNbr(Int nbr) {
  try {
    Int i,*tmp;
    
    if (nbr<0)
      errThrow("arrayOpN::setIParamsNbr",
	       string("   invalid iParams number : ")+strFromInt(nbr)+".");
    
    if (nbr>0)
      tmp=new Int[nbr];
    else
      tmp=NULL;
    for (i=0;i<min(nbr,iParamsNbr);i++)
      tmp[i]=iParams[i];
    delete[] iParams;
    iParamsNbr=nbr;
    iParams=tmp;
  }
  catchAll("arrayOpN::setIParamsNbr","")
}



Int arrayOpN::getIParamsNbr(void) const {
  return(iParamsNbr);
}



void arrayOpN::setIParam(Int index,Int param) {
  try {
    if (index<0||index>=iParamsNbr)
      errThrow("arrayOpN::setIParam",
	       string("   invalid index : ")+strFromInt(index)+".");
    iParams[index]=param;
  }
  catchAll("arrayOpN::setIParam","")
}



Int arrayOpN::getIParam(Int index) const {
  if (index<0||index>=iParamsNbr)
    errThrow("arrayOpN::getIParam",
	     string("   invalid index : ")+strFromInt(index)+".");
  return(iParams[index]);
}



void arrayOpN::setDParamsNbr(Int nbr) {
  try {
    Int i;
    Double *tmp;
    
    if (nbr<0)
      errThrow("arrayOpN::setDParamsNbr",
	       string("   invalid dParams number : ")+strFromInt(nbr)+".");
    
    if (nbr>0)
      tmp=new Double[nbr];
    else
      tmp=NULL;
    for (i=0;i<min(nbr,dParamsNbr);i++)
      tmp[i]=dParams[i];
    delete[] dParams;
    dParamsNbr=nbr;
    dParams=tmp;
  }
  catchAll("arrayOpN::setDParamsNbr","")
}



Int arrayOpN::getDParamsNbr(void) const {
  return(dParamsNbr);
}



void arrayOpN::setDParam(Int index,Double param) {
  try {
    if (index<0||index>=dParamsNbr)
      errThrow("arrayOpN::setDParam",
	       string("   invalid index : ")+strFromInt(index)+".");
    dParams[index]=param;
  }
  catchAll("arrayOpN::setDParam","")
}



Double arrayOpN::getDParam(Int index) const {
  if (index<0||index>=dParamsNbr)
    errThrow("arrayOpN::getDParam",
	     string("   invalid index : ")+strFromInt(index)+".");
  return(dParams[index]);
}



void arrayOpN::makeAtPoint(object *element,Int direct,Double *localCoord,
			   bool makeValue,bool makeDeriv) {
  Int i,j;
  Int TCStype;
  
  register Double *diter1,*diter2,*dFinal,dValue;
    
  if (!getNewPoint())
    return;
  newPoint::setNewPoint(false);
  
  try {
    
    if (type!=4) {
      for (j=0;j<arrayArgsNbr;j++)
        if (arrayArgs[j]!=NULL)
          arrayArgs[j]->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
    }
    
    switch(type) {
      
      // Addition :
    case 1 :
      for (i=0;i<nbrVal;i++) {
	scal::stackClear();
	for (j=0;j<arrayArgsNbr;j++)
	  scal::stackPushBack(1.0,arrayArgs[j]->componentValues[i]);
	componentValues[i].stackCalculate();
      }
      break;
      
      // Combili :
    case 2 :
      for (i=0;i<nbrVal;i++) {
	scal::stackClear();
	for (j=0;j<arrayArgsNbr;j++)
	  scal::stackPushBack(dParams[j],arrayArgs[j]->componentValues[i]);
	componentValues[i].stackCalculate();
      }
      break;
      
      // Physical Derivation :
    case 3 :
      TCStype=TCS->getType();
      
      if (TCStype>=1&&TCStype<=3) {
	for (i=0;i<nbrVal;i++)
	  componentValues[i]=(arrayArgs[arrayArgsNbr-1]->componentValues[i]);
      }
      else if (TCStype==4) {
	static scal *christVal,K022,K202,sqG22,tmpScal;
	static Int thisIndices[MAX_ORDER_ARRAY];
	static Int argsIndices[MAX_ORDER_ARRAY];
	static Int index;
	
	// One makes the assumption that only the first derivative is 
	// calculated.
	TCS->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	
	christVal=TCS->getChristoffel()->getComponentValues();
	K022=christVal[8];
	K202=christVal[20];
	sqG22=K022;
	
	for (i=0;i<MAX_ORDER_ARRAY;i++) {
	  thisIndices[i]=-1;
	  argsIndices[i]=-1;
	}
	
	for (i=0;i<nbrVal;i++) {
	  if (i%3==2) {
	    componentValues[i]=arrayArgs[1]->componentValues[i]*sqG22;
	    getIndicesFromIndex(i,thisIndices);
	    for (j=0;j<order-1;j++)
	      argsIndices[j]=thisIndices[j];
	    for (j=0;j<order-1;j++) {
	      if (thisIndices[j]==0) {
		argsIndices[j]=2;
		index=arrayArgs[0]->getIndexFromIndices(argsIndices);
		componentValues[i]+=K202*arrayArgs[0]->componentValues[index];
		argsIndices[j]=0;
	      }
	      else if (thisIndices[j]==2) {
		argsIndices[j]=0;
		index=arrayArgs[0]->getIndexFromIndices(argsIndices);
		componentValues[i]+=K022*arrayArgs[0]->componentValues[index];
		argsIndices[j]=2;
	      }
	    }
	  }
	  else {
	    componentValues[i]=arrayArgs[1]->componentValues[i];
	  }
	}
      }
      else
	errThrow("arrayOpN::makeAtPoint",
		 string("   no calculation can be made for TCS type : ")
		 +strFromInt(TCStype)+".");
      
      break;

      // upwind :    
    case 4 :
      {
        array *V,*rho,*k,*testU,*dXdt,*interp,*A,*aScal,*direction,
	   *invJacDirectNorm,*corlocDerivTestU,*invJacDirect;
      
        V=arrayArgs[0];
	rho=arrayArgs[1];
	k=arrayArgs[2];
	testU=arrayArgs[3];
	dXdt=arrayArgs[4];
	interp=arrayArgs[5];
	A=arrayArgs[6];
	aScal=arrayArgs[7];
	direction=arrayArgs[8];
	invJacDirectNorm=arrayArgs[9];
	corlocDerivTestU=arrayArgs[10];
	invJacDirect=arrayArgs[11];
	
	if (V!=NULL)
	  V->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (rho!=NULL)
	  rho->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (k!=NULL)
	  k->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (testU!=NULL)
	  testU->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (dXdt!=NULL)
	  dXdt->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (A!=NULL)
	  A->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (aScal!=NULL)
	  aScal->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	  
	if (aScal->componentValues[0].getVal()<1.0e-8) {
	  direction->componentValues[0]=1.0;
	  for (int i=1;i<direction->nbrVal;i++)
	     direction->componentValues[i]=0.0;
	}
	else {
	  for (int i=0;i<direction->nbrVal;i++)
	     direction->componentValues[i].div(A->componentValues[i],
	        aScal->componentValues[0]);
	}
	
	if (invJacDirect!=NULL)
	  invJacDirect->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (invJacDirectNorm!=NULL)
	  invJacDirectNorm->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	if (corlocDerivTestU!=NULL)
	  corlocDerivTestU->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
	
	static Double fact1,fact2,factor;
	fact1=interp->componentValues[0].getVal();
	if (element->getType()==3)
	   fact2=2.0*sqrt(2.0);
	else
	   fact2=1.0;
	factor=fact1*fact2;
	   
	static scal twoByH;
	twoByH.mult(invJacDirectNorm->componentValues[0],factor);
	   
	static scal num;
	num.mult(rho->componentValues[0],aScal->componentValues[0]); // rho*a
	
	static scal denom;
	denom.mult(k->componentValues[0],twoByH); // 2*lambda/h

	static scal alfa;
	if (25.0*abs(denom.val)<num.val||abs(denom.val)<1.0e-8)
	   alfa=25.0;
	else
	   alfa.div(num,denom);
	
	static scal nu;
	static DUpwind upF;
	nu.functionExec(&alfa,&upF,makeValue,makeDeriv);

	static scal fact,tmp1,tmp2[3];
	if (twoByH.getVal()>1.0e-5*nu.getVal())
   	   fact.div(nu,twoByH);
	else
	   fact=0.0;
	
	static int dim;
	dim=invJacDirect->getDim(0);
	for (i=0;i<dim;i++)
	   tmp2[i].mult(fact,invJacDirect->componentValues[i]);
	
        static bool optiScal;
        
        if (!scal::getWhite()) {
        
          // One first checks whether the optimized version is to be used :
        
          static Int maxIJDLigSize,maxIJDColSize;
          static Int maxTestULigSize,maxTestUColSize;

      	  maxIJDLigSize=0;
      	  maxIJDColSize=0;
          for (i=0;i<invJacDirect->nbrVal;i++) {
      	    maxIJDLigSize=max(maxIJDLigSize,
      	  	              invJacDirect->componentValues[i].ligSize);
      	    maxIJDColSize=max(maxIJDColSize,
      	  		      invJacDirect->componentValues[i].colSize);
      	  }

      	  maxTestULigSize=0;
      	  maxTestUColSize=0;
          for (i=0;i<nbrVal;i++) {
      	    maxTestULigSize=max(maxTestULigSize,testU->componentValues[i].ligSize);
      	    maxTestUColSize=max(maxTestUColSize,testU->componentValues[i].colSize);
      	  }

      	  if (maxIJDLigSize&&!maxIJDColSize&&!maxTestULigSize&&maxTestUColSize)
      	    optiScal=true;
      	  else
      	    optiScal=false;	    

//cerr << maxIJDLigSize << endl 
//     << maxIJDColSize << endl 
//     << maxTestULigSize << endl 
//     << maxTestUColSize << endl ;

        } // if (!scal::getWhite())
        else
          optiScal=false;
	  
//optiScal=false;
//cerr << "optiScal= " << optiScal << endl ;
	
        if (!optiScal) {
          // Classical calculation of the components :
          for (i=0;i<nbrVal;i++) {
	     componentValues[i]=testU->componentValues[i];
	     for (j=0;j<dim;j++)
	        componentValues[i]+=tmp2[j]*
		   corlocDerivTestU->componentValues[i*dim+j];
          } // for (i=0;i<nbrVal;i++)
	} // if (!optiScal)
	else {
	
//cerr << "optiScal=true" << endl ;
	
          // Optimized calculation of the components :
     	  
          static Int firstNonNULLComp=-1;
	       
          for (i=0;i<nbrVal;i++)
            if (testU->componentValues[i].getColSize()) {
              firstNonNULLComp=i;
      	      break;
      	    }
          if (firstNonNULLComp==-1)
            errThrow("arrayOpN::makeAtPoint",
      	  	     "   no not NULL component.");

          // (First not NULL component calculation) :
	
	  componentValues[firstNonNULLComp]=
	    testU->componentValues[firstNonNULLComp];
	  for (j=0;j<dim;j++)
	    componentValues[firstNonNULLComp]+=tmp2[j]*
	      corlocDerivTestU->componentValues[firstNonNULLComp*dim+j];
	  
          // (Other not NULL components calculation) :
	  
          for (i=firstNonNULLComp+1;i<nbrVal;i++) {
	    if (testU->componentValues[i].getColSize()) {
	      componentValues[i]=componentValues[firstNonNULLComp];
	      componentValues[i].colIndex=testU->componentValues[i].colIndex;
	      
	      componentValues[i].val=
	        testU->componentValues[firstNonNULLComp].val;
	      diter1=componentValues[i].lig;
	      dFinal=diter1+componentValues[i].ligSize;
	      while (diter1<dFinal)
	  	*(diter1++)=0.0;
	      for (j=0;j<dim;j++) {
	  	dValue=corlocDerivTestU->componentValues[i*dim+j].val;
	  	componentValues[i].val+=dValue*tmp2[j].val;
    	  	diter1=componentValues[i].lig+tmp2[j].ligIndex
	  	  -componentValues[i].ligIndex;
    	  	diter2=tmp2[j].lig;  
    	  	dFinal=diter1+tmp2[j].ligSize;
    	  	while (diter1<dFinal)
    	  	  *(diter1++)+=dValue*(*(diter2++));
	      } // for (j=0;j<dim;j++)
	    } // if (testU->componentValues[i].getColSize())
	  } // for (i=firstNonNULLComp+1;i<nbrVal;i++)
	  
        } // if (!optiScal) else
      }
      break;
      
      // Unknown operation type :
    default :
      errThrow("arrayOpN::makeAtPoint",
	       string("   unknown dyadic operator type : ")
	       +strFromInt(type)+".");
    }
    
    updateBoolZeroValue();
    
    return;
  }
  catchAll("arrayOpN::makeAtPoint","")
}



array* arrayOpN::makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
			   string derivType) const {
  try {
    Int i,j;
    string typeName;
    array *newArray,*arg;
    
    if (!(derivType=="explicit"||derivType=="coord"||derivType=="corloc"))
      errThrow("arrayOpN::makeDeriv",
	       string("   unknown dyadic operator type : \"")+derivType+"\".");
    
    if (derivType=="coord") {
      if (GL!=NULL) {
	nbrScal=GL->getCoordComponentsNbr();
	scalList=GL->getCoordComponents();
      }
      else
	errThrow("arrayOpN::makeDeriv",
		 "   no coord derivative can be calculated if GL==NULL");
    }
    
    typeName=getTypeName();
    
    if (typeName=="addition") {
      arrayOpN *tmpArray=new arrayOpN;
      tmpArray->setOrder(order+1);
      for (i=0;i<order;i++)
	tmpArray->setDim(i,dim[i]);
      tmpArray->setDim(order,nbrScal);
      tmpArray->setType(type);
      tmpArray->setArrayArgsNbr(arrayArgsNbr);
      for (i=0;i<arrayArgsNbr;i++) {
	arg=arrayArgs[i]->makeDeriv(nbrScal,scalList,GL,derivType);
	tmpArray->setArrayArg(i,arg);
      }    
      newArray=tmpArray;
    } // if (typeName=="addition")
    
    else if (typeName=="combili") {
      arrayOpN *tmpArray=new arrayOpN;
      tmpArray->setOrder(order+1);
      for (i=0;i<order;i++)
	tmpArray->setDim(i,dim[i]);
      tmpArray->setDim(order,nbrScal);
      tmpArray->setType(type);
      tmpArray->setArrayArgsNbr(arrayArgsNbr);
      for (i=0;i<arrayArgsNbr;i++) {
	arg=arrayArgs[i]->makeDeriv(nbrScal,scalList,GL,derivType);
	tmpArray->setArrayArg(i,arg);
      }
      tmpArray->setDParamsNbr(dParamsNbr);
      for (i=0;i<dParamsNbr;i++)
	tmpArray->setDParam(i,dParams[i]);
      newArray=tmpArray;
    } // if (typeName=="combili")
    
    else if (typeName=="physicalDeriv") {
      errThrow("arrayOpN::makeDeriv","   not implemented for physicalDeriv arrayOpN.");
    }
    
    else if (typeName=="upwind") {
      errThrow("arrayOpN::makeDeriv","   not implemented for upwind arrayOpN.");
    }
    
    else
      errThrow("arrayOpN::makeDeriv",
	       string("   unknown arrayOpN operator type : ")+typeName+".");
    
    if (derivType=="explicit"||derivType=="coord") {
      Int nbrIdentical=0;
      for (i=0;i<nbrVal;i++) {
	for (j=0;j<nbrScal;j++) {
	  if (componentValues+i==scalList[j])
	    nbrIdentical++;
	}
      }
      if (nbrIdentical>0) {
	cerr << "Warning from arrayOpN::makeDeriv :" << endl
	  << "   this part of the routine has never been checked."
	    << endl ;
	if (nbrIdentical==nbrVal) {
	  delete newArray;
	  newArray=new array;
	  newArray->setOrder(order+1);
	  for (i=0;i<order;i++)
	    newArray->setDim(i,dim[i]);
	  newArray->setDim(order,nbrScal);
	  for (i=0;i<nbrVal;i++) {
	    for (j=0;j<nbrScal;j++) {
	      if (componentValues+i==scalList[j])
		newArray->componentValues[i*nbrScal+j]=1.0;
	      else
		newArray->componentValues[i*nbrScal+j]=0.0;
	    }
	  }
	} // if (nbrIdentical==nbrVal)
	else {
	  arrayComp *tmpArray=new arrayComp;
	  tmpArray->setOrder(newArray->getOrder());
	  for (i=0;i<newArray->getOrder();i++)
	    tmpArray->setDim(i,newArray->getDim(i));
	  for (i=0;i<tmpArray->nbrVal;i++) {
	    tmpArray->arrayList[i]=newArray;
	    newArray->addParent(tmpArray->arrayList+i);
	    tmpArray->compCorresp[i]=i;
	  }
	  bool componentDeriv;
	  Int newIndex;
	  for (i=0;i<nbrVal;i++) {
	    componentDeriv=false;
	    for (j=0;j<nbrScal;j++)
	      if (componentValues+i==scalList[j])
		componentDeriv=true;
	    if (componentDeriv) {
	      for (j=0;j<nbrScal;j++) {
		newIndex=i*nbrScal+j;
		delinkAndDelete(tmpArray->arrayList[newIndex],
				tmpArray->arrayList+newIndex);
		tmpArray->arrayList[newIndex]=NULL;
		tmpArray->compCorresp[newIndex]=-1;
		if (componentValues+i==scalList[j])
		  tmpArray->componentValues[newIndex]=1.0;
		else
		  tmpArray->componentValues[newIndex]=0.0;
	      } // for (j=0;j<nbrScal;j++)
	    } // if (componentDeriv)
	  } // for (i=0;i<nbrVal;i++)
	  tmpArray->makeDependence();
	  newArray=tmpArray;
	} // if (nbrIdentical==nbrVal) ... else
      } // if (nbrIdentical>0)
    }
    
    return(newArray);
  }
  catchAll("arrayOpN::makeDeriv","")
}



array* arrayOpN::makeExplicitDeriv(Int nbrScal,scal **scalList) const {
  try {
    return(makeDeriv(nbrScal,scalList,NULL,"explicit"));
  }
  catchAll("arrayOpN::makeExplicitDeriv","")
}



array* arrayOpN::makeCoordDeriv(globLoc *GL) const {
  try {
    return(makeDeriv(0,NULL,GL,"coord"));
  }
  catchAll("arrayOpN::makeCoordDeriv","")
}



array* arrayOpN::makeCorlocDeriv(Int nbrDP) const {
  try {
    return(makeDeriv(nbrDP,NULL,NULL,"corloc"));  
  }
  catchAll("arrayOpN::makeCorlocDeriv","")
}



array* arrayOpN::makePhysicalDeriv(globLoc *GL) {
  try {
    string typeName=getTypeName();
    if (typeName=="addition"||typeName=="combili")
      return(buildPhysicalDeriv(GL));
    else if (typeName=="physicalDeriv") {
      string TCStypeName=TCS->getTypeName();
      if (TCStypeName=="cartesian 1D"||TCStypeName=="cartesian 2D"
	  ||TCStypeName=="cartesian 3D") {
	arrayOpN *tmpArray=new arrayOpN;
	tmpArray->setOrder(order+1);
	for (Int i=0;i<order+1;i++)
	  tmpArray->setDim(i,dim[i]);
	tmpArray->setTCS(TCS);
	tmpArray->setArrayArgsNbr(arrayArgsNbr+1);
	for (Int i=0;i<arrayArgsNbr;i++)
	  tmpArray->setArrayArg(i,arrayArgs[i]);
	tmpArray->setArrayArg(arrayArgsNbr,
			      arrayArgs[arrayArgsNbr-1]->makeCoordDeriv(GL));
	tmpArray->setType("physicalDeriv");
	return(tmpArray);
      }
      else if (TCStypeName=="cylindrical 3D") {
	errThrow("arrayOpN::makePhysicalDeriv",
		 string("   cannot calculate multiple physical derivative for \n")+
		 "   cylindrical coordinate system.");
      }
      else
	errThrow("arrayOpN::makePhysicalDeriv",
		 string("   cannot calculate physical derivative for \n")+
		 "   cylindrical coordinate system.");
    }
    else if (typeName=="upwind") {
      errThrow("arrayOpN::makePhysicalDeriv","not for upwind arrayOpN type.");
    }
    else
      errThrow("arrayOpN::makePhysicalDeriv",
	       string("   unknown arrayOpN operator type : ")+typeName+".");
  }
  catchAll("arrayOpN::makePhysicalDeriv","")
}



void arrayOpN::setNewPoint(bool b) {
  for (Int i=0;i<arrayArgsNbr;i++)
    if (arrayArgs[i]!=NULL)
      arrayArgs[i]->setNewPoint(b);
  newPoint::setNewPoint(b);
}



void arrayOpN::accessNbrInit(void) {
  for (Int i=0;i<arrayArgsNbr;i++)
    if (arrayArgs[i]!=NULL)
      arrayArgs[i]->accessNbrInit();
  newPoint::accessNbrInit();
}



printTree arrayOpN::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt,ptArg;
    Int i;
    
    typeName=getTypeName();
    Int nonNullNbr=0,nbr=0;
    
    if (typeName=="addition") {
      sprintf(cbuf,"arrayOpN=arrayArg1");
      sbuf=cbuf;
      for (i=1;i<arrayArgsNbr;i++) {
	sprintf(cbuf,"+arrayArg%d,",i+1);
	sbuf+=cbuf;
      }
    }
    else if (typeName=="combili") {
      sprintf(cbuf,"arrayOpN=%g*arrayArg1",dParams[0]);
      sbuf=cbuf;
      for (i=1;i<arrayArgsNbr;i++) {
	if (dParams[i]>=0)
	  sprintf(cbuf,"+%g*arrayArg%d",dParams[i],i+1);
	else
	  sprintf(cbuf,"%g*arrayArg%d",dParams[i],i+1);
	sbuf+=cbuf;
      }
    }
    else if (typeName=="physicalDeriv") {
      sprintf(cbuf,"arrayOpN=physicalDeriv(arrayArg1,arrayArg2)");
      sbuf=cbuf;
    }
    else if (typeName=="upwind") {
      sprintf(cbuf,"arrayOpN=upwind(");
      sbuf=cbuf;
      if (arrayArgs[0]!=NULL) {
         sbuf+="V";
	 nonNullNbr++;
      }
      if (arrayArgs[1]!=NULL) {
         sbuf+=",k";
	 nonNullNbr++;
      }
      if (arrayArgs[2]!=NULL) {
         sbuf+=",testU";
	 nonNullNbr++;
      }
      if (arrayArgs[3]!=NULL) {
         sbuf+=",X";
	 nonNullNbr++;
      }
      if (arrayArgs[4]!=NULL) {
         sbuf+=",dXdt";
	 nonNullNbr++;
      }
      sbuf+=")";
    }
    sprintf(cbuf," %p",static_cast<const void*>(this));
    sbuf+=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    if (values)
      pt+=getPrintTreeValues();
    
    sbuf="   ";
    pt+=sbuf;
    
    pt.prepareForArguments();
    
    if (typeName!="upwind") {
      for (i=0;i<arrayArgsNbr;i++) {
        sprintf(cbuf,"arrayArg%d",i+1);
        ptArg=arrayArgs[i]->getFirstPrintTree(cbuf,values);
        if (i<arrayArgsNbr-1)
    	  ptArg.prepareForAdd();
        else
    	  ptArg.prepareForAddLast();
        pt+=ptArg;
      }
    }
    else {
      for (i=0;i<5;i++) {
        sprintf(cbuf,"arrayArg%d",i+1);
	if (arrayArgs[i]!=NULL) {
	  nbr++;
          ptArg=arrayArgs[i]->getFirstPrintTree(cbuf,values);
          if (nbr<nonNullNbr)
    	    ptArg.prepareForAdd();
          else
    	    ptArg.prepareForAddLast();
          pt+=ptArg;
	}
      }
    }
    
    return(pt);
  }
  catchAll("arrayOpN::getPrintTree","")
}



stepList arrayOpN::getStepList(void) {
  try {
    if (order!=0) 
      errThrow("arrayOpN::getStepList",
	       "   the array has more than one component.");
    
    static stepList SL;
    SL.init();
    
    if (type==1) {
      for (Int i=0;i<arrayArgsNbr;i++)
	SL+=arrayArgs[i]->getStepList();
      return(SL);
    }
    else if (type==2) {
      static stepList tmpSL;
      for (Int i=0;i<arrayArgsNbr;i++) {
	tmpSL=arrayArgs[i]->getStepList();
	if (dParams[i]!=1.0)
	  tmpSL.multFactor(dParams[i]);
	SL+=tmpSL;
      }
      return(SL);
    }
    else
      errThrow("arrayOpN::getStepList",
	       "   this arrayOpN type is not allowed : "+strFromInt(type)+".");
  }
  catchAll("arrayOpN::getStepList","")
}



set<field*>* arrayOpN::buildActiveFieldList(void) {
  try {
    set<field*> tmpSF;
    set<field*>::iterator SFit;
    Int i;
    
    FL.clear();
    for (i=0;i<arrayArgsNbr;i++) {
      if (arrayArgs[i]!=NULL) {
        tmpSF=(*(arrayArgs[i]->buildActiveFieldList()));
        for (SFit=tmpSF.begin();SFit!=tmpSF.end();SFit++)
	  FL.insert(*SFit);
      }
    }
    
    return(&FL);
  }
  catchAll("arrayOpN::buildActiveFieldList","")
}



set<parents*> arrayOpN::getListOfType(string typeName) {
  try {
    Int i;
    set<parents*> S,tmpS;
    set<parents*>::iterator Sit;
    
    if (isOfType(typeName))
      S.insert(dynamic_cast<parents*>(this));
    
    for (i=0;i<arrayArgsNbr;i++) {
      if (arrayArgs[i]!=NULL) {
        tmpS=arrayArgs[i]->getListOfType(typeName);
        for (Sit=tmpS.begin();Sit!=tmpS.end();Sit++)
	  S.insert(*Sit);
      }
    }
    
    return(S);
  }
  catchAll("arrayOpN::getListOfType","")
}



