#ifndef ARRAY_OPN_H
#define ARRAY_OPN_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>
#include <string>

#include "array_base.h"

namespace keyFE2 {
  
  class field ;
  class globLoc ;
  class printTree ;
  class stepList ;
  
  class arrayOpN : public virtual array {
    
    protected :
      
      Int type ;
    
      Int arrayArgsNbr ;
      array **arrayArgs ;
    
      Int iParamsNbr ;
      Int *iParams ;
    
      Int dParamsNbr ;
      Double *dParams ;
    
    public : 
      
      arrayOpN(void) ;
      arrayOpN(const array &in) ;
      arrayOpN(const arrayOpN &in) ;
      ~arrayOpN(void) ;
    
      array* makeCloneForArrayTime(timeDep &TD) ;
      
      void setType(Int) ;
      void setType(std::string) ;
      Int getType(void) const ;
      std::string getTypeName(void) const ;
      
      void setArrayArgsNbr(Int) ;
      Int getArrayArgsNbr(void) const ;
      void setArrayArg(Int,array*) ;
      array* getArrayArg(Int) const ;
      
      void setIParamsNbr(Int nbr) ;
      Int getIParamsNbr(void) const ;
      void setIParam(Int index,Int param) ;
      Int getIParam(Int index) const ;
      
      void setDParamsNbr(Int nbr) ;
      Int getDParamsNbr(void) const ;
      void setDParam(Int index,Double param) ;
      Double getDParam(Int index) const ;
      
      void makeAtPoint(object *element,Int direct,Double *localCoord,
		       bool makeValue=true,bool makeDeriv=true) ;
      
      array* makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
		       std::string derivType) const ;
      array* makeExplicitDeriv(Int nbrScal,scal **scalList) const ;
      array* makeCoordDeriv(globLoc *GL) const ;
      array* makeCorlocDeriv(Int nbrDP) const ;
      array* makePhysicalDeriv(globLoc *GL) ;
      
      void setNewPoint(bool) ;
      void accessNbrInit(void) ;
      
      printTree getPrintTree(bool values) const ;
      
      stepList getStepList(void) ;
      std::set<field*>* buildActiveFieldList(void) ;
      std::set<parents*> getListOfType(std::string typeName) ;
      
  } ;
  
}

#endif

