
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <string>

#include "arrayTime.h"
#include "err/err.h"
#include "print/printTree.h"
#include "scal/scal.h"
#include "timeDep/timeDep_base.h"

using namespace std ;
using namespace keyFE2 ;



arrayTime::arrayTime(void) {
  currentArray=NULL;
  previousArray=NULL;
  currentTimeDep=NULL;
}



arrayTime::arrayTime(array &A,timeDep &TD) : array(A) {
  try {
    currentArray=NULL;
    setCurrentArray(&A);
    previousArray=NULL;
    setPreviousArray(A.makeCloneForArrayTime(TD));
    currentTimeDep=NULL;
    setCurrentTimeDep(&TD);
  }
  catchAll("arrayTime::arrayTime(array &A,timeDep &TD)","")
}



arrayTime::arrayTime(const arrayTime &in) : array(in) {
  try {
    currentArray=NULL;
    setCurrentArray(in.currentArray);
    previousArray=NULL;
    setPreviousArray(in.previousArray);
    currentTimeDep=NULL;
    setCurrentTimeDep(in.currentTimeDep);
  }
  catchAll("arrayTime::arrayTime(const arrayTime &in)","")
}



arrayTime::~arrayTime(void) {
  try {
    setCurrentArray(NULL);
    setPreviousArray(NULL);
    setCurrentTimeDep(NULL);
  }
  catchAll("arrayTime::~arrayTime","")
}



array* arrayTime::makeCloneForArrayTime(timeDep &TD) {
  errThrow("arrayTime::makeCloneForArrayTime",
	   "   cannot clone an arrayTime.");
}



void arrayTime::setCurrentArray(array *A) {
  try {
    delinkAndDelete(currentArray,&currentArray);
    currentArray=A;
    if (currentArray!=NULL)
      currentArray->addParent(&currentArray);
  }
  catchAll("arrayTime::setCurrentArray","")
}



void arrayTime::setPreviousArray(array *A) {
  try {
    delinkAndDelete(previousArray,&previousArray);
    previousArray=A;
    if (previousArray!=NULL)
      previousArray->addParent(&previousArray);
  }
  catchAll("arrayTime::setPreviousArray","")
}



void arrayTime::setCurrentTimeDep(timeDep *TD) {
  try {
    delinkAndDelete(currentTimeDep,&currentTimeDep);
    currentTimeDep=TD;
    if (currentTimeDep!=NULL)
      currentTimeDep->addParent(&currentTimeDep);
  }
  catchAll("arrayTime::setCurrentTimeDep","")
}



void arrayTime::makeAtPoint(object *element,Int direct,Double *localCoord,
		            bool makeValue,bool makeDeriv) {
  if (!getNewPoint())
    return;
  newPoint::setNewPoint(false);
  
  try {
    if (!currentTimeDep->isActive()) {
      for (Int i=0;i<nbrVal;i++) 
	componentValues[i]=0.0;
      setBoolZeroValue(true);
      return;
    }
    
    currentArray->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
    previousArray->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
    
    Double invThetaDT=1.0/(currentTimeDep->getThetaDT());
    for (Int i=0;i<nbrVal;i++) 
      componentValues[i]=invThetaDT*
        keyFE2::operator-(currentArray->componentValues[i],
			  previousArray->componentValues[i]);
  }
  catchAll("arrayTime::makeAtPoint","")
}



array* arrayTime::makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
		            string derivType) const {
  errThrow("arrayTime::makeDeriv","   illegal operation on arrayTime.");
}



array* arrayTime::makeExplicitDeriv(Int nbrScal,scal **scalList) const {
  errThrow("arrayTime::makeExplicitDeriv","   illegal operation on arrayTime.");
}



array* arrayTime::makeCoordDeriv(globLoc *GL) const {
  errThrow("arrayTime::makeCoordDeriv","   illegal operation on arrayTime.");
}



array* arrayTime::makeCorlocDeriv(Int nbrDP) const {
  errThrow("arrayTime::makeCorlocDeriv","   illegal operation on arrayTime.");
}



void arrayTime::setNewPoint(bool b) {
  if (currentArray!=NULL)
    currentArray->setNewPoint(b);
  if (previousArray!=NULL)
    previousArray->setNewPoint(b);
  newPoint::setNewPoint(b);
}



void arrayTime::accessNbrInit() {
  if (currentArray!=NULL)
    currentArray->accessNbrInit();
  if (previousArray!=NULL)
    previousArray->accessNbrInit();
  newPoint::accessNbrInit();
}



printTree arrayTime::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt,ptArg;
    
    sprintf(cbuf,"arrayTime");
    sbuf=cbuf;
    sprintf(cbuf," %p",static_cast<const void*>(this));
    sbuf+=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    if (values)
      pt+=getPrintTreeValues();
    
    sbuf="   ";
    pt+=sbuf;
    
    pt.prepareForArguments();
    
    ptArg=currentArray->getFirstPrintTree("currentArray",values);
    ptArg.prepareForAddLast();
    pt+=ptArg;
    
    return(pt);
  }
  catchAll("arrayTime::getPrintTree","")
}



set<field*>* arrayTime::buildActiveFieldList(void) {
  try {
    FL=*(currentArray->buildActiveFieldList());
    return(&FL);
  }
  catchAll("arrayTime::buildActiveFieldList","")
}



set<parents*> arrayTime::getListOfType(string typeName) {
  try {
    set<parents*> S,tmpS;
    set<parents*>::iterator it;
    
    if (isOfType(typeName))
      S.insert(dynamic_cast<parents*>(this));
    
    tmpS=currentArray->getListOfType(typeName);
    for (it=tmpS.begin();it!=tmpS.end();it++)
      S.insert(*it);
    
    return(S);
  }
  catchAll("arrayTime::getListOfType","")
}








