#ifndef ARRAY_TIME_H
#define ARRAY_TIME_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>
#include <string>

#include "array_base.h"

namespace keyFE2 {
  
  class Array ;
  class object ;
  class printTree ;
  class timeDep ;
  
  class arrayTime : public virtual array {
    
    friend class Array ;
    
    private :
      
      array *currentArray ;
      array *previousArray ;
      timeDep *currentTimeDep ;
    
    public :
      
      arrayTime(void) ;
      arrayTime(array &A,timeDep &TD) ;
      arrayTime(const arrayTime &in) ;
      ~arrayTime(void) ;
      
      array* makeCloneForArrayTime(timeDep &TD) ;
      
      void setCurrentArray(array *A) ;
      void setPreviousArray(array *A) ;
      void setCurrentTimeDep(timeDep *TD) ;
      
      void makeAtPoint(object *element,Int direct,Double *localCoord,
		       bool makeValue=true,bool makeDeriv=true) ;
      
      array* makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
		       std::string derivType) const ;
      array* makeExplicitDeriv(Int nbrScal,scal **scalList) const ;
      array* makeCoordDeriv(globLoc *GL) const ;
      array* makeCorlocDeriv(Int nbrDP) const ;
      
      void setNewPoint(bool) ;
      void accessNbrInit(void) ;
      
      printTree getPrintTree(bool values) const ;
      
      std::set<field*>* buildActiveFieldList(void) ;
      std::set<parents*> getListOfType(std::string typeName) ;

  } ;
  
}

#endif

