
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Constr.h"
#include "constr_base.h"
#include "constr1.h"
#include "constr2.h"
#include "array/array_base.h"
#include "array/arrayField.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/tensorCoordSys_base.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "param/param.h"
#include "param/paramList.h"

using namespace std ;
using namespace keyFE2 ;



Constr::Constr(void) {
  try {
    constr *C=new constr;
    data=NULL;
    setData(C);
  }
  catchAll("Constr::Constr(void)","")
}



Constr::Constr(const Constr &in) {
  try {
    data=NULL;
    setData(in.getData());
  }
  catchAll("Constr::Constr(const Constr &in)","")
}



Constr::Constr(param P) {
  try {
    paramList PL;
    PL.pushBack(P);
    Constr C(PL);
    data=NULL;
    setData(C.getData());
  }
  catchAll("Constr::Constr(param P)","")
}



Constr::Constr(paramList PL) {
  try {
    paramList plType=PL.getListOfType(TYPE);
    PL=PL.getListNotOfType(TYPE);  
    if (plType.getNbrParams()!=1)
      errThrow("Constr construction","exactly one TYPE parameter is needed.");
    
    string type=plType[0].getStringData();
    if (type=="usual") {
      constr1 *tmpConstr=new constr1(PL);
      data=NULL;
      setData(tmpConstr);
    }
    else if (type=="periodic") {
      constr2 *tmpConstr=new constr2(PL);
      data=NULL;
      setData(tmpConstr);
    }
    else
      errThrow("Constr construction",
	       string("unusual Constr TYPE data : ")+type+".");
  }
  catchAll("Constr::Constr(paramList PL)","")
}



Constr::~Constr(void) {
  try {
    setData(NULL);
  }
  catchAll("Constr::~Constr(void)","")
}



void Constr::setData(constr *newConstrData) {
  try {
    constr *C=getData();
    delinkAndDelete(C,&data);
    if (newConstrData!=NULL)
      newConstrData->addParent(&data);
    data=static_cast<void*>(newConstrData);
  }
  catchAll("Constr::setData","");
}



constr* Constr::getData(void) const {
  return(static_cast<constr*>(data));
}



void Constr::setName(string s) {
  try {
    constr *constrData=getData();
    constrData->setName(s);
  }
  catchAll("Constr::setName","")
}



string Constr::getName(void) const {
  constr *constrData=getData();
  return(constrData->getName());
}



void Constr::setLayer(Int nbr) {
  try{
    constr *constrData=getData();
    constrData->setLayer(nbr);
  }
  catchAll("Constr::setLayer","at the very end")  
}



Int Constr::getLayer(void) const {
  try {
    constr *constrData=getData();
    if (constrData!=NULL)
      return(constrData->getLayer());
    else
      return(0);
  }
  catchAll("Constr::getLayer","at the very end")  
}



void Constr::write(void) const {
  try {
    constr *constrData=getData();
    constrData->write();
  }
  catchAll("Constr::write","");
}



void Constr::write(ostream &out) const {
  try {
    constr *constrData=getData();
    constrData->write(out);
  }
  catchAll("Constr::write","");
}



void Constr::operator=(const Constr &arg) {
  if (this==&arg)
    return;
  
  try {
    setData(arg.getData());
  }
  catchAll("Constr::operator=","")
}








