
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iomanip>
#include <iostream>
#include <list>
#include <set>
#include <stdio.h>
#include <string>

#include "constr2.h"
#include "array/array_base.h"
#include "array/arrayFunctionsBuild.h"
#include "array/arrayInteg.h"
#include "err/err.h"
#include "expr/expression.h"
#include "field/field_base.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/tensorCoordSys_base.h"
#include "domain/domain_base.h"
#include "integ/integ_base.h"
#include "object/object_base.h"
#include "param/paramList.h"
#include "print/printTree.h"
#include "step/step.h"
#include "util/parents.h"

using namespace std ;
using namespace keyFE2 ;



constr2::constr2(void) : constr() {
  leftTest=NULL;
  leftEquation=NULL;
  leftExpr=NULL;
  leftDom=NULL;
  rightTest=NULL;
  rightEquation=NULL;
  rightExpr=NULL;
  rightDom=NULL;
  GL=NULL;
  TCS=NULL;
  integRule=-1;
}



constr2::constr2(paramList PL) : constr() {
  try {
    string tmpName;
    array *tmpLeftTest=NULL;
    array *tmpLeftEquation=NULL;
    domain *tmpLeftDomain=NULL;
    array *tmpRightTest=NULL;
    array *tmpRightEquation=NULL;
    domain *tmpRightDomain=NULL;
    globLoc *tmpGL=NULL;
    tensorCoordSys *tmpTCS=NULL;  
    Int tmpLayer;
    
    paramList plName=PL.getListOfType(NAME);
    PL=PL.getListNotOfType(NAME);
    paramList plLayer=PL.getListOfType(LAYER);
    PL=PL.getListNotOfType(LAYER);
    
    paramList plLeftTest=PL.getListOfType(LEFTTEST);
    PL=PL.getListNotOfType(LEFTTEST);
    paramList plLeftEquation=PL.getListOfType(LEFTEQUATION);
    PL=PL.getListNotOfType(LEFTEQUATION);
    paramList plLeftDomain=PL.getListOfType(LEFTDOM);
    PL=PL.getListNotOfType(LEFTDOM);
    
    paramList plRightTest=PL.getListOfType(RIGHTTEST);
    PL=PL.getListNotOfType(RIGHTTEST);
    paramList plRightEquation=PL.getListOfType(RIGHTEQUATION);
    PL=PL.getListNotOfType(RIGHTEQUATION);
    paramList plRightDomain=PL.getListOfType(RIGHTDOM);
    PL=PL.getListNotOfType(RIGHTDOM);
    
    paramList plGL=PL.getListOfType(GLOBLOC);
    PL=PL.getListNotOfType(GLOBLOC);
    paramList plTCS=PL.getListOfType(TENSORCOORDSYS);
    PL=PL.getListNotOfType(TENSORCOORDSYS);
    
    // Checks for illegal arguments :
    
    if (PL.getNbrParams()!=0) {
      string tmpString=strFromInt(PL.getNbrParams())+" invalid parameters :";
      for (Int i=0;i<PL.getNbrParams();i++) {
	tmpString+=" ";
	tmpString+=PL[i].getName();
      }
      tmpString+=".\n";
      errWrite("constr2 construction",tmpString);
    }
    
    if (plName.getNbrParams()>1) {
      errWrite("constr2 construction","only one NAME argument is allowed.");
    }
    else if (plName.getNbrParams()==1) {
      tmpName=plName[0].getStringData();
      if (tmpName=="")
	errWrite("constr2 construction",
		 "NAME= has to be followed by a string argument.");
    }
    
    tmpLayer=0;
    if (plLayer.getNbrParams()>1) {
      errWrite("constr1 construction","maximum one LAYER argument is allowed.");
    }
    else if (plLayer.getNbrParams()==1) {
      if(plLayer[0].getTypeName()!="Int") {
	errWrite("constr1 construction",
		 "NAME= has to be followed by an Int argument.");
      }
      else
        tmpLayer=plLayer[0].getIntData();
    }

    if (plLeftTest.getNbrParams()!=1) {
      errWrite("constr2 construction",
	       "exactly one LEFTTEST parameter is needed.");
    }
    else {
      parents *tmpParents=plLeftTest[0].getParentData();
      tmpLeftTest=dynamic_cast<array*>(tmpParents);
      if (tmpLeftTest==NULL) 
	errWrite("constr2 construction",
		 "LEFTTEST= must be followed by an Array.");
    }
    
    if (plLeftEquation.getNbrParams()!=1) {
      errWrite("constr2 construction",
	       "exactly one LEFTEQUATION parameter is needed.");
    }
    else {
      parents *tmpParents=plLeftEquation[0].getParentData();
      tmpLeftEquation=dynamic_cast<array*>(tmpParents);
      if (tmpLeftEquation==NULL)
	errWrite("constr2 construction",
		 "LEFTEQUATION= must be followed by an Array.");
    }
    
    if (plLeftDomain.getNbrParams()!=1) {
      errWrite("constr2 construction",
	       "exactly one LEFTDOM parameter is needed.");
    }
    else {
      parents *tmpParents=plLeftDomain[0].getParentData();
      tmpLeftDomain=dynamic_cast<domain*>(tmpParents);
      if (tmpLeftDomain==NULL)
	errWrite("constr2 construction",
		 "LEFTDOM= must be followed by a Domain.");
    }
    
    if (plRightTest.getNbrParams()!=1)  {
      errWrite("constr2 construction",
	       "exactly one RIGHTTEST parameter is needed.");
    }
    else {
      parents *tmpParents=plRightTest[0].getParentData();
      tmpRightTest=dynamic_cast<array*>(tmpParents);
      if (tmpRightTest==NULL)
	errWrite("constr2 construction",
		 "RIGHTTEST= must be followed by an Array.");
    }
    
    if (plRightEquation.getNbrParams()!=1) {
      errWrite("constr2 construction",
	       "exactly one RIGHTEQUATION parameter is needed.");
    }
    else {
      parents *tmpParents=plRightEquation[0].getParentData();
      tmpRightEquation=dynamic_cast<array*>(tmpParents);
      if (tmpRightEquation==NULL)
	errWrite("constr2 construction",
		 "RIGHTEQUATION= must be followed by an Array");
    }
    
    if (plRightDomain.getNbrParams()!=1)  {
      errWrite("constr2 construction",
	       "exactly one RIGHTDOM parameter is needed.");
    }
    else {
      parents *tmpParents=plRightDomain[0].getParentData();
      tmpRightDomain=dynamic_cast<domain*>(tmpParents);
      if (tmpRightDomain==NULL)
	errWrite("constr2 construction",
		 "RIGHTDOM= must be followed by a Domain.");
    }
    
    if (plGL.getNbrParams()>1) {
      errWrite("constr2 construction",
	       "maximum one GLOBLOC parameter is needed.");
    }
    else if (plGL.getNbrParams()==1) {
      parents *tmpParents=plGL[0].getParentData();
      tmpGL=dynamic_cast<globLoc*>(tmpParents);
      if (tmpGL==NULL)
	errWrite("constr2 construction",
		 "GLOBLOC= must be followed by a GlobLoc.");
    }
    
    if (plTCS.getNbrParams()>1) {
      errWrite("constr2 construction",
	       "maximum one TENSORCOORDSYS parameter is needed.");
    }
    else if (plTCS.getNbrParams()==1) {
      parents *tmpParents=plTCS[0].getParentData();
      tmpTCS=dynamic_cast<tensorCoordSys*>(tmpParents);
      if (tmpTCS==NULL)
	errWrite("constr2 construction",
		 "TENSORCOORDSYS= must be followed by a TensorCoordSys.");
    }
    
    if (!err::getOk())
      errThrow("constr2 construction","bailing out due to preceding errors.");
    
    leftTest=NULL;
    leftEquation=NULL;
    leftExpr=NULL;
    leftDom=NULL;
    rightTest=NULL;
    rightEquation=NULL;
    rightExpr=NULL;
    rightDom=NULL;
    GL=NULL;
    TCS=NULL;
    integRule=-1;
    
    setName(tmpName);
    setLayer(tmpLayer);
    
    setLeftDomain(tmpLeftDomain);
    setLeftTest(tmpLeftTest);
    setLeftEquation(tmpLeftEquation);
    
    setRightDomain(tmpRightDomain);
    setRightTest(tmpRightTest);
    setRightEquation(tmpRightEquation);
    
    setGlobLoc(tmpGL);
    setTCS(tmpTCS);
  }
  catchAll("constr2::constr2(paramList PL)","at the very end of function")
}



constr2::constr2(const constr2 &in) : constr(in) {
  try {
    leftTest=NULL;
    setLeftTest(in.leftTest);
    leftEquation=NULL;
    setLeftEquation(in.leftEquation);
    leftExpr=NULL;
    setLeftExpr(in.leftExpr);
    leftDom=NULL;
    setLeftDomain(in.leftDom);
    rightTest=NULL;
    setRightTest(in.rightTest);
    rightEquation=NULL;
    setRightEquation(in.rightEquation);
    rightExpr=NULL;
    setRightExpr(in.rightExpr);
    rightDom=NULL;
    setRightDomain(in.rightDom);
    GL=NULL;
    setGlobLoc(in.GL);
    TCS=NULL;
    setTCS(in.TCS);
    integRule=in.integRule;
  }
  catchAll("constr2::constr2(const constr2 &in)","")
}



constr2::~constr2(void) {
  try {
    setLeftTest(NULL);
    setLeftEquation(NULL);
    setLeftExpr(NULL);
    setLeftDomain(NULL);
    setRightTest(NULL);
    setRightEquation(NULL);
    setRightExpr(NULL);
    setRightDomain(NULL);
    setGlobLoc(NULL);
    setTCS(NULL);
    integRule=-1;
  }
  catchAll("constr2::~constr2(void)","")
}



void constr2::operator=(const constr2 &in) {
  if (this==&in)
    return;
  
  try {
    constr::operator=(in);
    setLeftTest(in.leftTest);
    setLeftEquation(in.leftEquation);
    setLeftExpr(in.leftExpr);
    setLeftDomain(in.leftDom);
    setRightTest(in.rightTest);
    setRightEquation(in.rightEquation);
    setRightExpr(in.rightExpr);
    setRightDomain(in.rightDom);
    setGlobLoc(in.GL);
    setTCS(in.TCS);
    integRule=in.integRule;
  }
  catchAll("constr2::operator=(const constr2 &in)","")
}



void constr2::setLeftTest(array *arg) {
  try {
    delinkAndDelete(leftTest,&leftTest);
    leftTest=arg;
    if (leftTest!=NULL)
      leftTest->addParent(&leftTest);  
    checkTestEquationMatch();
  }
  catchAll("constr2::setLeftTest","")
}



void constr2::setLeftEquation(array *arg) {
  try {
    delinkAndDelete(leftEquation,&leftEquation);
    leftEquation=arg;
    if (leftEquation!=NULL)
      leftEquation->addParent(&leftEquation);
    checkTestEquationMatch();
  }
  catchAll("constr2::setLeftEquation","")
}



void constr2::setLeftExpr(array *arg) {
  try {
    delinkAndDelete(leftExpr,&leftExpr);
    leftExpr=arg;
    if (leftExpr!=NULL)
      leftExpr->addParent(&leftExpr);
  }
  catchAll("constr2::setLeftExpr","")
}



void constr2::setLeftDomain(domain *arg) {
  try {
    delinkAndDelete(leftDom,&leftDom);
    leftDom=arg;
    if (leftDom!=NULL)
      leftDom->addParent(&leftDom);
    checkTestEquationMatch();
  }
  catchAll("constr2::setLeftDomain","")
}



void constr2::setRightTest(array *arg) {
  try {
    delinkAndDelete(rightTest,&rightTest);
    rightTest=arg;
    if (rightTest!=NULL)
      rightTest->addParent(&rightTest);  
    checkTestEquationMatch();
  }
  catchAll("constr2::setRightTest","")
}



void constr2::setRightEquation(array *arg) {
  try {
    delinkAndDelete(rightEquation,&rightEquation);
    rightEquation=arg;
    if (rightEquation!=NULL)
      rightEquation->addParent(&rightEquation);
    checkTestEquationMatch();
  }
  catchAll("constr2::setRightEquation","")
}



void constr2::setRightExpr(array *arg) {
  try {
    delinkAndDelete(rightExpr,&rightExpr);
    rightExpr=arg;
    if (rightExpr!=NULL)
      rightExpr->addParent(&rightExpr);
  }
  catchAll("constr2::setRightExpr","")
}



void constr2::setRightDomain(domain *arg) {
  try {
    delinkAndDelete(rightDom,&rightDom);
    rightDom=arg;
    if (rightDom!=NULL)
      rightDom->addParent(&rightDom);
    checkTestEquationMatch();
  }
  catchAll("constr2::setRightDomain","")
}



void constr2::setGlobLoc(globLoc *arg) {
  try {
    delinkAndDelete(GL,&GL);
    GL=arg;
    if (GL!=NULL)
      GL->addParent(&GL);
    checkTestEquationMatch();
  }
  catchAll("constr2::setGlobLoc","")
}



void constr2::setTCS(tensorCoordSys *arg) {
  try {
    delinkAndDelete(TCS,&TCS);
    TCS=arg;
    if (TCS!=NULL)
      TCS->addParent(&TCS);
    checkTestEquationMatch();
  }
  catchAll("constr2::setTCS","")
}



void constr2::checkTestEquationMatch(void) {
  try {
    Int rightRule=-1,leftRule=-1;
    
    if (leftTest!=NULL&&leftEquation!=NULL) {
      if (leftTest->getOrder()!=leftEquation->getOrder())
	errThrow("constr2::checkTestEquationMatch",
		 string("left order mismatch :  ")
		 +strFromInt(leftTest->getOrder())+"!="
		 +strFromInt(leftEquation->getOrder())
		 +" in constr2 named "+getName()+".");
      for (Int i=0;i<leftTest->getOrder();i++) {
	if (leftTest->getDim(i)!=leftEquation->getDim(i))
	  errThrow("constr2::checkTestEquationMatch",
		   string("left dimension mismatch for index ")
		   +strFromInt(i)+" :  "+strFromInt(leftTest->getDim(i))
		   +"!="+strFromInt(leftEquation->getDim(i))+".");
      }
      leftTest->buildActiveFieldList();
      set<field*> *tmpSF=leftTest->getFieldList();
      if (tmpSF->size()!=1)
	errThrow("constr2::checkTestEquationMatch",
		 "LEFTTEST expression should depend on exactly one field");
      field *F=*(tmpSF->begin());
      string ITN=F->getInterpolationTypeName();
      if (ITN=="constantOnDomain"||ITN=="constantPerElement")
	leftRule=100;
      else if (ITN=="linear")
	leftRule=101;
      else if (ITN=="quadratic")
	leftRule=102;
      else
	leftRule=-1;
      array *tmpArray=accumulationBuild(leftTest,leftEquation,
					leftTest->getOrder());
      setLeftExpr(tmpArray);
    }
    if (rightTest!=NULL&&rightEquation!=NULL) {
      if (rightTest->getOrder()!=rightEquation->getOrder())
	errThrow("constr2::checkTestEquationMatch",
		 string("right order mismatch :  ")
		 +strFromInt(rightTest->getOrder())+"!="
		 +strFromInt(rightEquation->getOrder())+".");
      for (Int i=0;i<rightTest->getOrder();i++) {
	if (rightTest->getDim(i)!=rightEquation->getDim(i)) 
	  errThrow("constr2::checkTestEquationMatch",
		   string("right dimension mismatch for index ")
		   +strFromInt(i)+" :  "+strFromInt(rightTest->getDim(i))
		   +"!="+strFromInt(rightEquation->getDim(i))+".");
      }
      rightTest->buildActiveFieldList();
      set<field*> *tmpSF=rightTest->getFieldList();
      if (tmpSF->size()!=1)
	errThrow("constr2::checkTestEquationMatch",
		 "RIGHTTEST expression should depend on exactly one field");
      field *F=*(tmpSF->begin());
      string ITN=F->getInterpolationTypeName();
      if (ITN=="constantOnDomain"||ITN=="constantPerElement")
	rightRule=100;
      else if (ITN=="linear")
	rightRule=101;
      else if (ITN=="quadratic")
	rightRule=102;
      else
	rightRule=-1;
      array *tmpArray=accumulationBuild(rightTest,rightEquation,
					rightTest->getOrder());
      setRightExpr(tmpArray);
    }
    
    if (rightRule!=leftRule&&!(rightRule==-1||leftRule==-1)) {
      errThrow("constr2::checkTestEquationMatch","rightRule!=leftRule");
    }
    else
      integRule=leftRule;
    
    if (GL!=NULL&&rightDom!=NULL)
      if (GL->getLocDim()!=rightDom->getDim())
	errThrow("constr2::checkTestEquationMatch",
		 "right domain dimension incompatible with globLoc argument.");
    if (GL!=NULL&&leftDom!=NULL)
      if (GL->getLocDim()!=leftDom->getDim())
	errThrow("constr1::checkTestEquationMatch",
		 "left domain dimension incompatible with globLoc argument.");

    return;
  }
  catchAll("constr2::checkTestEquationMatch","")
}



void constr2::generateExprCollect(bool makeValue,bool makeDeriv) {
  try {
    Int i,j,k,l;
    Int ligSize,colSize,ligIndex,colIndex,*map,counter,indexPos;
    step leftStep,rightStep;
    Double *col,*lig,**mat;
    Double tmpDouble1,tmpDouble2,factor;
    static scal scalFactor,tmpScal;
    
    object *element;
    Int elementDirect;
    
    integ ITG(GL,TCS,integRule);
    Double **integPoints,*integWeights;
    Int nbrPoints=0,nbrCoord=0;
    
    list<expression> EL1,leftEL,rightEL;
    list<expression>::iterator leftELit,rightELit,ELit;
    static expression tmpExpr,tmpExpr1,tmpExpr2;
    
    leftExpr->buildActiveFieldList();
    scal *leftCV=leftExpr->getComponentValues();
    leftStep.setEquation(leftExpr);
    rightExpr->buildActiveFieldList();
    scal *rightCV=rightExpr->getComponentValues();
    rightStep.setEquation(rightExpr);
    
    for (i=0;i<leftDom->getObjectsNbr();i++) {
      
      // Left part :
      
      element=leftDom->getObject(i);
      elementDirect=leftDom->getObjectDirect(i);
      ITG.getIntegPoints(element->getType(),nbrPoints,nbrCoord,integPoints,
			 integWeights);
      leftEL.clear();
      leftStep.setElement(element);
      leftStep.setDirection(elementDirect);
      for (l=0;l<nbrPoints;l++) {
	// One calculates entities at a point :
	leftStep.exec(integPoints[l]);
	ITG.setNewPoint(true);
	ITG.makeAtPoint(element,elementDirect,integPoints[l],makeValue,
			makeDeriv);    
	scalFactor=ITG.getIntegFactor(element,elementDirect,l,nbrPoints,
				      nbrCoord,integPoints,integWeights,
				      makeValue,makeDeriv);
	factor=scalFactor.getVal();
	tmpScal=(*leftCV);
	// One analyzes the tmpScal :
	ligSize=tmpScal.getLigSize();
	ligIndex=tmpScal.getLigIndex();
	colSize=tmpScal.getColSize();
	colIndex=tmpScal.getColIndex();
	col=tmpScal.getCol();
	mat=tmpScal.getMat();
	map=tmpScal.getMap();
	for (j=0;j<colSize;j++) {
	  lig=mat[j];
	  counter=0;
	  tmpDouble1=0.0;
	  tmpDouble2=0.0;
	  for (k=0;k<ligSize;k++) {
	    if (lig[k]!=0.0)
	      counter++;
	    if (map[ligIndex+k]==map[colIndex+j])
	      tmpDouble1=lig[k];
	  } // for k ...
	  if (abs(tmpDouble1)>tmpExpr.zero) {
	    tmpExpr.clear();
	    tmpExpr.setCapacity(max(counter,tmpExpr.getCapacity()));
	    tmpExpr.index=map[colIndex+j];
	    tmpExpr.weight=factor;
	    tmpExpr.independentValue=col[colIndex+j];
	    for (k=0;k<ligSize;k++) {
	      if (lig[k]!=0.0)
		tmpExpr.addTerm(map[ligIndex+k],lig[k]);
	    }
	    leftEL.push_back(tmpExpr);
	  } // if (...)
	} // for j ...
      } // for l ...
      
      // Right part :
      
      element=rightDom->getObject(i);
      elementDirect=rightDom->getObjectDirect(i);
      ITG.getIntegPoints(element->getType(),nbrPoints,nbrCoord,integPoints,
			 integWeights);
      rightEL.clear();
      rightStep.setElement(element);
      rightStep.setDirection(elementDirect);
      for (l=0;l<nbrPoints;l++) {
	// One calculates entities at a point :
	rightStep.exec(integPoints[l]);
	ITG.setNewPoint(true);
	ITG.makeAtPoint(element,elementDirect,integPoints[l],makeValue,
			makeDeriv);    
	scalFactor=ITG.getIntegFactor(element,elementDirect,l,nbrPoints,
				      nbrCoord,integPoints,integWeights,
				      makeValue,makeDeriv);
	factor=scalFactor.getVal();
	tmpScal=(*rightCV);
	// One analyzes the tmpScal :
	ligSize=tmpScal.getLigSize();
	ligIndex=tmpScal.getLigIndex();
	colSize=tmpScal.getColSize();
	colIndex=tmpScal.getColIndex();
	col=tmpScal.getCol();
	mat=tmpScal.getMat();
	map=tmpScal.getMap();
	for (j=0;j<colSize;j++) {
	  lig=mat[j];
	  counter=0;
	  tmpDouble1=0.0;
	  tmpDouble2=0.0;
	  for (k=0;k<ligSize;k++) {
	    if (lig[k]!=0.0)
	      counter++;
	    if (map[ligIndex+k]==map[colIndex+j])
	      tmpDouble1=lig[k];
	  } // for k ...
	  if (abs(tmpDouble1)>tmpExpr.zero) {
	    tmpExpr.clear();
	    tmpExpr.setCapacity(max(counter,tmpExpr.getCapacity()));
	    tmpExpr.index=map[colIndex+j];
	    tmpExpr.weight=factor;
	    tmpExpr.independentValue=col[colIndex+j];
	    for (k=0;k<ligSize;k++) {
	      if (lig[k]!=0.0)
		tmpExpr.addTerm(map[ligIndex+k],lig[k]);
	    }
	    rightEL.push_back(tmpExpr);
	  } // if (...)
	} // for j ...
      } // for l ...
      
      // Assembling of left and right parts :
      if (leftEL.size()!=rightEL.size()) 
	errThrow("constr2::generateExprCollect :",
		 "left and right expressionList sizes do not match.");
      for (leftELit=leftEL.begin(),rightELit=rightEL.begin();
	   leftELit!=leftEL.end();leftELit++,rightELit++) {
	tmpExpr1=*leftELit;
	tmpExpr2=*rightELit;
	tmpExpr.clear();
	tmpExpr.setCapacity(max(tmpExpr1.size+tmpExpr2.size-1,
				tmpExpr.getCapacity()));
	indexPos=-1;
	for (j=0;j<tmpExpr1.size;j++)
	  if (tmpExpr1.index==tmpExpr1.indices[j])
	    indexPos=j;
	if (indexPos==-1)
	  errThrow("constr2::generateExprCollect",
		   "left test and equation do not match.");
	tmpExpr.index=tmpExpr1.index;
	tmpExpr.weight=tmpExpr1.weight;
	tmpDouble1=tmpExpr1.factors[indexPos];
	tmpDouble2=1.0/tmpDouble1;
	tmpExpr.independentValue=-tmpExpr1.independentValue*tmpDouble2;
	for (j=0;j<tmpExpr1.size;j++)
	  if (j!=indexPos)
	    tmpExpr.addTerm(tmpExpr1.indices[j],
			    -tmpExpr1.factors[j]*tmpDouble2);
	tmpExpr.independentValue-=tmpExpr2.independentValue*tmpDouble2;
	for (j=0;j<tmpExpr2.size;j++)
	  tmpExpr.addTerm(tmpExpr2.indices[j],-tmpExpr2.factors[j]*tmpDouble2);
	EL1.push_back(tmpExpr);
      } // for leftELit rightELit ...    
    } // for i ...
    
    EL1.sort();
    ELit=EL1.begin();
    tmpExpr=*ELit;
    ELit++;
    EC.setCapacity(EL1.size());
    for (;ELit!=EL1.end();ELit++) {
      tmpExpr1=*ELit;
      if (tmpExpr1.index!=tmpExpr.index) {
	tmpExpr.simplify();
	if (tmpExpr.index!=-1)
	  EC.pushBack(tmpExpr);
	tmpExpr=tmpExpr1;
      }
      else {
	tmpExpr2=tmpExpr;
	tmpExpr.merge(tmpExpr1,tmpExpr2);
      }
    }
    tmpExpr.simplify();
    if (tmpExpr.index!=-1)
      EC.pushBack(tmpExpr);
      
    EC.setLayer(layer);
    
    if (EC.getSize()==0)
      errThrow("constr2::generateExprCollect",
	       string("no expressions generated by constr2 named \"")+getName()+"\".");
  }
  catchAll("constr2::generateExprCollect","")
}



printTree constr2::getPrintTree(bool values) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  printTree pt,ptArg;
    
  try {
    sprintf(cbuf,"constr2");
    sbuf=cbuf;
    sprintf(cbuf," %p",static_cast<const void*>(this));
    sbuf+=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    sbuf="   ";
    pt+=sbuf;
    
    pt.prepareForArguments();
    
    ptArg=leftTest->getFirstPrintTree("leftTest",values);
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    ptArg=leftEquation->getFirstPrintTree("leftEquation",values);
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    ptArg=leftDom->getFirstPrintTree("leftDom",values);
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    ptArg=rightTest->getFirstPrintTree("rightTest",values);
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    ptArg=rightEquation->getFirstPrintTree("rightEquation",values);
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    ptArg=rightDom->getFirstPrintTree("rightDom",values);
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    ptArg=GL->getFirstPrintTree("GL",values);
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    sprintf(cbuf,"integRule = %d",integRule);
    ptArg=cbuf;
    ptArg.prepareForAdd();
    pt+=ptArg;
    pt+="   ";
    
    return(pt);
  }
  catchAll("constr1::getPrintTree","")
}



