
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iomanip>
#include <iostream>
#include <stdio.h>
#include <string>

#include "constr_base.h"
#include "err/err.h"
#include "expr/expression.h"
#include "print/printTree.h"

using namespace std ;
using namespace keyFE2 ;



constr::constr(void) : name(), parents() {
  layer=0;
}



constr::constr(const constr &in) : name(in), parents() {
  layer=in.layer;
  EC=in.EC;
}



constr::~constr(void) {
  layer=0;
  EC.setCapacity(0);
}



void constr::operator=(const constr &in) {
  if (this==&in)
    return;
  
  layer=0;
  setName(in.getName());
  EC=in.EC;
}



void constr::setLayer(Int nbr) {
  layer=nbr;
}



Int constr::getLayer(void) const {
  return layer;
}



void constr::generateExprCollect(bool makeValue,bool makeDeriv) {
  errThrow("constr::generateExprCollect",
	   "cannot generate exprCollect for base constr object type.");
}



void constr::resolveExprCollect(void) {
  EC.solve();
}



const exprCollect & constr::getExprCollect(void) const {
  return(EC);
}



printTree constr::getPrintTreeData(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  printTree pt;
  
  try {
    if (getName()=="")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    if (layer!=0) {
      sprintf(cbuf,"   layer = %d",layer);
      sbuf=cbuf;
      pt+=sbuf;
    }
        
    return(pt);
  }
  catchAll("constr::getPrintTreeData","")
}



printTree constr::getPrintTree(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  printTree pt;
  
  try {
    sprintf(cbuf,"constr %p",static_cast<const void*>(this));
    sbuf=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    sbuf="   ";
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("constr::getPrintTree","")
}



printTree constr::getPrintTreeSummary(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt;
    
    sprintf(cbuf,"constr %p",static_cast<const void*>(this));
    pt+=cbuf;
    
    if (getName()=="")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    pt+=cbuf;
    
    if (layer!=0) {
      sprintf(cbuf,"   layer = %d",layer);
      sbuf=cbuf;
      pt+=sbuf;
    }
        
    sbuf="   ";
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("constr::getPrintTreeSummary","")
}

    
    




