
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "GlobLoc.h"
#include "globLoc_base.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "coordSys/TensorCoordSys.h"
#include "coordSys/tensorCoordSys_base.h"
#include "err/err.h"
#include "param/param.h"
#include "param/paramList.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {

  GlobLoc GlobLoc::defaultGL("undefined") ;

}



using namespace keyFE2 ;



GlobLoc::GlobLoc(void) {
  try {
    data=NULL;
    globLoc *GL=new globLoc;
    setData(GL);
  }
  catchAll("GlobLoc::GlobLoc(void)","")
}



GlobLoc::GlobLoc(string str) {
  try {
    data=NULL;
    globLoc *GL=new globLoc;
    GL->setName(str);
    setData(GL);
  }
  catchAll("GlobLoc::GlobLoc(string)","")
}



GlobLoc::GlobLoc(param P) {
  try {
    paramList PL;
    PL.pushBack(P);
    GlobLoc GL(PL);
    data=NULL;
    setData(GL.getData());
  }
  catchAll("GlobLoc::GlobLoc(param P)","")
}



GlobLoc::GlobLoc(paramList PL) {
  try {
    string glName="";
    string glType="";
    array *coord=NULL;
    
    paramList plName=PL.getListOfType(NAME);
    PL=PL.getListNotOfType(NAME);
    paramList plGLT=PL.getListOfType(GLTYPE);
    PL=PL.getListNotOfType(GLTYPE);
    paramList plCoord=PL.getListOfType(COORD);
    PL=PL.getListNotOfType(COORD);
    
    // Checks for illegal arguments :
    
    if (PL.getNbrParams()!=0) {
      string tmpString=strFromInt(PL.getNbrParams())+" invalid parameters :";
      for (Int i=0;i<PL.getNbrParams();i++) {
	tmpString+=" ";
	tmpString+=PL[i].getName();
      }
      tmpString+=".\n";
      errWrite("GlobLoc construction",tmpString);
    }
    
    // Sets the name of the globloc :
    
    if (plName.getNbrParams()>1) {
      errWrite("GlobLoc construction","only one NAME argument is allowed.");
    }
    else if (plName.getNbrParams()==1) {
      glName=plName[0].getStringData();
      if (glName=="")
	errWrite("GlobLoc construction",
		 "NAME= has to be followed by a string argument.");
    }
    
    // Sets the type of the globloc :
    
    if (plGLT.getNbrParams()!=1) {
      errWrite("GlobLoc construction","exactly one GLTYPE argument is needed.");
    }
    else {
      glType=plGLT[0].getStringData();
      if (glType=="")
	errWrite("GlobLoc construction",
		 "GLTYPE= has to be followed by a string argument.");
    }
    
    // Gets the coordinates of the globloc :
    
    if (plCoord.getNbrParams()!=1) {
      errWrite("GlobLoc construction","exactly one COORD argument is needed.");
    }
    else {
      parents *tmpParent=plCoord[0].getParentData();
      coord=dynamic_cast<array*>(tmpParent);
      if (coord==NULL)
	errWrite("GlobLoc construction",
		 "COORD= has to be followed by an Array argument.");
    }
    
    if (!err::getOk())
      errThrow("GlobLoc construction","bailing out due to preceding errors.");
    
    // Tests:
    
    if (glType!="0D"&&glType!="0D0D"
        &&glType!="1D1D"&&glType!="1D0D"
	&&glType!="2D2D"&&glType!="2D1D"&&glType!="2D0D"
	&&glType!="3D2D"&&glType!="3D3D"&&glType!="3D1D"&&glType!="3D0D")
      errThrow("GlobLoc construction","unkwnown type: "+glType);
    
    if (coord->getOrder()!=1)
      errThrow("GlobLoc construction","order of COORD Array must be 1");
    
    if ((glType=="1D1D"||glType=="1D0D")&&coord->getDim(0)!=1) {
      errThrow("GlobLoc construction","dim of COORD Array must be 1");
    }
    else if ((glType=="2D2D"||glType=="2D1D"||glType=="2D0D")
             &&coord->getDim(0)!=2) {
      errThrow("GlobLoc construction","dim of COORD Array must be 2");
    }
    else if ((glType=="3D2D"||glType=="3D3D"||glType=="3D1D"||glType=="3D0D")
	     &&coord->getDim(0)!=3)
      errThrow("GlobLoc construction","dim of COORD Array must be 3");
    
    globLoc *outputGL;
    if (glType=="0D"||glType=="0D0D")
      outputGL=build0D0DGlobLoc(coord);
    else if (glType=="1D1D")
      outputGL=build1D1DGlobLoc(coord);
    else if (glType=="1D0D")
      outputGL=build1D0DGlobLoc(coord);
    else if (glType=="2D2D")
      outputGL=build2D2DGlobLoc(coord);
    else if (glType=="2D1D")
      outputGL=build2D1DGlobLoc(coord);
    else if (glType=="2D0D")
      outputGL=build2D0DGlobLoc(coord);
    else if (glType=="3D2D")
      outputGL=build3D2DGlobLoc(coord);
    else if (glType=="3D3D")
      outputGL=build3D3DGlobLoc(coord);
    else if (glType=="3D1D")
      outputGL=build3D1DGlobLoc(coord);
    else if (glType=="3D0D")
      outputGL=build3D0DGlobLoc(coord);
    else
      errThrow("GlobLoc construction",
	       string("illegal GLTYPE argument : ")+glType+".");
    if (glName!="")
      outputGL->setName(glName);
    
    data=NULL;
    setData(outputGL);
  }
  catchAll("GlobLoc::GlobLoc(paramList PL)","at the end of function")
}



GlobLoc::GlobLoc(const GlobLoc &in) {
  try {
    data=NULL;
    setData(in.getData());
  }
  catchAll("GlobLoc::GlobLoc(const GlobLoc &in)","")
}



GlobLoc::~GlobLoc(void) {
  try {
    setData(NULL);
  }
  catchAll("GlobLoc::~GlobLoc(void)","")
}



void GlobLoc::setData(globLoc *newGL) {
  try {
    globLoc *oldGL=getData();
    if (oldGL!=NULL)
      delinkAndDelete(oldGL,&data);

    if (newGL!=NULL)
      newGL->addParent(&data);

    data=static_cast<void*>(newGL);
  }
  catchAll("GlobLoc::setData","")
}



globLoc* GlobLoc::getData(void) const {
  return(static_cast<globLoc*>(data));
}



void GlobLoc::setName(string s) {
  globLoc *GL=getData();
  GL->setName(s);
}



string GlobLoc::getName(void) const {
  globLoc *GL=getData();
  return(GL->getName());
}



GlobLoc GlobLoc::operator=(const GlobLoc &arg) {
  if (this==&arg)
    return(*this);
  
  try {
    setData(arg.getData());
    return(*this);
  }
  catchAll("GlobLoc::operator=","")
}



void GlobLoc::setDefaultGL(const GlobLoc &GL) {
  try {
    defaultGL=GL;
  }
  catchAll("GlobLoc::setDefaultGL","")
}



void GlobLoc::unsetDefaultGL(void) {
  try {
    globLoc *gl=new globLoc("undefined");
    defaultGL.setData(gl);
  }
  catchAll("GlobLoc::unsetDefaultGL","")
}



GlobLoc GlobLoc::getDefaultGL(void) {
  return(defaultGL);
}



string GlobLoc::getTypeName(void) const {
  globLoc *GL=getData();
  return(GL->getTypeName());
}



Int GlobLoc::getGlobDimFromName(string &name) {
  try {
    return globLoc::getGlobDimFromName(name);
  }
  catchAll("GlobLoc::getGlobDimFromName","")
}



Int GlobLoc::getLocDimFromName(string &name) {
  try {
    return globLoc::getLocDimFromName(name);
  }
  catchAll("GlobLoc::getLocDimFromName","")
}



Array& GlobLoc::getNormal(const TensorCoordSys &TCS) const {
  try {
    globLoc *GL=getData();
    tensorCoordSys *tcs=TCS.getData();
    
    array *a=GL->getNormal(tcs);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getNormal(const TensorCoordSys &TCS)","")
}



Array& GlobLoc::getTangeant(const TensorCoordSys &TCS) const {
  try {
    globLoc *GL=getData();
    tensorCoordSys *tcs=TCS.getData();
    
    array *a=GL->getTangeant(tcs);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getTangeant(const TensorCoordSys &TCS)","")
}



Array& GlobLoc::getVector1(const TensorCoordSys &TCS) const {
  try {
    globLoc *GL=getData();
    tensorCoordSys *tcs=TCS.getData();
    
    array *a=GL->getVector1(tcs);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getVector1(const TensorCoordSys &TCS)","")
}



Array& GlobLoc::getVector2(const TensorCoordSys &TCS) const {
  try {
    globLoc *GL=getData();
    tensorCoordSys *tcs=TCS.getData();
    
    array *a=GL->getVector2(tcs);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getVector2(const TensorCoordSys &TCS)","")
}



Array& GlobLoc::getVector3(const TensorCoordSys &TCS) const {
  try {
    globLoc *GL=getData();
    tensorCoordSys *tcs=TCS.getData();
    
    array *a=GL->getVector3(tcs);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getVector3(const TensorCoordSys &TCS)","")
}



Array& GlobLoc::getNormal(void) const {
  try {
    globLoc *GL=getData();
    
    array *a=GL->getNormal(NULL);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getNormal(void)","")
}



Array& GlobLoc::getTangeant(void) const {
  try {
    globLoc *GL=getData();
    
    array *a=GL->getTangeant(NULL);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getTangeant(void)","")
}



Array& GlobLoc::getVector1(void) const {
  try {
    globLoc *GL=getData();
    
    array *a=GL->getVector1(NULL);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getVector1(void)","")
}



Array& GlobLoc::getVector2(void) const {
  try {
    globLoc *GL=getData();
    
    array *a=GL->getVector2(NULL);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getVector2(void)","")
}



Array& GlobLoc::getVector3(void) const {
  try {
    globLoc *GL=getData();
    
    array *a=GL->getVector3(NULL);
    
    static Array A;
    A.setData(a);
    
    return(A);
  }
  catchAll("GlobLoc::getVector3(void)","")
}



void GlobLoc::write(void) const {
  try {
    globLoc *globLocData=getData();
    globLocData->write();
  }
  catchAll("GlobLoc::write","");
}



void GlobLoc::writeValues(void) const {
  try {
    globLoc *globLocData=getData();
    globLocData->writeValues();
  }
  catchAll("GlobLoc::writeValues","");
}



void GlobLoc::write(ostream &out) const {
  try {
    globLoc *globLocData=getData();
    globLocData->write(out);
  }
  catchAll("GlobLoc::write","");
}



void GlobLoc::writeValues(ostream &out) const {
  try {
    globLoc *globLocData=getData();
    globLocData->writeValues(out);
  }
  catchAll("GlobLoc::writeValues","");
}




