#ifndef GLOBLOC_BASE_H
#define GLOBLOC_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>
#include <string>

#include "types/params.h"
#include "types/types.h"

#include "util/name.h"
#include "util/newPoint.h"
#include "util/parents.h"
#include "print/print.h"
#include "scal/scal.h"

namespace keyFE2 {
  
  class array ;
  class field ;
  class object ;
  class tensorCoordSys ;
  
  class globLoc : public virtual name, public virtual newPoint, 
		  public virtual parents, public virtual print {
								 
    private :
      
      Int type ;
    
      array *coord ;
      array *coordDeriv ;
    
      array *Jac ;
      array *invJac ;
      scal detJac ;
      
      array *vectors ;
      
    public :
      
      void init(void) ;
    
      globLoc(void) ;
      globLoc(std::string str) ;
      ~globLoc(void) ;
      
      void setType(Int nbr) ;
      void setType(std::string typeName) ;
      Int getType(void) const ;
      std::string getTypeName(void) const ;
      
      void setCoord(array*) ;
      array* getCoord(void) const ;
      Int getCoordComponentsNbr(void) const ;
      scal** getCoordComponents(void) const ;
      
      void setCoordDeriv(array*) ;
      array* getCoordDeriv(void) const ;
      void setJac(array*) ;
      array* getJac(void) const ;
      void setInvJac(array*) ;
      array* getInvJac(void) const ;
      const scal& getDetJac(void) const ;
      
      Int getLocDim(void) const ;
      Int getGlobDim(void) const ;      
      static Int getGlobDimFromName(std::string &name) ;
      static Int getLocDimFromName(std::string &name) ;
      
      void setVectors(array*) ;
      void setVectors(void) ;
      array* getVectors(void) const ;
      array* getNormal(tensorCoordSys *TCS) const ;
      array* getTangeant(tensorCoordSys *TCS) const ;
      array* getVector1(tensorCoordSys *TCS) const ;
      array* getVector2(tensorCoordSys *TCS) const ;
      array* getVector3(tensorCoordSys *TCS) const ;
      
      void makeAtPoint(object *element,Int direct,Double *localCoord,
		       bool makeValue=true,bool makeDeriv=true);
      
      void setNewPoint(bool);
      void accessNbrInit(void);
      
      std::set<field*>* buildActiveFieldList(void) ;
    
      virtual printTree getPrintTreeData(void) const ;
      virtual printTree getPrintTreeValues(void) const ;
      virtual printTree getPrintTree(bool values) const ;
      
  } ;
  
  globLoc* build0D0DGlobLoc(array *Coord) ;
  globLoc* build1D1DGlobLoc(array *Coord) ;
  globLoc* build1D0DGlobLoc(array *Coord) ;
  globLoc* build2D2DGlobLoc(array *Coord) ;
  globLoc* build2D1DGlobLoc(array *Coord) ;
  globLoc* build2D0DGlobLoc(array *Coord) ;
  globLoc* build3D2DGlobLoc(array *Coord) ;
  globLoc* build3D3DGlobLoc(array *Coord) ;
  globLoc* build3D1DGlobLoc(array *Coord) ;
  globLoc* build3D0DGlobLoc(array *Coord) ;
  
}

#endif
