#ifndef DOMAIN_H
#define DOMAIN_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>
#include <iostream>
#include <fstream>

#include "object/Object.h"
#include "types/types.h"

namespace keyFE2 {
  
  class domain ;
  class mesh ;
  class object ;
  
  class Domain {
    
    private :
      
      void *data ;
    
    public :
      
      Domain(void) ;
      Domain(std::string) ;
      Domain(const Domain &) ;
      ~Domain(void) ;
    
    // domain interface :
    
      void setData(domain*) ;
      domain* getData(void) const ;
      void setName(std::string str) ;
      std::string getName(void)const ;
      
      Int getDim(void) const ;
      mesh* getBaseMesh(void) const ;
      Int getObjectsNbr(void) const ;
//      object* getObject(Int) const ;
//      Int getObjectDirect(Int) const ;
      Object getObject(Int) const ;
      Int getAllObjectsNbr(void) const ;
      object* getAllObject(Int) const ;
      Int getAllObject(object *) const ;
      void makeAllObjectsLst(void) ;
      
      void write(void) const ;
      void writeValues(void) const ;
      void write(std::ostream &out) const ;
      void writeValues(std::ostream &out) const ;
      
      void splitWrite(std::string outFile );
      void splitWrite(std::ostream &myCout) ;
      
      void operator=(const Domain&) ;
      
      Domain operator+(const Domain&) const ;
      Domain operator-(const Domain&) const ;
      Domain operator-() const ;
      Domain operator*(const Domain&) const ;
      
      Domain getBoundary(void) const ;
      void getExtremities(Domain &D1,Domain &D2) const ;
      Domain getFirstNode(void) const ;
    
  } ;
  
}

#endif
