#ifndef DOMAIN_BASE_H
#define DOMAIN_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <string>

#include "print/print.h"
#include "util/utils.h"
#include "util/parents.h"

namespace keyFE2 {
  
  class mesh;
  class object;
  
  class domain : public virtual name, public virtual parents,
		 public virtual print {
					
    friend class mesh ;
    
    private :
      
      mesh *baseMesh ;
    
      Int dim ;
      Int objectsNbr ;
      object** objectsLst ;
      Int *objectsDirect ;
    
      Int allObjectsNbr ;
      object** allObjectsLst ;
    
    public :
      
      domain(void) ;
      domain(std::string) ;
      domain(const domain &) ;
      ~domain(void) ;
      
      void setBaseMesh(mesh*) ;
      mesh* getBaseMesh(void) const ;
      void setObjectsNbr(Int) ;
      Int getObjectsNbr(void) const ;
      void setObject(Int,object*,Int) ;
      object* getObject(Int) const ;
      Int getObjectDirect(Int) const ;
      
      void setAllObjectsNbr(Int) ;
      Int getAllObjectsNbr(void) const ;
      void setAllObject(Int,object*) ;
      object* getAllObject(Int) const ;
      Int getAllObject(object *) const ;
      void makeAllObjectsLst(void) ;
      
      printTree getPrintTreeData(void) const ;
      printTree getPrintTreeValues(void) const ;
      printTree getPrintTree(bool values) const ;
      printTree getPrintTreeSummary(void) const ;
      
      void splitWrite(std::ostream &myCout) ;
      void splitWrite(std::string outFile) ;
      
      Int setDim(void) ;
      Int getDim(void) const ;
      
      void operator=(const domain&) ;
      domain operator+(const domain&) const ;
      domain operator-(const domain&) const ;
      domain operator-() const ;
      domain operator*(const domain&) const ;
      void reorder1DDomain(const domain &arg) ;
      
      domain getBoundary(void) const ;
      void getExtremities(domain &d1,domain &d2) const ;
      domain* getFirstNode(void) const ;
      
  } ;
  
}

#endif
