#ifndef ERR_H
#define ERR_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>
#include <iostream>

namespace keyFE2 {

  class err {
    
    private :
      
      std::string functionName ;
      std::string fileName ;
      int lineNumber ;
      std::string message ;
      
      static bool Ok ;
      
    public :
      
      err(void) ;
      err(std::string fuName,std::string fiName,int liNumber,std::string msg) ;
      err(std::string fiName,int liNumber,std::string msg) ;
      err(const err &in) ;
      ~err(void) ;
      
      void write(std::ostream &out) const ;
      void exec(std::ostream &out) ;
      void exec(void) ;
      
      static void setOk(bool) ;
      static bool getOk(void) ;
    
  } ;
  
}

#define errThrow(a,b) { \
   err brol(a,__FILE__,__LINE__,b); \
   brol.write(cerr); \
   throw(brol); \
}

#define errWrite(a,b) { \
   err (a,__FILE__,__LINE__,b).exec(); \
   err::setOk(false); \
}
    
#define catchAll(a,b) catch(err) { \
   cerr << "Error caught in :  " << __FILE__ << ":" << __LINE__ << endl ; \
   cerr << "   in function :  " << a << endl ; \
   cerr << "   with message :  " << b << endl ; \
   err brol(a,__FILE__,__LINE__,b) ; \
   throw(brol) ; \
}

#endif

