#ifndef EXPRESSION_H
#define EXPRESSION_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <map>

#include "types/types.h"


namespace keyFE2 {

  typedef std::pair<Int,Double> IntDouble ;
  
  class expression {
    
    friend class constr ;
    friend class constr1 ;
    friend class constr2 ;
    friend class exprCollect ;
    friend class solver ;
    
    private :
      
      static Double zero ;
    
    public :
    
      Int layer ;
      Double weight ;
      Int index ;
      Double independentValue ;
      
      Int capacity ;
      Int size ;
      Double *factors ;
      Int *indices ;
      
    public :
      
      expression(void) ;
      expression(Int initCapacity) ;
      expression(const expression &in) ;
      ~expression(void) ;
      
      void setZero(Double d) ;
      Double getZero(void) const ;
      
      void operator=(const expression &in) ;
      bool operator<(const expression &right) ;
      
      void setLayer(Int nbr) ;
      void setIndex(Int nbr) ;
      void setIndependentValue(Double val) ;
      void setWeight(Double w) ;
      Double getWeight(void) const ;
      
      void setCapacity(Int nbr) ;
      Int getCapacity(void) const ;
      
      void clear(void) ;
      void addTerm(Int nbr,Double val) ;
      
      void simplify(void) ;
      void merge(const expression &left,const expression &right) ;
      void substitute(const expression &left,const expression &right) ;
      
      void write(std::ostream &os) const ;
      
  } ;
  
}

#endif
