#ifndef FIELD_H
#define FIELD_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <string>

#include "types/types.h"

namespace keyFE2 {
  
  class field ;
  class Array ;
  class Domain ;
  class param ;
  class paramList ;
  class printTree ;
  
  class Field {
    
    private :
      
      void *data ;
    
    public :
      
      Field(void) ;
      Field(const Field &F) ;
      Field(param P) ;
      Field(paramList PL) ;
      ~Field(void) ;
      
      // field interface :
      
      void setData(field *f) ;
      field* getData(void) const ;
      void setName(std::string s) ;
      std::string getName(void)const ;
      
      void operator=(const Field &F) ;
      void makeCopy(const Field &F) ;
      
      Int getNbrComponents(void) const ;
      std::string getComponentName(Int index) const ;
      
      Int getInterpolationType(void) const ;
      std::string getInterpolationTypeName(void) const ;
      
      void interpolate(Array &origArray,Domain &interpDom,Array &coordArray) ;
      void interpolate(Field &origField,Domain &interpDom,Field &coordField) ;
      printTree getPrintTree(bool values) const ;
      Field& component(Int index) ;
      
      void write(std::ostream &os) const ;
      void write(void) const ;      
      void writeValues(std::ostream &os) const ;
      void writeValues(void) const ;

  } ;
  
}

#endif
