
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <stdio.h>
#include <typeinfo>
#include <iomanip>
#include <cmath>
#include <new>
#include <string>

#include "field_base.h"
#include "array/Array.h"
#include "array/arrayField.h"
#include "coordSys/globLoc_base.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "Field.h"
#include "integ/integ_base.h"
#include "object/object_base.h"
#include "scal/scal.h"
#include "shape/interpolationTypes.h"
#include "shape/shape.h"
#include "param/param.h"
#include "param/paramList.h"
#include "print/printTree.h"

using namespace std ;
using namespace keyFE2 ;



Int field::allFieldsCapacity=10;
Int field::allFieldsSize=0;
Int field::allNodalValuesSize=0;
//#warning modifier quand l erreur interne de compilateur sera corrige!
field** field::allFields=new field* [10]; // 10 = field::allFieldsCapacity !!!



field::field(void) : name() {
  try {
    defDomain=NULL;
    
    nbrComponents=0;
    componentName=NULL;
    
    interpolationType=shape::interpolationType("none");
    
    nodalValuesNbr=0;
    nodalValuesNbrPC=0;
    nodalValues=NULL;
    objectNodalValuesIndexNbr=0;
    objectNodalValuesIndex=NULL;
    
    currentObject=NULL;
    currentObjectDirect=0;
    
    localNodalValuesSize=0;
    localNodalValuesCapacity=0;
    mappingLocalGlobal=NULL;
    mappingLocalScal=NULL;
    localNodalValues=NULL;
    
    currentComponent=-1;    
    minRefForRelativeVariation=1.0e-8;
    
    addToAllFields(this);
  }
  catchAll("field::field(void)","")
}



field::field(domain* dom) : name() {
  try {
    defDomain=NULL;
    setDefDomain(NULL);
    
    nbrComponents=0;
    componentName=NULL;
    
    interpolationType=shape::interpolationType("none");
    
    nodalValuesNbr=0;
    nodalValuesNbrPC=0;
    nodalValues=NULL;
    objectNodalValuesIndexNbr=0;
    objectNodalValuesIndex=NULL;
    
    currentObject=NULL;
    currentObjectDirect=0;
    
    localNodalValuesSize=0;
    localNodalValuesCapacity=0;
    mappingLocalGlobal=NULL;
    mappingLocalScal=NULL;
    localNodalValues=NULL;
    
    currentComponent=-1;
    minRefForRelativeVariation=1.0e-8;
    
    addToAllFields(this);
  }
  catchAll("field::field(domain* dom)","")
}



field::field(domain* dom,Int nbrCmp) : name() {
  try {
    char chaine[12];
    Int i;
    
    defDomain=NULL;
    setDefDomain(dom);
    
    nbrComponents=nbrCmp;
    if (nbrComponents!=0)
      componentName=new string[nbrComponents];
    else
      componentName=NULL;
    for (i=0;i<nbrComponents;i++) {
      sprintf(chaine,"%d",i);
      componentName[i]=chaine;
    }
    
    interpolationType=shape::interpolationType("none");
    
    nodalValuesNbr=0;
    nodalValuesNbrPC=0;
    nodalValues=NULL;
    objectNodalValuesIndexNbr=0;
    objectNodalValuesIndex=NULL;
    
    currentObject=NULL;
    currentObjectDirect=0;
    
    localNodalValuesSize=0;
    localNodalValuesCapacity=0;
    mappingLocalGlobal=NULL;
    mappingLocalScal=NULL;
    localNodalValues=NULL;
    
    currentComponent=-1;
    minRefForRelativeVariation=1.0e-8;
    
    addToAllFields(this);
  }
  catchAll("field::field(domain* dom,Int nbrCmp)","")
}



field::field(domain* dom,Int nbrCmp,string interpType) : name() {
  try {
    char chaine[12];
    Int i;
    
    defDomain=NULL;
    setDefDomain(dom);
    
    nbrComponents=nbrCmp;
    if (nbrComponents!=0)
      componentName=new string[nbrComponents];
    else
      componentName=NULL;
    for (i=0;i<nbrComponents;i++) {
      sprintf(chaine,"%d",i);
      componentName[i]=chaine;
    }
    
    interpolationType=shape::interpolationType(interpType);
    
    nodalValuesNbr=0;
    nodalValuesNbrPC=0;
    nodalValues=NULL;
    objectNodalValuesIndexNbr=0;
    objectNodalValuesIndex=NULL;
    makeObjectNodalValuesIndex();
    
    currentObject=NULL;
    currentObjectDirect=0;
    
    localNodalValuesSize=0;
    localNodalValuesCapacity=0;
    mappingLocalGlobal=NULL;
    mappingLocalScal=NULL;
    localNodalValues=NULL;
    
    currentComponent=-1;
    minRefForRelativeVariation=1.0e-8;
    
    addToAllFields(this);
  }
  catchAll("field::field(domain* dom,Int nbrCmp,string interpType)","")
}



field::field(const field& FLD) : name() {
  try {
    Int i;
    
    defDomain=NULL;
    setDefDomain(FLD.getDefDomain());
    
    nbrComponents=FLD.nbrComponents;
    if (nbrComponents!=0) {
      componentName=new string[nbrComponents];
      for (i=0;i<nbrComponents;i++)
	componentName[i]=FLD.componentName[i];
    }
    else
      componentName=NULL;
    
    interpolationType=FLD.interpolationType;
    
    nodalValuesNbr=FLD.nodalValuesNbr;
    nodalValuesNbrPC=FLD.nodalValuesNbrPC;
    if (nodalValuesNbr!=0) {
      nodalValues=new Double[nodalValuesNbr];
      for (i=0;i<nodalValuesNbr;i++)
	nodalValues[i]=FLD.nodalValues[i];
    }
    else
      nodalValues=NULL;
    objectNodalValuesIndexNbr=FLD.objectNodalValuesIndexNbr;
    if (objectNodalValuesIndexNbr!=0) {
      objectNodalValuesIndex=new Int[objectNodalValuesIndexNbr];
      for (i=0;i<objectNodalValuesIndexNbr;i++)
	objectNodalValuesIndex[i]=FLD.objectNodalValuesIndex[i];
    }
    else
      objectNodalValuesIndex=NULL;
    
    currentObject=FLD.currentObject;
    currentObjectDirect=FLD.currentObjectDirect;
    
    localNodalValuesSize=FLD.localNodalValuesSize;
    localNodalValuesCapacity=FLD.localNodalValuesCapacity;
    if (localNodalValuesCapacity!=0) {
      mappingLocalGlobal=new Int[localNodalValuesCapacity];
      mappingLocalScal=new Int[localNodalValuesCapacity];
      localNodalValues=new Double[localNodalValuesCapacity];
      for (i=0;i<localNodalValuesCapacity;i++) {
	localNodalValues[i]=FLD.localNodalValues[i];
	mappingLocalGlobal[i]=FLD.mappingLocalGlobal[i];
	mappingLocalScal[i]=FLD.mappingLocalScal[i];
      }
    }
    else {
      mappingLocalGlobal=NULL;
      mappingLocalScal=NULL;
      localNodalValues=NULL;
    }
    
    currentComponent=-1;
    minRefForRelativeVariation=FLD.minRefForRelativeVariation;
    
    addToAllFields(this);
  }
  catchAll("field::field(const field& FLD)","")
}



field::~field(void) {
  try {
    setDefDomain(NULL);
    
    nbrComponents=0;
    if (componentName!=NULL) {
      delete[] componentName;
      componentName=NULL;
    }
    
    interpolationType=shape::interpolationType("none");
    
    nodalValuesNbr=0;
    nodalValuesNbrPC=0;
    if (nodalValues!=NULL) {
      delete[] nodalValues;
      nodalValues=NULL;
    }
    objectNodalValuesIndexNbr=0;
    if (objectNodalValuesIndex!=NULL) {
      delete[] objectNodalValuesIndex;
      objectNodalValuesIndex=NULL;
    }
    
    currentObject=NULL;
    currentObjectDirect=0;
    
    localNodalValuesSize=0;
    localNodalValuesCapacity=0;
    if (mappingLocalGlobal!=NULL) {
      delete[] mappingLocalGlobal;
      mappingLocalGlobal=NULL;
    }
    if (mappingLocalScal!=NULL) {
      delete[] mappingLocalScal;
      mappingLocalScal=NULL;
    }
    if (localNodalValues!=NULL) {
      delete[] localNodalValues;
      localNodalValues=NULL;
    }
    
    currentComponent=-1;
    
    removeFromAllFields(this);
  }
  catchAll("field::~field(void)","")
}



void field::operator=(const field& FLD) {
  if (this==&FLD)
    return;
  
  try {
    Int i;
    
    setDefDomain(FLD.getDefDomain());
    
    nbrComponents=FLD.nbrComponents;
    delete[] componentName;
    if (nbrComponents!=0) {
      componentName=new string[nbrComponents];
      for (i=0;i<nbrComponents;i++)
	componentName[i]=FLD.componentName[i];
    }
    else
      componentName=NULL;
    
    interpolationType=FLD.interpolationType;
    
    nodalValuesNbr=FLD.nodalValuesNbr;
    nodalValuesNbrPC=FLD.nodalValuesNbrPC;
    nodalValues=NULL;
    if (nodalValuesNbr!=0) {
      nodalValues=new Double[nodalValuesNbr];
      for (i=0;i<nodalValuesNbr;i++)
	nodalValues[i]=FLD.nodalValues[i];
    }
    else
      nodalValues=NULL;
    
    objectNodalValuesIndexNbr=FLD.objectNodalValuesIndexNbr;
    delete[] objectNodalValuesIndex;
    if (objectNodalValuesIndexNbr!=0) {
      objectNodalValuesIndex=new Int[objectNodalValuesIndexNbr];
      for (i=0;i<objectNodalValuesIndexNbr;i++)
	objectNodalValuesIndex[i]=FLD.objectNodalValuesIndex[i];
    }
    else
      objectNodalValuesIndex=NULL;
    
    currentObject=FLD.currentObject;
    currentObjectDirect=FLD.currentObjectDirect;
    
    localNodalValuesSize=FLD.localNodalValuesSize;
    localNodalValuesCapacity=FLD.localNodalValuesCapacity;
    delete[] mappingLocalGlobal;
    delete[] mappingLocalScal;
    delete[] localNodalValues;
    if (localNodalValuesCapacity!=0) {
      mappingLocalGlobal=new Int[localNodalValuesCapacity];
      mappingLocalScal=new Int[localNodalValuesCapacity];
      localNodalValues=new Double[localNodalValuesCapacity];
      for (i=0;i<localNodalValuesCapacity;i++) {
	localNodalValues[i]=FLD.localNodalValues[i];
	mappingLocalGlobal[i]=FLD.mappingLocalGlobal[i];
	mappingLocalScal[i]=FLD.mappingLocalScal[i];
      }
    }
    else {
      mappingLocalGlobal=NULL;
      mappingLocalScal=NULL;
      localNodalValues=NULL;
    }
    
    currentComponent=-1;
    minRefForRelativeVariation=FLD.minRefForRelativeVariation;
  }
  catchAll("void field::operator=(const field& FLD)","")
}



void field::setDefDomain(domain *dom) {
  try {
    delinkAndDelete(defDomain,&defDomain);
    defDomain=dom;
    if (defDomain!=NULL)
      defDomain->addParent(&defDomain);
  }
  catchAll("void field::setDefDomain(domain *dom)","")
}



domain* field::getDefDomain(void) const {
  return(defDomain);
}



void field::setNbrComponents(Int nbr) {
  try {
    char chaine[12];
    Int i;
    
    Int dim=0;
    if (defDomain!=NULL)
      dim=defDomain->getDim();
    
    nbrComponents=nbr;
    if (componentName!=NULL)
      delete[] componentName;
    if (nbrComponents!=0)
      componentName=new string[nbrComponents];
    else
      componentName=NULL;
    for (i=0;i<nbrComponents;i++) {
      sprintf(chaine,"%d",i);
      componentName[i]=chaine;
    }
    makeObjectNodalValuesIndex();
    
    currentComponent=-1;
  }
  catchAll("void field::setNbrComponents(Int nbr)","")
}



Int field::getNbrComponents(void) const {
  return(nbrComponents);
}



void field::setComponentName(Int index,string name) {
  try {
    if ((index<0)||(index>nbrComponents)) 
      errThrow("field::setComponentName",string("component index ")
	       +strFromInt(index)+" is out of range.");
    componentName[index]=name;
    
    currentComponent=-1;
  }
  catchAll("void field::setComponentName(Int index,string name)","")
}



string field::getComponentName(Int index) const {
  try {
    if ((index<0)||(index>nbrComponents))
      errThrow("field::getComponentName",string("component index ")
	       +strFromInt(index)+" is out of range.");
    return(componentName[index]);
  }
  catchAll("string field::getComponentName(Int index) const","")
}



void field::setInterpolationType(string s) {
  try {
    interpolationType=shape::interpolationType(s);
    makeObjectNodalValuesIndex();
    
    currentComponent=-1;
  }
  catchAll("void field::setInterpolationType(string s)","")
}



Int field::getInterpolationType(void) const {
  return(interpolationType);
}



string field::getInterpolationTypeName(void) const {
  try {
    return(shape::interpolationTypeName(interpolationType));
  }
  catchAll("string field::getInterpolationTypeName(void) const","")
}



void field::makeObjectNodalValuesIndex(void) {
  
//#warning field::makeObjectNodalValuesIndex should work with "PerElement"
  
  try {
    
    // objectNodalValuesIndex is an array containing integers. Those integers
    // are indices allowing to find in the list of nodal values, the one
    // corresponding to a particular object. It also counts the number
    // of nodalValues and allocates the corresponding array.
    
    Int i;
    
    if (defDomain==NULL)
      return;
    
    if (interpolationType==0||interpolationType==1) {
      objectNodalValuesIndexNbr=0;
      if (objectNodalValuesIndex!=NULL)
	delete[] objectNodalValuesIndex;
      
      nodalValuesNbr=nbrComponents;
      nodalValuesNbrPC=1;
      if (nodalValues!=NULL)
	delete[] nodalValues;
      nodalValues=NULL;
      if (nodalValuesNbr!=0) {
	nodalValues=new Double[nodalValuesNbr];
	for (i=0;i<nodalValuesNbr;i++)
	  nodalValues[i]=0.0;
      }
    }
    else {
      objectNodalValuesIndexNbr=defDomain->getAllObjectsNbr();
      if (objectNodalValuesIndex!=NULL)
	delete[] objectNodalValuesIndex;
      if (objectNodalValuesIndexNbr!=0)
	objectNodalValuesIndex=new Int[objectNodalValuesIndexNbr];
      else
	objectNodalValuesIndex=NULL;
      
      object *currentObject;
      Int i,totalIndex=0,nbr;
      for (i=0;i<objectNodalValuesIndexNbr;i++) {
	currentObject=defDomain->getAllObject(i);
	nbr=shape::getSpecificNVnbr(currentObject,interpolationType,defDomain);
	if (nbr!=0) {
	  objectNodalValuesIndex[i]=totalIndex;
	  totalIndex+=nbr;
	}
	else {
	  objectNodalValuesIndex[i]=-1;
	}      
      }
      
      nodalValuesNbr=nbrComponents*totalIndex;
      nodalValuesNbrPC=totalIndex;
      if (nodalValues!=NULL)
	delete[] nodalValues;
      nodalValues=NULL;
      if (nodalValuesNbr!=0) {
	nodalValues=new Double[nodalValuesNbr];
	for (i=0;i<nodalValuesNbr;i++)
	  nodalValues[i]=0.0;
      }
    }  
    currentComponent=-1;
  }
  catchAll("void field::makeObjectNodalValuesIndex(void)","")
}



Int field::getNodalValuesNbr(void) const {
  return(nodalValuesNbr);
}



Int field::getNodalValuesNbrPC(void) const {
  return(nodalValuesNbrPC);
}



Double*field::getNodalValues(void) const {
  return(nodalValues);
}



void field::setNodalValue(Int Comp,Int index,Double v) {
  try {
    nodalValues[Comp*nodalValuesNbrPC+index]=v;
    
    currentComponent=-1;
  }
  catchAll("void field::setNodalValue(Int Comp,Int index,Double v)","")
}



Double field::getNodalValue(Int Comp,Int index) const {
  try {
    return(nodalValues[Comp*nodalValuesNbrPC+index]);
  }
  catchAll("Double field::getNodalValue(Int Comp,Int index) const","")
}



void field::fillLocalNodalValues(object *obj,Int direct,
				 bool makeMappingLocalScal,
                                 bool fillInAnyCase) {
  try {
    Int i,j,k,nbrNV,nbrObj,index,counter,tmp1,tmp2;
    Int localNbrNVPerComponent,globalNbrNVPerComponent;
    object* listObj[MAX_OBJ_PER_ELT];
    Int listDir[MAX_OBJ_PER_ELT];
    Int permutation[MAX_OBJ_PER_ELT];
        
    if ((obj==currentObject)&&(direct==currentObjectDirect)
	&&(!fillInAnyCase))
      return;
    
    currentObject=obj;
    currentObjectDirect=direct;
    
    switch (interpolationType) {
      case none:
      case constantOnDomain:
      
        localNbrNVPerComponent=1;
        globalNbrNVPerComponent=nodalValuesNbrPC;
        localNodalValuesSize=nbrComponents;
        
        if (localNodalValuesSize>localNodalValuesCapacity) {
    	  localNodalValuesCapacity=localNodalValuesSize;
    	  if (localNodalValues!=NULL)
    	    delete[] localNodalValues;
    	  if (localNodalValuesCapacity!=0)
    	    localNodalValues=new Double[localNodalValuesCapacity];
    	  else
    	    localNodalValues=NULL;

    	  if (mappingLocalGlobal!=NULL)
    	    delete[] mappingLocalGlobal;
    	  if (mappingLocalScal!=NULL)
    	    delete[] mappingLocalScal;
    	  if (localNodalValuesCapacity!=0) {
    	    mappingLocalGlobal=new Int[localNodalValuesCapacity];
    	    mappingLocalScal=new Int[localNodalValuesCapacity];
    	  }
    	  else {
    	    mappingLocalGlobal=NULL;
    	    mappingLocalScal=NULL;
    	  }
        }
        
        for (i=0;i<nbrComponents;i++) {
    	  mappingLocalGlobal[i]=i;
    	  localNodalValues[i]=nodalValues[i];
    	  if (makeMappingLocalScal)
    	    mappingLocalScal[i]=scal::mapSize+i;
    	  else
    	    mappingLocalScal[i]=-1;
        }
      break;
	
      case linear:
      case quadratic:
      case Serendip:
      
        if (defDomain->getAllObject(obj)==-1)
    	  errThrow("field::fillLocalNodalValues",
    	  	   "object not in definition domain of the field.");
        
        nbrObj=obj->split(direct,listObj,listDir);
        
        localNbrNVPerComponent=0;
        for (i=0;i<nbrObj;i++)
    	  localNbrNVPerComponent+=shape::getSpecificNVnbr(listObj[i],
    	  						  interpolationType,
    	  						  defDomain);
        globalNbrNVPerComponent=nodalValuesNbrPC;
        
        localNodalValuesSize=localNbrNVPerComponent*nbrComponents;
        if (localNodalValuesSize>localNodalValuesCapacity) {
    	  localNodalValuesCapacity=localNodalValuesSize;
    	  if (localNodalValues!=NULL)
    	    delete[] localNodalValues;
    	  if (localNodalValuesCapacity!=0)
    	    localNodalValues=new Double[localNodalValuesCapacity];
    	  else
    	    localNodalValues=NULL;

    	  if (mappingLocalGlobal!=NULL)
    	    delete[] mappingLocalGlobal;
    	  if (mappingLocalScal!=NULL)
    	    delete[] mappingLocalScal;
    	  if (localNodalValuesCapacity!=0) {
    	    mappingLocalGlobal=new Int[localNodalValuesCapacity];
    	    mappingLocalScal=new Int[localNodalValuesCapacity];
    	  }
    	  else {
    	    mappingLocalGlobal=NULL;
    	    mappingLocalScal=NULL;
    	  }
        }
        
        counter=0;
        for (i=0;i<nbrObj;i++) {
    	  nbrNV=shape::getSpecificNVPermutation(listObj[i],interpolationType,
    	  					listDir[i],permutation,defDomain);
    	  index=objectNodalValuesIndex[defDomain->getAllObject(listObj[i])];
    	  for (j=0;j<nbrNV;j++)
    	    for (k=0;k<nbrComponents;k++) {
    	      tmp1=k*localNbrNVPerComponent+counter+j;
    	      tmp2=k*globalNbrNVPerComponent+index+permutation[j];
    	      // The mapping allows to retrieve a global index from a local index.
    	      mappingLocalGlobal[tmp1]=tmp2;
    	      localNodalValues[tmp1]=nodalValues[tmp2];
    	      if (makeMappingLocalScal)
    	        mappingLocalScal[tmp1]=scal::mapSize+tmp1;
    	      else
    	        mappingLocalScal[tmp1]=-1;
    	    }
    	  counter+=nbrNV;
        }
      break ;
      
      case constantPerElement:
      case linearPerElement:
      case quadraticPerElement:
      case SerendipPerElement:

    	localNbrNVPerComponent=shape::getSpecificNVnbr(obj,interpolationType,
    	  					       defDomain);
        globalNbrNVPerComponent=nodalValuesNbrPC;
	
        localNodalValuesSize=localNbrNVPerComponent*nbrComponents;
        if (localNodalValuesSize>localNodalValuesCapacity) {
    	  localNodalValuesCapacity=localNodalValuesSize;
    	  if (localNodalValues!=NULL)
    	    delete[] localNodalValues;
    	  if (localNodalValuesCapacity!=0)
    	    localNodalValues=new Double[localNodalValuesCapacity];
    	  else
    	    localNodalValues=NULL;

    	  if (mappingLocalGlobal!=NULL)
    	    delete[] mappingLocalGlobal;
    	  if (mappingLocalScal!=NULL)
    	    delete[] mappingLocalScal;
    	  if (localNodalValuesCapacity!=0) {
    	    mappingLocalGlobal=new Int[localNodalValuesCapacity];
    	    mappingLocalScal=new Int[localNodalValuesCapacity];
    	  }
    	  else {
    	    mappingLocalGlobal=NULL;
    	    mappingLocalScal=NULL;
    	  }
        }
        
    	nbrNV=shape::getSpecificNVPermutation(obj,interpolationType,
    					      direct,permutation,defDomain);
    	index=objectNodalValuesIndex[defDomain->getAllObject(obj)];
    	for (j=0;j<nbrNV;j++)
    	  for (k=0;k<nbrComponents;k++) {
    	    tmp1=k*localNbrNVPerComponent+j;
    	    tmp2=k*globalNbrNVPerComponent+index+permutation[j];
    	    // The mapping allows to retrieve a global index from a local index.
    	    mappingLocalGlobal[tmp1]=tmp2;
    	    localNodalValues[tmp1]=nodalValues[tmp2];
    	    if (makeMappingLocalScal)
    	      mappingLocalScal[tmp1]=scal::mapSize+tmp1;
    	    else
    	      mappingLocalScal[tmp1]=-1;
    	  }
      
      break ;
      
      default:
        errThrow("field::fillLocalNodalValues",
		 "unknown interpolation type"
		    +strFromInt(interpolationType)+".");
    } // switch (interpolationType)
    
    currentComponent=-1;
    
  }
  catchAll("field::fillLocalNodalValues","")
}



void field::interpolate(array *origArray,domain *interpDom,array *Coord) {
  try {
    Int i,j,k,l;
    
    Int nbrObj,objDirect,dim;
    object *obj;
    
    Int parentBidon,parentBidon2;
    origArray->addParent(&parentBidon);
    interpDom->addParent(&parentBidon);
    Coord->addParent(&parentBidon2);  
    
    nbrObj=interpDom->getObjectsNbr(); 
    for(i=0;i<nbrObj;i++) {
      obj=interpDom->getObject(i);
      if (defDomain->getAllObject(obj)==-1)
	errThrow("field::interpolate",string("domain ")+defDomain->getName()
		 +" not included in definition domain of this field.");
    }
    
    // Initialisation:
    
    Double *diag=new Double[nodalValuesNbrPC];
    Double *tmpValues=new Double[nodalValuesNbr];
    for (i=0;i<nodalValuesNbrPC;i++)
      diag[i]=0.0;
    for (i=0;i<nodalValuesNbr;i++)
      tmpValues[i]=0.0;
    
    // for shape functions:
    
    Double funct[MAX_OBJ_PER_ELT];
    Int corlocDeriv[3]={0,0,0};
    Int nbrFunct;
    
    // Construction of data needed for integration laws:
    
    dim=interpDom->getDim();
    
    globLoc *GL=NULL;
    if (dim==0) {
      if (Coord->getOrder()!=1)
        GL=build0D0DGlobLoc(Coord);
      else {
        int coordDim=Coord->getDim(0);
        if (coordDim==0)
          GL=build0D0DGlobLoc(Coord);
        else if (coordDim==1)
          GL=build1D0DGlobLoc(Coord);
        else if (coordDim==2)
          GL=build2D0DGlobLoc(Coord);
        else if (coordDim==3)
          GL=build3D0DGlobLoc(Coord);
	else
	  errThrow("field::interpolate",
	    "invalid dimension of the coordinates array.");
      }
    }
    else {
      if (Coord->getOrder()!=1)
	errThrow("field::interpolate",
	  "order of the coordinates array must be 1.");
      int coordDim=Coord->getDim(0);
      if (dim==1) {
        if (coordDim==1)
          GL=build1D1DGlobLoc(Coord);
        else if (coordDim==2)
          GL=build2D1DGlobLoc(Coord);
        else if (coordDim==3)
          GL=build3D1DGlobLoc(Coord);
	else
	  errThrow("field::interpolate",
	    "invalid dimension of the coordinates array.");
      }
      else if (dim==2) {
        if (coordDim==2)
          GL=build2D2DGlobLoc(Coord);
        else if (coordDim==3)
          GL=build3D2DGlobLoc(Coord);
	else
	  errThrow("field::interpolate",
	    "invalid dimension of the coordinates array.");
      }
      GL->addParent(&parentBidon); 
    } 
    
    integ ITG;
    if (interpolationType==linear
        ||interpolationType==linearPerElement)
      ITG=integ(GL,NULL,"nodal linear");
    else if (interpolationType==quadratic
             ||interpolationType==quadraticPerElement)
      ITG=integ(GL,NULL,"nodal quadratic");
    else if (interpolationType==Serendip
             ||interpolationType==SerendipPerElement) {
      errThrow("field::interpolate","does not work for Serendip interpolation.");
    }
    else
      ITG=integ(GL,NULL,"nodal constant per element");
       
    Double **integPoints,*integWeights;
    Int nbrPoints=0,nbrCoord=0;
    
    // Other data:
    
    scal *val;
    static scal factor;
    Double factorValue;
    
    // Loop on the objects of interpolation domain:
    for(i=0;i<nbrObj;i++) {
      obj=interpDom->getObject(i);
      objDirect=interpDom->getObjectDirect(i);
      ITG.getIntegPoints(obj->getType(),nbrPoints,nbrCoord,integPoints,integWeights);
      // Preparation of the Field:
      setNewPoint(true);
      fillLocalNodalValues(obj,objDirect,false,true);
      for (j=0;j<nbrPoints;j++) {
	// Preparation of the array:
	origArray->setNewPoint(true);
	origArray->makeAtPoint(obj,objDirect,integPoints[j],true,false);
	val=origArray->getComponentValues();
	// Preparation of shape functions:
        nbrFunct=shape::function(interpolationType,obj->getType(),defDomain,
				 integPoints[j],corlocDeriv,funct);
	if (nbrFunct!=localNodalValuesSize/nbrComponents)
	  errThrow("field::interpolate","nbrFunct!=localNodalValuesSize/nbrComponents");
	// Preparation of integration factor:
        factor=ITG.getIntegFactor(obj,objDirect,j,nbrPoints,nbrCoord,
				  integPoints,integWeights,true,false);
	factorValue=factor.getVal();
	// Loop on the nodal values:
	for (k=0;k<nbrFunct;k++) {
	  diag[mappingLocalGlobal[k]]+=funct[k]*factorValue;
	  // Loop on the components:
	  for (l=0;l<nbrComponents;l++) {
	    tmpValues[l*nodalValuesNbrPC+mappingLocalGlobal[k]]
	      +=funct[k]*factorValue*val[l].getVal();
	  } // for l
	} // for k
      } // for j
    } // for i
    
    for (k=0;k<nodalValuesNbrPC;k++)
      for (l=0;l<nbrComponents;l++)
	if (abs(diag[k])>1.0e-12) {
    	  nodalValues[l*nodalValuesNbrPC+k]=
	    tmpValues[l*nodalValuesNbrPC+k]/diag[k];
	}
    
    ITG.setGlobLoc(NULL);
    GL->supParent(&parentBidon);      
    currentComponent=-1;    
    delete GL;
    GL=NULL;
    delete[] tmpValues;
    delete[] diag;
    diag=NULL;
    
    origArray->supParent(&parentBidon);
    interpDom->supParent(&parentBidon);      
    Coord->supParent(&parentBidon2); 

    currentObject=NULL;
    currentObjectDirect=-1;
  }
  catchAll("void field::interpolate(array *origArray,domain *interpDom)","");
}



void field::interpolate(field *origField,domain *interpDom,field *Coord) {
  try {
    // This is a trick to avoid destruction of arguments at the
    // end of the function :
    Int parentBidon,parentBidon2;
    origField->addParent(&parentBidon);
    interpDom->addParent(&parentBidon);  
    Coord->addParent(&parentBidon2);  
    {
      domain *origDom;
      Int i;
      Int interpolationTypeOrig;
      
      if (nbrComponents!=origField->getNbrComponents())
	errThrow("field::interpolate",
		 "the number of components of the fields has to correspond.");
      
      if (Coord->getNbrComponents()<2)
	errThrow("field::interpolate","Coord must have two components.");
	
      Field FCoord;
      FCoord.setData(Coord);
      Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Coordinates",
		    DERIV=false,COMPONENT(0)=FCoord.component(0),
		    COMPONENT(1)=FCoord.component(1)));
       
      origDom=origField->getDefDomain();
      interpolationTypeOrig=origField->getInterpolationType();
      
      arrayField origArray;
      origArray.setField(origField);
      origArray.setOrder(1);
      origArray.setDim(0,nbrComponents);
      for (i=0;i<nbrComponents;i++)
	origArray.component(i)=origField->component(i);
      
      interpolate(&origArray,interpDom,ACoord.getData());
    }
    origField->supParent(&parentBidon);
    interpDom->supParent(&parentBidon);
    Coord->supParent(&parentBidon2);  
  }
  catchAll("void field::interpolate(field *origField,domain *interpDom)","")
}



void field::setCurrentComponent(Int nbr) {
  currentComponent=nbr;
}



field& field::component(Int index) {
  try {
    if (index>=nbrComponents)
      errThrow("field::component",string("the number of components is ")
	       +strFromInt(nbrComponents)+".");
    currentComponent=index;
    return(*this);
  }
  catchAll("field& field::component(Int index)","")
}



Int field::getCurrentComponent(void) const {
  return(currentComponent);
}



void field::setMinRefForRelativeVariation(Double value) {
  minRefForRelativeVariation=value;
}



Double field::getMinRefForRelativeVariation(void) const {
  return(minRefForRelativeVariation);
}



void field::printNodalValues(ostream &fOut) const {
  try {
    Int i,j;
    
    for (i=0;i<nodalValuesNbrPC;i++) {
      for (j=0;j<nbrComponents;j++)
	fOut << setw(20) << nodalValues[j*nodalValuesNbrPC+i] ;
      fOut << endl ;
    }
  }
  catchAll("void field::printNodalValues(ostream &fOut) const","")
}



void field::printCompNodalValues(ostream &fOut,Int comp) const {
  try {
    Int i;    
    for (i=0;i<nodalValuesNbrPC;i++) {
      fOut << setw(20) << nodalValues[comp*nodalValuesNbrPC+i] << endl ;
    }
  }
  catchAll("void field::printCompNodalValues(ostream &fOut,Int comp) const","")
}



void field::addToAllFields(field *f) {
  try {
    if (allFieldsSize==allFieldsCapacity) {
      field **tmpF=allFields;
      allFieldsCapacity*=2;
      if (allFieldsCapacity>0)
	allFields=new field* [allFieldsCapacity];
      else
	allFields=NULL;
      for (Int i=0;i<allFieldsSize;i++)
	allFields[i]=tmpF[i];
      delete[] tmpF;
    }
    allFields[allFieldsSize++]=f;
  }
  catchAll("void field::addToAllFields(field *f)","")
}



void field::removeFromAllFields(field *f) {
  try {
    for (Int i=0;i<allFieldsSize;i++) {
      if (allFields[i]==f) {
	for (Int j=i;j<allFieldsSize-1;j++)
	  allFields[j]=allFields[j+1];
	allFieldsSize--;
	return;
      }
    }
    errThrow("field::removeFromAllFields","field does not exist in the list.");
  }
  catchAll("void field::removeFromAllFields(field *f)","")
}



Int field::getAllFieldsSize(void) {
  return(allFieldsSize);
}



field* field::getAllField(Int index) {
  try {
    return(allFields[index]);
  }
  catchAll("field* field::getAllField(Int index)","")
}



Int field::getAllFieldIndex(field *F) {
  try {
    for (Int i=0;i<allFieldsCapacity;i++)
      if (allFields[i]==F)
	return(i);
    errThrow("field::getAllFieldIndex","unknown field.");
    return(-1);
  }
  catchAll("Int field::getAllFieldIndex(field *F)","")
}



void field::updateAllFieldsIndices(void) {
  try {
    Int index=0;
    for (Int i=0;i<allFieldsSize;i++) {
      allFields[i]->firstNodalValueIndex=index;
      index+=allFields[i]->nodalValuesNbr;
    }
    allNodalValuesSize=index;
  }
  catchAll("void field::updateAllFieldsIndices(void)","")
}



Int field::getAllNodalValuesSize(void) {
  return(allNodalValuesSize);
}



void field::writeInfoOnNodalVariable(Int nodalIndex) {
  try {
    field *F;
    Int localIndex,comp;
    for (Int i=0;i<allFieldsSize;i++) {
      F=allFields[i];
      if (nodalIndex>=F->firstNodalValueIndex
	  &&nodalIndex<F->firstNodalValueIndex+F->nodalValuesNbr) {
	localIndex=nodalIndex-F->firstNodalValueIndex;
	cerr << "         nodal value " << localIndex << " of field named \"" 
	  << F->getName() << "\"," << endl ;
	comp=localIndex/F->nodalValuesNbrPC;
	cerr << "         component " << comp << " named \"" 
	  << F->componentName[comp] 
	    << "\", index " << localIndex%F->nodalValuesNbrPC << "." << endl ;
      }
    }
    return;
  }
  catchAll("void field::writeInfoOnNodalVariable(Int nodalIndex)","")
}



Double field::getNodalVariableValue(Int nodalIndex) {
  try {
    field *F;
    Int localIndex;
    for (Int i=0;i<allFieldsSize;i++) {
      F=allFields[i];
      if (nodalIndex>=F->firstNodalValueIndex
	  &&nodalIndex<F->firstNodalValueIndex+F->nodalValuesNbr) {
	localIndex=nodalIndex-F->firstNodalValueIndex;
	return F->nodalValues[localIndex];
      }
    }
    errThrow("field::getNodalVariableValue","index not found!");
  }
  catchAll("void field::writeInfoOnNodalVariable(Int nodalIndex)","")
}



Double field::updateAllFieldsNodalValues(Double *delta,bool *activity,bool verbose) {
  try {
    Int index,totalIndex;
    Double totalVariation=0.0;
    field *F;
    bool active;
    for (Int i=0;i<allFieldsSize;i++) {
      F=allFields[i];
      index=F->firstNodalValueIndex;
      Double deltaMax=0.0;
      Double newValMax=0.0;
      Double oldValMax=0.0;
      active=false;
      for (Int j=0;j<F->nodalValuesNbr;j++) {
	totalIndex=index+j;
	if (activity[totalIndex]) {
	  oldValMax=max(oldValMax,abs(F->nodalValues[j]));
	  F->nodalValues[j]+=delta[totalIndex];
	  deltaMax=max(deltaMax,abs(delta[totalIndex]));
	  newValMax=max(newValMax,abs(F->nodalValues[j]));
	  active=true;
	}
      }
      if (active) {
        if (oldValMax>F->minRefForRelativeVariation) {
	  if (verbose)
	    cerr << "   Relative variation of field " << F->getName() << " :  "
	      << deltaMax/oldValMax << endl ;
	  totalVariation=max(totalVariation,deltaMax/oldValMax);
	}
	else {
	  newValMax=max(F->minRefForRelativeVariation,newValMax);
	  if (verbose)
	    cerr << "   Relative variation of field " << F->getName() << " :  "
	      << deltaMax/newValMax << endl ;
	  totalVariation=max(totalVariation,deltaMax/newValMax);
	}
      }
    }
    return(totalVariation);
  }
  catchAll("Double field::updateAllFieldsNodalValues(Double *delta,bool *activity)","")
}



void field::backupAllFieldsNodalValues(Double *backup) {
  try {
    Int i,j,index=0;
    field *F=NULL;
    for (i=0;i<allFieldsSize;i++) {
      F=allFields[i];
      for (j=0;j<F->nodalValuesNbr;j++)
	backup[index++]=F->nodalValues[j];
    }
  }
  catchAll("void field::backupAllFieldsNodalValues(Double *backup)","")
}



void field::restoreAllFieldsNodalValues(Double *backup) {
  try {
    Int i,j,index=0;
    field *F=NULL;
    for (i=0;i<allFieldsSize;i++) {
      F=allFields[i];
      for (j=0;j<F->nodalValuesNbr;j++)
	F->nodalValues[j]=backup[index++];
    }
  }
  catchAll("void field::restoreAllFieldsNodalValues(Double *backup)","")
}



void field::setAllNodalValues(Double d) {
  try {
    for (Int i=0;i<nodalValuesNbr;i++)
      nodalValues[i]=d;
  }
  catchAll("void field::setAllNodalValues(Double d)","")
}



void field::setAllNodalValues(Double d1,field &f1) {
  try {
    if (nodalValuesNbr!=f1.nodalValuesNbr)
      errThrow("field::setAllNodalValues",
	       "number of nodal values mismatch with first field argument.");
    for (Int i=0;i<nodalValuesNbr;i++)
      nodalValues[i]=d1*f1.nodalValues[i];
  }
  catchAll("void field::setAllNodalValues(Double d1,field &f1)","")
}



void field::setAllNodalValues(Double d1,field &f1,Double d2,field &f2) {
  try {
    if (nodalValuesNbr!=f1.nodalValuesNbr)
      errThrow("field::setAllNodalValues",
	       "number of nodal values mismatch with first field argument.");
    if (nodalValuesNbr!=f2.nodalValuesNbr)
      errThrow("field::setAllNodalValues",
	       "number of nodal values mismatch with second field argument.");
    for (Int i=0;i<nodalValuesNbr;i++)
      nodalValues[i]=d1*f1.nodalValues[i]+d2*f2.nodalValues[i];
  }
  catchAll("void field::setAllNodalValues(Double d1,field &f1,Double d2,field &f2)","")
}



void field::setAllNodalValues(Double d1,field &f1,Double d2,field &f2,
			      Double d3,field &f3) {
  try {
    if (nodalValuesNbr!=f1.nodalValuesNbr)
      errThrow("field::setAllNodalValues",
	       "number of nodal values mismatch with first field argument.");
    if (nodalValuesNbr!=f2.nodalValuesNbr)
      errThrow("field::setAllNodalValues",
	       "number of nodal values mismatch with second field argument.");
    if (nodalValuesNbr!=f3.nodalValuesNbr)
      errThrow("field::setAllNodalValues",
	       "number of nodal values mismatch with third field argument.");
    for (Int i=0;i<nodalValuesNbr;i++)
      nodalValues[i]=d1*f1.nodalValues[i]+d2*f2.nodalValues[i]
	+d3*f3.nodalValues[i];
  }
  catchAll("void field::setAllNodalValues(Double d1,field &f1,Double d2,field &f2,Double d3,field &f3)","")
}



printTree field::getPrintTreeData(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt;
    Int i;
    
    if (getName()=="")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    if (nbrComponents==0)
      sprintf(cbuf,"   no component");
    else if (nbrComponents==1)
      sprintf(cbuf,"   %d component =",nbrComponents);
    else
      sprintf(cbuf,"   %d components =",nbrComponents);
    sbuf=cbuf;
    if (nbrComponents>0) {
      sprintf(cbuf," %s",componentName[0].c_str());
      sbuf+=cbuf; 
      for (i=1;i<nbrComponents;i++) {
	sprintf(cbuf,", %s",componentName[i].c_str());
	sbuf+=cbuf;
      }
      pt+=sbuf;
    }
    
    sprintf(cbuf,"   interpolation type = %s",
	    shape::interpolationTypeName(interpolationType).c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    sprintf(cbuf,"   number of nodal values = %d",nodalValuesNbr);
    sbuf=cbuf;
    pt+=sbuf;
    
    return pt;
  }
  catchAll("printTree field::getPrintTreeData(void) const","")
}



printTree field::getPrintTreeValues(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt;
    Int i,j;
    Int nbrNVperComp;
    
    nbrNVperComp=nodalValuesNbr/nbrComponents;
    for (i=0;i<nbrNVperComp;i++) {
      sbuf="      ";
      sprintf(cbuf,"%6d",i);
      sbuf+=cbuf;
      for (j=0;j<nbrComponents;j++) {
	sprintf(cbuf,"%15g",nodalValues[j*nbrNVperComp+i]);
	sbuf+=cbuf;
      }
      pt+=sbuf;
    }
    return pt;
  }
  catchAll("printTree field::getPrintTreeValues(void) const","")
}



printTree field::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt;
    
    sprintf(cbuf,"field %p",static_cast<const void*>(this));
    sbuf=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    if (values)
      pt+=getPrintTreeValues();
      
    sbuf="   ";
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("printTree field::getPrintTree(bool values) const","")
}



printTree field::getPrintTreeSummary(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt;
    
    sprintf(cbuf,"field %p",static_cast<const void*>(this));
    sbuf=cbuf;
    pt+=sbuf;
    
    if (getName()=="")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    sbuf="   ";
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("field::getPrintTreeSummary","")
}






