#ifndef FILTER_BASE_H
#define FILTER_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>

#include "print/print.h"
#include "types/types.h"
#include "util/name.h"
#include "util/parents.h"

namespace keyFE2 {
  
  class array ;
  class domain ;
  class paramList ;
  
  class filter : public virtual name, public virtual parents,
		 public virtual print {
					
    friend class arrayFilter ;

    protected :
      
      bool lineType ;
      array *A ;
      domain *dom ;
      
      std::set<Int> blacks;
      
    public :
      
      filter(void) ;
      filter(paramList PL) ;
      filter(const filter &in) ;
      virtual ~filter(void) ;
      
      void operator=(const filter &in) ;
      
      void setLineType(bool b) ;
      bool getLineType(void) const ;
      void setArray(array *a) ;
      array* getArray(void) const ;
      void setDom(domain *d) ;
      domain* getDom(void) const ;
      
      void generateBlacks(void) ;
      void cleanBlacks(void) ;

      virtual printTree getPrintTreeData(void) const ;
      virtual printTree getPrintTreeValues(void) const ;
      virtual printTree getPrintTree(bool values) const ;

  } ;
  
}

#endif

