
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DCosh.h"
#include "DSinh.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DCosh::DCosh(Double coshfact,Double argfact) : function1() {
   CoshFact=coshfact;
   ArgFact=argfact;
}



DCosh::DCosh(const DCosh &in) : function1() {
   CoshFact=in.CoshFact;
   ArgFact=in.ArgFact;
}



DCosh::~DCosh(void) {
}



Double DCosh::value(Double arg) const {
   return(CoshFact*cosh(ArgFact*arg));
}



Double DCosh::deriv(Double arg) const {
   return(ArgFact*(CoshFact*sinh(ArgFact*arg)));
}



void DCosh::setCoshFact(Double x) {
   CoshFact=x;
}



Double DCosh::getCoshFact(void) const {
   return(CoshFact);
}



void DCosh::setArgFact(Double x) {
   ArgFact=x;
}



Double DCosh::getArgFact(void) const {
   return(ArgFact);
}



function1* DCosh::makeCopy(void) const {
   return(new DCosh(*this));
}



function1* DCosh::makeFunctionDeriv(void) const {
   if (CoshFact==0.0||ArgFact==0.0) 
      return(new DConst(0.0));
   else
      return(new DSinh(CoshFact*ArgFact,ArgFact));
}



string DCosh::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  
  if (getCoshFact()==1.0)
    cbuf[0]=0;
  else if (getCoshFact()==-1.0)
    sprintf(cbuf,"-");
  else
    sprintf(cbuf,"%g*",getCoshFact());
  sbuf=cbuf;
  
  sbuf+="cosh(";
  
  if (getArgFact()==1.0)
    sbuf+="";
  else if (getArgFact()==-1.0)
    sbuf+="-";
  else {
    sprintf(cbuf,"%g*",getArgFact());
    sbuf+=cbuf;
  }
  
  sprintf(cbuf,"arrayArg)");
  sbuf+=cbuf;
  
  return sbuf;
}


