
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DPowMult.h"
#include "DConst.h"
#include "DMult.h"

using namespace std ;
using namespace keyFE2 ;



DPowMult::DPowMult(void) : function1() {
  DParam1=0.0;
  DParam2=1.0;
}



DPowMult::DPowMult(Double factor,Double exponent) : function1() {
  DParam1=factor;
  DParam2=exponent;
}



DPowMult::DPowMult(const DPowMult &in) : function1() {
  DParam1=in.DParam1;
  DParam2=in.DParam2;
}



DPowMult::~DPowMult(void) {
}



Double DPowMult::value(Double arg) const {
  return(DParam1*pow(arg,DParam2));
}



Double DPowMult::deriv(Double arg) const {
  return(DParam1*DParam2*pow(arg,DParam2-1.0));
}



void DPowMult::setDParam1(Double x) {
  DParam1=x;
}



Double DPowMult::getDParam1(void) const {
  return(DParam1);
}



void DPowMult::setDParam2(Double x) {
  DParam2=x;
}



Double DPowMult::getDParam2(void) const {
  return(DParam2);
}



function1* DPowMult::makeCopy(void) const {
  return(new DPowMult(*this));
}



function1* DPowMult::makeFunctionDeriv(void) const {
  if (DParam2==1.0) {
    return(new DConst(DParam1));
  }
  else if (DParam2==2.0) {
    return(new DMult(2.0*DParam1));
  }
  else
    return(new DPowMult(DParam1*DParam2,DParam2-1.0));
}



string DPowMult::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"%g*arrayArg^%g",getDParam1(),getDParam2());
  return cbuf;
}
 

