
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DSinCos.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DSinCos::DSinCos(Double sinfact,Double cosfact,Double argfact) 
      : function1() {
   SinFact=sinfact;
   CosFact=cosfact;
   ArgFact=argfact;
}



DSinCos::DSinCos(const DSinCos &in) : function1() {
   SinFact=in.SinFact;
   CosFact=in.CosFact;
   ArgFact=in.ArgFact;
}



DSinCos::~DSinCos(void) {
}



Double DSinCos::value(Double arg) const {
   return(SinFact*sin(ArgFact*arg)+CosFact*cos(ArgFact*arg));
}



Double DSinCos::deriv(Double arg) const {
   return(ArgFact*(SinFact*cos(ArgFact*arg)-CosFact*sin(ArgFact*arg)));
}



void DSinCos::setSinFact(Double x) {
   SinFact=x;
}



Double DSinCos::getSinFact(void) const {
   return(SinFact);
}



void DSinCos::setCosFact(Double x) {
   CosFact=x;
}



Double DSinCos::getCosFact(void) const {
   return(CosFact);
}



void DSinCos::setArgFact(Double x) {
   ArgFact=x;
}



Double DSinCos::getArgFact(void) const {
   return(ArgFact);
}



function1* DSinCos::makeCopy(void) const {
   return(new DSinCos(*this));
}



function1* DSinCos::makeFunctionDeriv(void) const {
   if ((SinFact==0.0&&CosFact==0.0)||(ArgFact==0.0)) 
      return(new DConst(0.0));
   else
      return(new DSinCos(-CosFact*ArgFact,SinFact*ArgFact,ArgFact));
}



string DSinCos::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"%g*sin(%g*arrayArg)+(%g)*cos(%g*arrayArg)",
	  SinFact,ArgFact,CosFact,ArgFact);
  return cbuf;
}
 

