
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DSinh.h"
#include "DCosh.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DSinh::DSinh(Double sinhfact,Double argfact) : function1() {
   SinhFact=sinhfact;
   ArgFact=argfact;
}



DSinh::DSinh(const DSinh &in) : function1() {
   SinhFact=in.SinhFact;
   ArgFact=in.ArgFact;
}



DSinh::~DSinh(void) {
}



Double DSinh::value(Double arg) const {
   return(SinhFact*sinh(ArgFact*arg));
}



Double DSinh::deriv(Double arg) const {
   return(ArgFact*(SinhFact*cosh(ArgFact*arg)));
}



void DSinh::setSinhFact(Double x) {
   SinhFact=x;
}



Double DSinh::getSinhFact(void) const {
   return(SinhFact);
}



void DSinh::setArgFact(Double x) {
   ArgFact=x;
}



Double DSinh::getArgFact(void) const {
   return(ArgFact);
}



function1* DSinh::makeCopy(void) const {
   return(new DSinh(*this));
}



function1* DSinh::makeFunctionDeriv(void) const {
   if (SinhFact==0.0||ArgFact==0.0) 
      return(new DConst(0.0));
   else
      return(new DCosh(SinhFact*ArgFact,ArgFact));
}



string DSinh::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  
  if (getSinhFact()==1.0)
    cbuf[0]=0;
  else if (getSinhFact()==-1.0)
    sprintf(cbuf,"-");
  else
    sprintf(cbuf,"%g*",getSinhFact());
  sbuf=cbuf;
  
  sbuf+="sinh(";
  
  if (getArgFact()==1.0)
    sbuf+="";
  else if (getArgFact()==-1.0)
    sbuf+="-";
  else {
    sprintf(cbuf,"%g*",getArgFact());
    sbuf+=cbuf;
  }
  
  sprintf(cbuf,"arrayArg)");
  sbuf+=cbuf;
  
  return sbuf;
}


