
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DpwConst1.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DpwConst1::DpwConst1(void) : function1() {
  PA=0.0;
  X1=0.0;
  PB=1.0;
}



DpwConst1::DpwConst1(Double pa,Double x1,Double pb) : function1() {
  PA=pa;
  X1=x1;
  PB=pb;
}



DpwConst1::DpwConst1(const DpwConst1 &in) : function1() {
  PA=in.PA;
  X1=in.X1;
  PB=in.PB;
}



DpwConst1::~DpwConst1(void) {
  PA=0.0;
  X1=0.0;
  PB=1.0;
}



Double DpwConst1::value(Double arg) const {
   if (arg<X1)
      return PA;
   else
      return PB;
}



Double DpwConst1::deriv(Double arg) const {
   return 0.0;
}



void DpwConst1::setPA(Double x) {
  PA=x;
}



Double DpwConst1::getPA(void) const {
  return(PA);
}



void DpwConst1::setX1(Double x) {
  X1=x;
}



Double DpwConst1::getX1(void) const {
  return(X1);
}



void DpwConst1::setPB(Double x) {
  PB=x;
}



Double DpwConst1::getPB(void) const {
  return(PB);
}



function1* DpwConst1::makeCopy(void) const {
  return(new DpwConst1(*this));
}



function1* DpwConst1::makeFunctionDeriv(void) const {
  return(new DConst(0.0));
}



string DpwConst1::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"pwConst1(pa=%g,x1=%g,pb=%g,arrayArg)",
          PA,X1,PB);
  return cbuf;
}
 

