
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DpwConst3.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DpwConst3::DpwConst3(void) : function1() {
  PA=0.0;
  X1=0.0;
  P12=0.0;
  X2=0.0;
  P23=0.0;
  X3=0.0;
  PB=0.0;
}



DpwConst3::DpwConst3(Double pa,
                   Double x1,Double p12,
                   Double x2,Double p23,
                   Double x3,Double pb) : function1() {
  PA=pa;
  X1=x1;
  P12=p12;
  X2=x2;
  P23=p23;
  X3=x3;
  PB=pb;
}



DpwConst3::DpwConst3(const DpwConst3 &in) : function1() {
  PA=in.PA;
  X1=in.X1;
  P12=in.P12;
  X2=in.X2;
  P23=in.P23;
  X3=in.X3;
  PB=in.PB;
}



DpwConst3::~DpwConst3(void) {
  PA=0.0;
  X1=0.0;
  P12=0.0;
  X2=0.0;
  P23=0.0;
  X3=0.0;
  PB=0.0;
}



Double DpwConst3::value(Double arg) const {
   if (arg<X1)
      return PA;
   else if (arg<X2)
      return P12;
   else if (arg<X3)
      return P23;
   else
      return PB;
}



Double DpwConst3::deriv(Double arg) const {
   return 0.0;
}



void DpwConst3::setPA(Double x) {
  PA=x;
}



Double DpwConst3::getPA(void) const {
  return(PA);
}



void DpwConst3::setX1(Double x) {
  X1=x;
}



Double DpwConst3::getX1(void) const {
  return(X1);
}



void DpwConst3::setP12(Double x) {
  P12=x;
}



Double DpwConst3::getP12(void) const {
  return(P12);
}



void DpwConst3::setX2(Double x) {
  X2=x;
}



Double DpwConst3::getX2(void) const {
  return(X2);
}



void DpwConst3::setP23(Double x) {
  P23=x;
}



Double DpwConst3::getP23(void) const {
  return(P23);
}



void DpwConst3::setX3(Double x) {
  X3=x;
}



Double DpwConst3::getX3(void) const {
  return(X3);
}



void DpwConst3::setPB(Double x) {
  PB=x;
}



Double DpwConst3::getPB(void) const {
  return(PB);
}



function1* DpwConst3::makeCopy(void) const {
  return(new DpwConst3(*this));
}



function1* DpwConst3::makeFunctionDeriv(void) const {
    return(new DConst(0.0));

}



string DpwConst3::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"pwConst3(pa=%g,x1=%g,p12=%g,x2=%g,p23=%g,x3=%g,pb=%g,arrayArg)",
          PA,X1,P12,X2,P23,X3,PB);
  return cbuf;
}
 

