
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DpwLinear1.h"
#include "DpwConst1.h"

using namespace std ;
using namespace keyFE2 ;



DpwLinear1::DpwLinear1(void) : function1() {
  PA=0.0;
  X1=0.0;
  Y1=0.0;
  PB=1.0;
}



DpwLinear1::DpwLinear1(Double pa,Double x1,Double y1,Double pb) : function1() {
  PA=pa;
  X1=x1;
  Y1=y1;
  PB=pb;
}



DpwLinear1::DpwLinear1(const DpwLinear1 &in) : function1() {
  PA=in.PA;
  X1=in.X1;
  Y1=in.Y1;
  PB=in.PB;
}



DpwLinear1::~DpwLinear1(void) {
  PA=0.0;
  X1=0.0;
  Y1=0.0;
  PB=1.0;
}



Double DpwLinear1::value(Double arg) const {
   if (arg<X1)
      return Y1+PA*(arg-X1);
   else
      return Y1+PB*(arg-X1);
}



Double DpwLinear1::deriv(Double arg) const {
   if (arg<X1)
      return PA;
   else
      return PB;
}



void DpwLinear1::setPA(Double x) {
  PA=x;
}



Double DpwLinear1::getPA(void) const {
  return(PA);
}



void DpwLinear1::setX1(Double x) {
  X1=x;
}



Double DpwLinear1::getX1(void) const {
  return(X1);
}



void DpwLinear1::setY1(Double x) {
  Y1=x;
}



Double DpwLinear1::getY1(void) const {
  return(Y1);
}



void DpwLinear1::setPB(Double x) {
  PB=x;
}



Double DpwLinear1::getPB(void) const {
  return(PB);
}



function1* DpwLinear1::makeCopy(void) const {
  return(new DpwLinear1(*this));
}



function1* DpwLinear1::makeFunctionDeriv(void) const {
    return(new DpwConst1(PA,X1,PB));

}



string DpwLinear1::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"pwLinear1(pa=%g,x1=%g,y1=%g,pb=%g,arrayArg)",
          PA,X1,Y1,PB);
  return cbuf;
}
 

