
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DpwLinear2.h"
#include "DpwConst2.h"

using namespace std ;
using namespace keyFE2 ;



DpwLinear2::DpwLinear2(void) : function1() {
  PA=0.0;
  X1=0.0;
  Y1=0.0;
  X2=0.0;
  Y2=0.0;
  PB=0.0;
}



DpwLinear2::DpwLinear2(Double pa,
                     Double x1,Double y1,
                     Double x2,Double y2,
		     Double pb) : function1() {
  PA=pa;
  X1=x1;
  Y1=y1;
  X2=x2;
  Y2=y2;
  PB=pb;
}



DpwLinear2::DpwLinear2(const DpwLinear2 &in) : function1() {
  PA=in.PA;
  X1=in.X1;
  Y1=in.Y1;
  X2=in.X2;
  Y2=in.Y2;
  PB=in.PB;
}



DpwLinear2::~DpwLinear2(void) {
  PA=0.0;
  X1=0.0;
  Y1=0.0;
  X2=0.0;
  Y2=0.0;
  PB=1.0;
}



Double DpwLinear2::value(Double arg) const {
   if (arg<X1)
      return Y1+PA*(arg-X1);
   else if (arg<X2)
      return Y1+((Y2-Y1)/(X2-X1))*(arg-X1);
   else
      return Y2+PB*(arg-X2);
}



Double DpwLinear2::deriv(Double arg) const {
   if (arg<X1)
      return PA;
   else if (arg<X2)
      return (Y2-Y1)/(X2-X1);
   else
      return PB;
}



void DpwLinear2::setPA(Double x) {
  PA=x;
}



Double DpwLinear2::getPA(void) const {
  return(PA);
}



void DpwLinear2::setX1(Double x) {
  X1=x;
}



Double DpwLinear2::getX1(void) const {
  return(X1);
}



void DpwLinear2::setY1(Double x) {
  Y1=x;
}



Double DpwLinear2::getY1(void) const {
  return(Y1);
}



void DpwLinear2::setX2(Double x) {
  X2=x;
}



Double DpwLinear2::getX2(void) const {
  return(X2);
}



void DpwLinear2::setY2(Double x) {
  Y2=x;
}



Double DpwLinear2::getY2(void) const {
  return(Y2);
}



void DpwLinear2::setPB(Double x) {
  PB=x;
}



Double DpwLinear2::getPB(void) const {
  return(PB);
}



function1* DpwLinear2::makeCopy(void) const {
  return(new DpwLinear2(*this));
}



function1* DpwLinear2::makeFunctionDeriv(void) const {
    return(new DpwConst2(PA,
                         X1,(Y2-Y1)/(X2-X1),
			 X2,PB));
}



string DpwLinear2::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"pwLinear2(pa=%g,x1=%g,y1=%g,x2=%g,y2=%g,pb=%g,arrayArg)",
          PA,X1,Y1,X2,Y2,PB);
  return cbuf;
}
 

