
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DpwLinear4.h"
#include "DpwConst4.h"

using namespace std ;
using namespace keyFE2 ;



DpwLinear4::DpwLinear4(void) : function1() {
  PA=0.0;
  X1=0.0;
  Y1=0.0;
  X2=0.0;
  Y2=0.0;
  X3=0.0;
  Y3=0.0;
  X4=0.0;
  Y4=0.0;
  PB=0.0;
}



DpwLinear4::DpwLinear4(Double pa,
                     Double x1,Double y1,
                     Double x2,Double y2,
                     Double x3,Double y3,
                     Double x4,Double y4,
		     Double pb) : function1() {
  PA=pa;
  X1=x1;
  Y1=y1;
  X2=x2;
  Y2=y2;
  X3=x3;
  Y3=y3;
  X4=x4;
  Y4=y4;
  PB=pb;
}



DpwLinear4::DpwLinear4(const DpwLinear4 &in) : function1() {
  PA=in.PA;
  X1=in.X1;
  Y1=in.Y1;
  X2=in.X2;
  Y2=in.Y2;
  X3=in.X3;
  Y3=in.Y3;
  X4=in.X4;
  Y4=in.Y4;
  PB=in.PB;
}



DpwLinear4::~DpwLinear4(void) {
  PA=0.0;
  X1=0.0;
  Y1=0.0;
  X2=0.0;
  Y2=0.0;
  X3=0.0;
  Y3=0.0;
  X4=0.0;
  Y4=0.0;
  PB=1.0;
}



Double DpwLinear4::value(Double arg) const {
   if (arg<X1)
      return Y1+PA*(arg-X1);
   else if (arg<X2)
      return Y1+((Y2-Y1)/(X2-X1))*(arg-X1);
   else if (arg<X3)
      return Y2+((Y3-Y2)/(X3-X2))*(arg-X2);
   else if (arg<X4)
      return Y3+((Y4-Y3)/(X4-X3))*(arg-X3);
   else
      return Y4+PB*(arg-X4);
}



Double DpwLinear4::deriv(Double arg) const {
   if (arg<X1)
      return PA;
   else if (arg<X2)
      return (Y2-Y1)/(X2-X1);
   else if (arg<X3)
      return (Y3-Y2)/(X3-X2);
   else if (arg<X4)
      return (Y4-Y3)/(X4-X3);
   else
      return PB;
}



void DpwLinear4::setPA(Double x) {
  PA=x;
}



Double DpwLinear4::getPA(void) const {
  return(PA);
}



void DpwLinear4::setX1(Double x) {
  X1=x;
}



Double DpwLinear4::getX1(void) const {
  return(X1);
}



void DpwLinear4::setY1(Double x) {
  Y1=x;
}



Double DpwLinear4::getY1(void) const {
  return(Y1);
}



void DpwLinear4::setX2(Double x) {
  X2=x;
}



Double DpwLinear4::getX2(void) const {
  return(X2);
}



void DpwLinear4::setY2(Double x) {
  Y2=x;
}



Double DpwLinear4::getY2(void) const {
  return(Y2);
}



void DpwLinear4::setX3(Double x) {
  X3=x;
}



Double DpwLinear4::getX3(void) const {
  return(X3);
}



void DpwLinear4::setY3(Double x) {
  Y3=x;
}



Double DpwLinear4::getY3(void) const {
  return(Y3);
}



void DpwLinear4::setX4(Double x) {
  X4=x;
}



Double DpwLinear4::getX4(void) const {
  return(X4);
}



void DpwLinear4::setY4(Double x) {
  Y4=x;
}



Double DpwLinear4::getY4(void) const {
  return(Y4);
}



void DpwLinear4::setPB(Double x) {
  PB=x;
}



Double DpwLinear4::getPB(void) const {
  return(PB);
}



function1* DpwLinear4::makeCopy(void) const {
  return(new DpwLinear4(*this));
}



function1* DpwLinear4::makeFunctionDeriv(void) const {
    return(new DpwConst4(PA,
                         X1,(Y2-Y1)/(X2-X1),
			 X2,(Y3-Y2)/(X3-X2),
			 X3,(Y4-Y3)/(X4-X3),
			 X4,PB));
}



string DpwLinear4::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"pwLinear4(pa=%g,x1=%g,y1=%g,x2=%g,y2=%g,x3=%g,y3=%g,x4=%g,y4=%g,pb=%g,arrayArg)",
          PA,X1,Y1,X2,Y2,X3,Y3,X4,Y4,PB);
  return cbuf;
}
 

