
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <stdio.h>

#include "integ_base.h"
#include "array/array_base.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/tensorCoordSys_base.h"
#include "err/err.h"
#include "object/object_base.h"
#include "object/objectTypes.h"
#include "print/printTree.h"

using namespace std ;
using namespace keyFE2 ;



integ::integ(void) {
  GL=NULL;
  TCS=NULL;
  rule=-1;
}



integ::integ(globLoc *argGL,tensorCoordSys *argTCS,Int integRule) {
  try {
    GL=NULL;
    setGlobLoc(argGL);
    TCS=NULL;
    setTCS(argTCS);
    setRule(integRule);
  }
  catchAll("integ::integ(globLoc *argGL,tensorCoordSys *argTCS,Int integRule)","at the very end")
}



integ::integ(globLoc *argGL,tensorCoordSys *argTCS,string integRuleName) {
  try {
    GL=NULL;
    setGlobLoc(argGL);
    TCS=NULL;
    setTCS(argTCS);
    setRule(integRuleName);
  }
  catchAll("integ::integ(globLoc *argGL,tensorCoordSys *argTCS,string integRuleName","at the very end")
}



integ::integ(const integ &argInteg) {
  try {
    GL=NULL;
    setGlobLoc(argInteg.getGlobLoc());
    TCS=NULL;
    setTCS(argInteg.getTCS());
    setRule(argInteg.getRule());
  }
  catchAll("integ::integ(const integ &argInteg)","at the very end")
}



integ::~integ(void) {
  try {
    setGlobLoc(NULL);
    setTCS(NULL);
    setRule(-1);
  }
  catchAll("integ::~integ(void)","at the very end")
}



void integ::operator=(const integ &argInteg) {
  try {
    GL=NULL;
    setGlobLoc(argInteg.getGlobLoc());
    TCS=NULL;
    setTCS(argInteg.getTCS());
    setRule(argInteg.getRule());
  }
  catchAll("integ::operator=(const integ &argInteg)","at the very end");
}



void integ::setGlobLoc(globLoc *argGL) {
  try {
    delinkAndDelete(GL,&GL);
    GL=argGL;
    if (GL!=NULL)
      GL->addParent(&GL);
  }
  catchAll("void integ::setGlobLoc(globLoc *argGL)","at the very end")
}



globLoc* integ::getGlobLoc(void) const {
  return(GL);
}



void integ::setTCS(tensorCoordSys *argTCS) {
  try {
    delinkAndDelete(TCS,&TCS);
    TCS=argTCS;
    if (TCS!=NULL)
      TCS->addParent(&TCS);
  }
  catchAll("void integ::setTCS(tensorCoordSys *argTCS)","at the very end")
}



tensorCoordSys* integ::getTCS(void) const {
  return(TCS);
}



void integ::setRule(Int integRule) {
  rule=integRule;
}



void integ::setRule(string integRuleName) {
  if (integRuleName=="none")
    rule=0;
  else if (integRuleName=="order 1")
    rule=1;
  else if (integRuleName=="order 3")
    rule=2;
  else if (integRuleName=="order 5")
    rule=3;
  else if (integRuleName=="nodal constant per element")
    rule=100;
  else if (integRuleName=="nodal linear")
    rule=101;
  else if (integRuleName=="nodal quadratic")
    rule=102;  
  else 
    errThrow("integ::setRule","unkown integration rule : "+integRuleName+".");
}



Int integ::getRule(void) const {
  return(rule);
}



string integ::getRuleName(void) const {
  switch(rule) {
  case 0 :
    return("none");
    break;
  case 1 :
    return("order 1");
    break;
  case 2 :
    return("order 3");
    break;
  case 3 :
    return("order 5");
    break;
  case 100 :
    return("nodal constant per element");
    break;
  case 101 :
    return("nodal linear");
    break;
  case 102 :
    return("nodal quadratic");
    break;
  default :
    errThrow("integ::setRule",string("unkown integration rule : ")
	     +strFromInt(rule)+".");
  }
  return("unknown");
}




void integ::setNewPoint(bool b) {
  try {
    if (GL!=NULL) 
      GL->setNewPoint(b);
    if (TCS!=NULL)
      TCS->setNewPoint(b);
  }
  catchAll("void integ::setNewPoint(bool b)","at the very end")
}



void integ::accessNbrInit(void) {
  try {
    if (GL!=NULL) 
      GL->accessNbrInit();
    if (TCS!=NULL)
      TCS->accessNbrInit();
  }
  catchAll("void integ::accessNbrInit(void)","at the very end")
}



void integ::makeAtPoint(object *element,Int direct,Double *localCoord,
			bool makeValue,bool makeDeriv) {
  try {
    if (GL!=NULL) 
      GL->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
    if (TCS!=NULL)
      TCS->makeAtPoint(element,direct,localCoord,makeValue,makeDeriv);
  }
  catchAll("integ::makeAtPoint","at the very end")
}



void integ::getIntegPoints(Int objType,Int &nbrPoints,Int &nbrCoord,
                           Double** &integPoints,Double* &integWeights) {
  try {
    switch(objType) {
    case node :
      {
	static bool firstHere=true;
	static Double **points=NULL;
	static Double weights[1];
	nbrCoord=0;
	nbrPoints=1;
	if (firstHere) {
	  weights[0]=1.0;
	  static Double *tmp=new Double[1];
	  points=new Double* [1];
	  points[0]=tmp;
	  points[0][0]=0.0;
	}
	integPoints=points;
	integWeights=weights;
	firstHere=false;
      }
      break;
    case segment :
      nbrCoord=1;
      switch(rule) {
      case 1 :
      case 100 :
	{
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[1];
	  nbrPoints=1;
	  if (firstHere) {
	    weights[0]=2.0;
	    static Double *tmp=new Double[1];
	    points=new Double* [1];
	    points[0]=tmp;
	    points[0][0]=0.0;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 2 :
	{
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[2];
	  nbrPoints=2;
	  if (firstHere) {
	    weights[0]=1.0;
	    weights[1]=1.0;
	    static Double *tmp=new Double[2];
	    points=new Double* [2];
	    points[0]=tmp;
	    points[1]=tmp+1;
	    points[0][0]=-sqrt(3.0)/3.0;
	    points[1][0]=sqrt(3.0)/3.0;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 3 :
	{
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[3];
	  nbrPoints=3;
	  if (firstHere) {
	    weights[0]=5.0/9.0;
	    weights[1]=8.0/9.0;
	    weights[2]=5.0/9.0;
	    static Double *tmp=new Double[3];
	    points=new Double* [3];
	    points[0]=tmp;
	    points[1]=tmp+1;
	    points[2]=tmp+2;
	    points[0][0]=-sqrt(0.6);
	    points[1][0]=0.0;
	    points[2][0]=sqrt(0.6);
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 101 :
	{
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[2];
	  nbrPoints=2;
	  if (firstHere) {
	    weights[0]=0.5;
	    weights[1]=0.5;
	    static Double *tmp=new Double[2];
	    points=new Double* [2];
	    points[0]=tmp;
	    points[1]=tmp+1;
	    points[0][0]=-1.0;
	    points[1][0]=1.0;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 102 :
	{
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[3];
	  nbrPoints=3;
	  if (firstHere) {
	    weights[0]=1.0/3.0;
	    weights[1]=1.0/3.0;
	    weights[2]=1.0/3.0;
	    static Double *tmp=new Double[3];
	    points=new Double* [3];
	    points[0]=tmp;
	    points[1]=tmp+1;
	    points[2]=tmp+2;
	    points[0][0]=-1.0;
	    points[1][0]=0.0;
	    points[2][0]=1.0;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      default :
	errThrow("integ::getPoints",string("integration rule ")
		 +strFromInt(rule)+" not implemented for segments.");
      }
      break;
    case triangle :
      nbrCoord=2;
      switch(rule) {
      case 1 :
      case 100 :
	{
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[1];
	  nbrPoints=1;
	  if (firstHere) {
	    weights[0]=0.5;
	    static Double *tmp=new Double[2];
	    points=new Double* [1];
	    points[0]=tmp;
	    points[0][0]=1.0/3.0;
	    points[0][1]=1.0/3.0;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 2 :
	{
	  Int i;
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[3];
	  nbrPoints=3;
	  if (firstHere) {
	    static Double *tmp=new Double[6];
	    points=new Double* [3];
	    for (i=0;i<3;i++)
	      points[i]=tmp+2*i;
	    weights[0]=1.0/6.0;
	    weights[1]=1.0/6.0;
	    weights[2]=1.0/6.0;
	    points[0][0]=0.5;
	    points[0][1]=0.0;
	    points[1][0]=0.5;
	    points[1][1]=0.5;
	    points[2][0]=0.0;
	    points[2][1]=0.5;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 3 :
	{
	  Int i;
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[4];
	  nbrPoints=4;
	  if (firstHere) {
	    static Double *tmp=new Double[8];
	    points=new Double* [4];
	    for (i=0;i<4;i++)
	      points[i]=tmp+2*i;
	    weights[0]=-27.0/96.0;
	    weights[1]=25.0/96.0;
	    weights[2]=25.0/96.0;
	    weights[3]=25.0/96.0;
	    points[0][0]=1.0/3.0;
	    points[0][1]=1.0/3.0;
	    points[1][0]=0.2;
	    points[1][1]=0.2;
	    points[2][0]=0.6;
	    points[2][1]=0.2;
	    points[3][0]=0.2;
	    points[3][1]=0.6;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 101 :
	{
	  Int i;
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[3];
	  nbrPoints=3;
	  if (firstHere) {
	    static Double *tmp=new Double[6];
	    points=new Double* [3];
	    for (i=0;i<3;i++)
	      points[i]=tmp+2*i;
	    weights[0]=1.0/6.0;
	    weights[1]=1.0/6.0;
	    weights[2]=1.0/6.0;
	    points[0][0]=0.0;
	    points[0][1]=0.0;
	    points[1][0]=1.0;
	    points[1][1]=0.0;
	    points[2][0]=0.0;
	    points[2][1]=1.0;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      case 102 :
	{
	  Int i;
	  static bool firstHere=true;
	  static Double **points=NULL;
	  static Double weights[6];
	  nbrPoints=6;
	  if (firstHere) {
	    static Double *tmp=new Double[12];
	    points=new Double* [6];
	    for (i=0;i<6;i++)
	      points[i]=tmp+2*i;
	    weights[0]=1.0/12.0;
	    weights[1]=1.0/12.0;
	    weights[2]=1.0/12.0;
	    weights[3]=1.0/12.0;
	    weights[4]=1.0/12.0;
	    weights[5]=1.0/12.0;
	    points[0][0]=0.0;
	    points[0][1]=0.0;
	    points[1][0]=1.0;
	    points[1][1]=0.0;
	    points[2][0]=0.0;
	    points[2][1]=1.0;
	    points[3][0]=0.5;
	    points[3][1]=0.0;
	    points[4][0]=0.5;
	    points[4][1]=0.5;
	    points[5][0]=0.0;
	    points[5][1]=0.5;
	  }
	  integPoints=points;
	  integWeights=weights;
	  firstHere=false;
	}
	break;
      default :
	errThrow("integ::getPoints",string("integration rule ")
		 +strFromInt(rule)+" not implemented for triangles.");
	break;
      }
      break;
    case quadrangle :
      nbrCoord=2;
      switch(rule) {
      case 1 :
	{
#include "integQuadrangleFromSegment.h"
	}
	break;
      case 2 :
	{
#include "integQuadrangleFromSegment.h"
	}
	break;
      case 3 :
	{
#include "integQuadrangleFromSegment.h"
	}
	break;
      case 100 :
	{
#include "integQuadrangleFromSegment.h"
	}
	break;
      case 101 :
	{
#include "integQuadrangleFromSegment.h"
	}
	break;
      case 102 :
	{
#include "integQuadrangleFromSegment.h"
	}
	break;
      default :
	errThrow("integ::getPoints",string("integration rule ")
		 +strFromInt(rule)+" not implemented for quadrangles.");
      }
      break;
    }
  }
  catchAll("integ::getIntegPoints","at the very end");
}



scal& integ::getIntegFactor(object *element,int direct,Int pointIndex,
                            Int nbrPoInts,Int nbrCoord,Double** integPoints,
                            Double* integWeights,bool makeValue,bool makeDeriv) {
  try {
    static scal outScal;
    static scal integFactor,tmpDetJac;
    
    if (GL!=NULL)
      GL->makeAtPoint(element,direct,integPoints[pointIndex],makeValue,makeDeriv);
    if (TCS!=NULL)
      TCS->makeAtPoint(element,direct,integPoints[pointIndex],makeValue,makeDeriv);
    
    if (GL!=NULL)
      tmpDetJac=GL->getDetJac();
    else
      tmpDetJac=1.0;
    
    if (TCS!=NULL&&TCS->getType()==4) {
      const static Double twoPi=2.0*3.14159265358979323846;
      scal *coordCV=TCS->getCoord()->getComponentValues();
      outScal=coordCV[0]*twoPi*tmpDetJac;
      outScal*=integWeights[pointIndex];
    }
    else
      outScal=tmpDetJac*integWeights[pointIndex];

    return(outScal);
  }
  catchAll("integ::getIntegFactor","at the very end")
}



printTree integ::getPrintTreeData(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt;
  
    if (getName()=="at the very end")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    sbuf="   rule = ";
    sbuf+=getRuleName();
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("integ::getPrintTreeData","at the very end")
}



printTree integ::getPrintTreeValues(void) const { 
  try {
    printTree pt;
    return pt;
  }
  catchAll("integ::getPrintTreeValues","at the very end")
}



printTree integ::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt,ptArg;
    
    sprintf(cbuf,"integ %p",static_cast<const void*>(this));
    sbuf=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    sbuf="   ";
    pt+=sbuf;
    
    pt.prepareForArguments();  
    
    if (GL!=NULL) {
      ptArg=GL->getFirstPrintTree("GL",values);
    }
    else {
      sbuf="undefined GL";
      ptArg=sbuf;
      sbuf="   ";
      ptArg+="sbuf";
    }
    ptArg.prepareForAdd();
    pt+=ptArg;
    
    if (TCS!=NULL) {
      ptArg=TCS->getFirstPrintTree("TCS",values);
    }
    else {
      sbuf="undefined GL";
      ptArg=sbuf;
      sbuf="   ";
      ptArg+="sbuf";
    }
    ptArg.prepareForAddLast();
    pt+=ptArg;
    
    return(pt);
  }
  catchAll("integ::getPrintTree","at the very end")
}



