#ifndef MESH_H
#define MESH_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>
#include <iostream>
#include <fstream>

#include "types/types.h"

namespace keyFE2 {
  
  class Array ;
  class Domain ;
  class Field ;
  class mesh ;
  
  class Mesh {
    
    private :
      
      void *data;
    
    public :
      
      Mesh(void) ;
      Mesh(const Mesh &M) ;
      ~Mesh(void) ;
      
      // mesh interface :
      
      void setData(mesh *m) ;
      mesh* getData(void) const ;
      void setName(std::string str) ;
      std::string getName(void)const ;
      
      void operator=(const Mesh &M) ;
      
      void read(std::string fileName,std::string typeMesh) ;
      void write(std::string fileName,std::string typeMesh) ;
      
      Int getDomainsNbr(void) ;
      Domain getDomain(Int) ;
      Domain getDomainFromIntId(Int) ;
      Int getDomainIntId(Int) ;
      void writeDomain(std::ofstream &outFile,Domain &Dom) ;
      void writeDomain(std::string fileName,Domain &Dom) ;
      Domain getDefDomain(void) ;
      
      Field getCoordinateField(Domain &Dom) ;
      
      void write(std::ostream &os) const ;
      void write(void) const ;      
      void writeValues(std::ostream &os) const ;
      void writeValues(void) const ;

  } ;
  
  Mesh renumberDefDomainFrontal(const Mesh &M,Domain D) ;
  Mesh renumberDefDomainArray(const Mesh &M,Array A) ;
  
}

#endif
