
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>

#include "mesh_base.h"
#include "meshTypes.h"
#include "domain/Domain.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "field/Field.h"
#include "field/field_base.h"
#include "object/object_base.h"
#include "object/objectTypes.h"

using namespace std ;
using namespace keyFE2 ;



Field mesh::getCoordinateField(Domain &Dom) const {
  try {
    field current;
    Field Current;
    Int i,index;
    object *obj=NULL;
    
    domain *dom=Dom.getData();
    
    if (type==rsMesh||type==emc2Mesh||type==gmsh2dMesh) {
      current.setDefDomain(dom);
      current.setNbrComponents(2);
      current.setInterpolationType("linear");
      current.setComponentName(0,"X");
      current.setComponentName(1,"Y");
      
      current.makeObjectNodalValuesIndex();
      
      Int nbrObjects=dom->getAllObjectsNbr();
      for (i=0;i<nbrObjects;i++) {
	obj=dom->getAllObject(i);
	if (obj->getType()==node) {
	  index=obj-objectsLst;         
	  current.setNodalValue(0,i,coord[index][0]);
	  current.setNodalValue(1,i,coord[index][1]);
	}
      }
    }
    else
      errThrow("mesh::getCoordinateField",
	       "   unknown mesh type : "+strFromInt(type)+".");
    
    field *outField=new field(current);
    Current.setData(outField);
    return(Current);
  }
  catchAll("Field mesh::getCoordinateField(Domain &Dom) const","")
}



field* mesh::getCoordinateField(domain *dom) const {
  try {
    field* current;
    Int i,index;
    object *obj=NULL;
    
    current=new field;
    
    if (type==rsMesh||type==emc2Mesh||type==gmsh2dMesh) {
      current->setDefDomain(dom);
      current->setNbrComponents(2);
      current->setInterpolationType("linear");
      current->setComponentName(0,"X");
      current->setComponentName(1,"Y");
      
      current->makeObjectNodalValuesIndex();
      
      Int nbrObjects=dom->getAllObjectsNbr();
      for (i=0;i<nbrObjects;i++) {
	obj=dom->getAllObject(i);
	if (obj->getType()==node) {
	  index=obj-objectsLst;         
	  current->setNodalValue(0,i,coord[index][0]);
	  current->setNodalValue(1,i,coord[index][1]);
	}
      }
    }
    else
      errThrow("mesh::getCoordinateField",
	       "   unknown mesh type : "+strFromInt(type)+".");
    
    return(current);
  }
  catchAll("field* mesh::getCoordinateField(domain *dom) const","")
}










