
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <iostream>

#include "mesh_base.h"
#include "domain/Domain.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "object/object_base.h"

using namespace std ;
using namespace keyFE2 ;



void mesh::read(string fileName,string typeMesh) {
  try {
    ifstream inFile(fileName.c_str());
    if (!inFile)
      errThrow("mesh::read",
	       "   impossible to open mesh file \""+fileName+"\".");
    
    nodesNbr=0;
    if (coord!=NULL) {
      delete[] coord;
      coord=NULL;
    }
    if (objectsLst!=NULL) {
      delete[] objectsLst;
      objectsLst=NULL;
    }
    if (domainsLst!=NULL) {
      delete[] domainsLst;
      domainsLst=NULL;
    }
    if (domainsIntId!=NULL) {
      delete[] domainsIntId;
      domainsIntId=NULL;
    }
    
    if (typeMesh=="rsMesh") {
      this->read_rsMesh(inFile);
    }
    else if (typeMesh=="2DMesh") {
      this->read_2DMesh(inFile);
    }
    else if (typeMesh=="emc2Mesh") {
      this->read_emc2Mesh(inFile);
    }
    else if (typeMesh=="gmsh2dMesh") {
      this->read_gmsh2dMesh(inFile);
    }
    else
      errThrow("mesh::read","   unknown mesh type \""+typeMesh+"\".");
    
    for (Int i=0;i<domainsNbr;i++) {
      domainsLst[i].setDim();
      domainsLst[i].makeAllObjectsLst();
      domainsLst[i].setBaseMesh(this);
    }
    
    inFile.close();
  }
  catchAll("void mesh::read(string fileName,string typeMesh)","")
}



void mesh::write(string fileName,string typeMesh) const {
  try {
    ofstream outFile(fileName.c_str());
    if (!outFile)
      errThrow("mesh::write",
	       "   impossible to open mesh file \""+fileName+"\".");
    
    if (typeMesh=="rsMesh") {
      write_rsMesh(outFile);
    }
    else if (typeMesh=="2DMesh") {
      write_2DMesh(outFile);
    }
    else if (typeMesh=="emc2Mesh") {
      write_emc2Mesh(outFile);
    }
    else if (typeMesh=="gmsh2dMesh") {
      write_gmsh2dMesh(outFile);
    }
    else
      errThrow("mesh::write","   unknown mesh type "+typeMesh+".");
    
    outFile.close();
  }
  catchAll("void mesh::write(string fileName,string typeMesh) const","")
}



void mesh::writeDomain(ostream &myCout,Domain &Dom) const {
  try {
    domain *dom=Dom.getData();
    writeDomain(myCout,*dom);
  }
  catchAll("void mesh::writeDomain(ostream &myCout,Domain &Dom) const","")
}



void mesh::writeDomain(string fileName,Domain &Dom) const {
  try {
    domain *dom=Dom.getData();
    writeDomain(fileName,*dom);
  }
  catchAll("void mesh::writeDomain(string fileName,Domain &Dom) const","")
}



void mesh::writeDomain(ostream &myCout,domain &dom) const {
  try {
    myCout << setw(6) << dom.getObjectsNbr() << endl ;
    if (!myCout) 
      errThrow("mesh::writeDomain","   error in writing number of objects.");
    
    if (dom.getObjectsNbr()!=0) {
      for (Int i=0;i<dom.getObjectsNbr();i++) {
	if (i!=0&&i%10==0) {
	  myCout << endl ;
	  if (!myCout)
	    errThrow("mesh::writeDomain","   error in writing end of line.");
	}
	if (dom.getObjectDirect(i)>=0)
	  // Ici, il faudra introduire une modification pour tenir compte des 
	  //    dcalages possibles :
	  myCout << setw(6) << (dom.getObject(i)+1)-objectsLst ;
	else
	  myCout << setw(6) << objectsLst-(dom.getObject(i)+1) ;
	if (!myCout)
	  errThrow("mesh::writeDomain",
		   "   error in writing object "+strFromInt(i+1)+".");
      }
      
      myCout << endl ;
      if (!myCout)
	errThrow("mesh::writeDomain","   error in writing end of line.");
    }
  }catchAll("void mesh::writeDomain(ostream &myCout,domain &dom) const","")
}



void mesh::writeDomain(string fileName,domain &dom) const {
  try {
    ofstream outFile(fileName.c_str());
    if (!outFile)
      errThrow("mesh::writeDomain",
	       "   error in opening output file "+fileName+".");
    writeDomain(outFile,dom);
    outFile.close();
  }
  catchAll("void mesh::writeDomain(string fileName,domain &dom) const","")
}




