
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <map>
#include <set>

#include "mesh_base.h"
#include "meshTypes.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "object/object_base.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;



void mesh::read_2DMesh(istream &inFile) {
  errThrow("mesh::read_2DMesh","   not yet implemented.");
}



void mesh::write_2DMesh(ostream &outFile) const {
  try {
    Int segmentsNbr=0,elementsNbr=0;
    string objectType;
    for (Int i=0;i<objectsNbr;i++) {
      objectType=objectsLst[i].getTypeName();
      if (objectType=="segment")
	segmentsNbr++;
      else if (objectType=="triangle"||objectType=="quadrangle")
	elementsNbr++;
    }
    
    // Writing of preliminary data :
    
    outFile << setw(6) << nodesNbr << setw(6) << segmentsNbr
      << setw(6) << elementsNbr << endl ;
    if (!outFile)
      errThrow("mesh::write","   error in writing first line.");
    
    // Writing of nodes :
    
    outFile.setf(ios::scientific);
    outFile << setprecision(7);
    for (Int i=0;i<nodesNbr;i++) {
      outFile << setw(6) << i+1 << setw(6) << 1 << setw(6) << 3 ;
      if (!outFile)
	errThrow("mesh::write",
		 string("   error in writing node ")+strFromInt(i+1)+".");
      for (Int j=0;j<3;j++) {
	outFile << setw(16) << coord[i][j] ;
	if (!outFile) 
	  errThrow("mesh::write",
		   string("   error in writing node ")+strFromInt(i+1)+".");
      }
      outFile << endl ;
      if (!outFile)
	errThrow("mesh::write",
		 string("   error in writing node ")+strFromInt(i+1)+".");
    }
    
    // Writing of other elements (segments, triangles, quadrangles) :
    
    Int nbr=segmentsNbr+elementsNbr;
    for (Int i=0;i<nbr;i++) {
      outFile << setw(6) << nodesNbr+i+1 ;
      if (!outFile) 
	errThrow("mesh::write",
		 string("   error in writing object ")+strFromInt(i+1)+".");
      object *obj=objectsLst+nodesNbr;
      outFile << setw(6) << obj[i].getType() ;
      if (!outFile) 
	errThrow("mesh::write",
		 string("   error in writing object ")+strFromInt(i+1)+".");
      outFile << setw(6) << obj[i].getMinusNbr() ;
      for (Int j=0;j<obj[i].getMinusNbr();j++) {
	// Ici, il faudra introduire une modification pour tenir compte des dcalages
	// possibles :
	if (obj[i].getMinusDirect(j)>=0)
	  outFile << setw(6) << (obj[i].getMinus(j)+1)-objectsLst ;
	else
	  outFile << setw(6) << objectsLst-(obj[i].getMinus(j)+1) ;
	if (!outFile) 
	  errThrow("mesh::write",
		   string("   error in writing object ")+strFromInt(i+1)+".");
      }
      outFile << endl ;
      if (!outFile) 
	errThrow("mesh::write",
		 string("   error in writing object ")+strFromInt(i+1)+".");
    }
    
    outFile << setw(6) << domainsNbr << endl ;
    if (!outFile)
      errThrow("mesh::write",
	       "   error in writing number of domains.");
    for (Int i=0;i<domainsNbr;i++) {
      outFile << setw(6) << domainsIntId[i] ;
      if (!outFile)
	errThrow("mesh::write",
		 "   error in writing identifier of domain "
		 +strFromInt(i)+".");
      writeDomain(outFile,domainsLst[i]);
    }
  }
  catchAll("void mesh::write_2DMesh(ostream &outFile) const","")
}



