
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iomanip>
#include <iostream>

#include "mesh_base.h"
#include "meshTypes.h"
#include "err/err.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;



#define MAX_ILIST_SIZE 10



mesh::intList::intList(void) {
   size=0;
   for (int i=0;i<MAX_ILIST_SIZE;i++)
      ilist[i]=-1;
}



mesh::intList::intList(const mesh::intList &in) {
   size=in.size;
   for (int i=0;i<size;i++)
      ilist[i]=in.ilist[i];
}



mesh::intList::~intList(void) {
   size=0;
   for (int i=0;i<MAX_ILIST_SIZE;i++)
      ilist[i]=-1;
}



void mesh::intList::setSize(Int nbr) {
   size=nbr;
   if (size<0||size>MAX_ILIST_SIZE)
      errThrow("mesh::intList::setSize","invalid size: "+strFromInt(nbr)+".");
}



const mesh::intList& mesh::intList::operator=(const mesh::intList &in) {
   size=in.size;
   for (int i=0;i<size;i++)
      ilist[i]=in.ilist[i];
   return(*this);
}



bool mesh::intList::operator==(const mesh::intList &in) const {
   if (size!=in.size)
      return false;
   for (int i=0;i<size;i++)
      if (ilist[i]!=in.ilist[i])
         return false;
   return true;
}



bool mesh::intList::operator!=(const mesh::intList &in) const {
   if (size!=in.size)
      return true;
   for (int i=0;i<size;i++)
      if (ilist[i]!=in.ilist[i])
         return true;
   return false;
}



bool mesh::intList::operator<=(const mesh::intList &in) const {
   return((*this)<in||(*this)==in);
}



bool mesh::intList::operator>=(const mesh::intList &in) const {
   return((*this)>in||(*this)==in);
}



bool mesh::intList::operator<(const mesh::intList &in) const {
   if (size<in.size)
      return true;
   if (size>in.size)
      return false;
   for (int i=0;i<size;i++)
      if (ilist[i]<in.ilist[i])
         return true;
      else if (ilist[i]>in.ilist[i])
         return false;
   return false;
}



bool mesh::intList::operator>(const mesh::intList &in) const {
   if (size>in.size)
      return true;
   if (size<in.size)
      return false;
   for (int i=0;i<size;i++)
      if (ilist[i]>in.ilist[i])
         return true;
      else if (ilist[i]<in.ilist[i])
         return false;
   return false;
}



void mesh::intList::write(void) const {
   cerr << size << " : " ;
   for (int i=0;i<size;i++)
      cerr << " " << ilist[i] ;
   cerr << endl ;
}
