
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "param.h"
#include "paramList.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "coordSys/GlobLoc.h"
#include "coordSys/TensorCoordSys.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/tensorCoordSys_base.h"
#include "constr/Constr.h"
#include "constr/constr_base.h"
#include "domain/Domain.h"
#include "domain/domain_base.h"
#include "field/Field.h"
#include "field/field_base.h"
#include "filter/Filter.h"
#include "filter/filter_base.h"
#include "function/function1.h"
#include "integ/Integ.h"
#include "integ/integ_base.h"
#include "mesh/Mesh.h"
#include "mesh/mesh_base.h"
#include "problem/Problem.h"
#include "problem/problem_base.h"
#include "result/Result.h"
#include "result/result_base.h"
#include "timeDep/TimeDep.h"
#include "timeDep/timeDep_base.h"
#include "util/parents.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  const param ACTIVE("ACTIVE");
  const param ARRAY("ARRAY");
  const param AUTOFIX("AUTOFIX");
  const param COMPONENT("COMPONENT");
  const param COMPONENTNAME("COMPONENTNAME");
  const param CONSTR("CONSTR");
  const param CONVERGENCE("CONVERGENCE");
  const param COORD("COORD");
  const param DELAY("DELAY");
  const param DERIV("DERIV");
  const param DIM("DIM");
  const param DIVERGENCE("DIVERGENCE");
  const param DOM("DOM");
  const param EPSILON("EPSILON");
  const param EQUATION("EQUATION");
  const param EXPR("EXPR");
  const param FIELD("FIELD");
  const param FILTER("FILTER");
  const param GLTYPE("GLTYPE");
  const param GLOBLOC("GLOBLOC");
  const param INTERPOLATION("INTERPOLATION");
  const param INTERPMESH("INTERPMESH");
  const param LAYER("LAYER");
  const param LEFTDOM("LEFTDOM");
  const param LEFTEQUATION("LEFTEQUATION");
  const param LEFTTEST("LEFTTEST");
  const param LINETYPE("LINETYPE");
  const param RIGHTDOM("RIGHTDOM");
  const param RIGHTEQUATION("RIGHTEQUATION");
  const param RIGHTTEST("RIGHTTEST");
  const param NAME("NAME");
  const param NITMAX("NITMAX");
  const param NBRCOMPONENTS("NBRCOMPONENTS");
  const param ORDER("ORDER");
  const param PERMUT("PERMUT");
  const param PROBLEM("PROBLEM");
  const param RESULT("RESULT");
  const param RULE("RULE");
  const param RULENAME("RULENAME");
  const param TCSTYPE("TCSTYPE");
  const param TENSORCOORDSYS("TENSORCOORDSYS");
  const param TEST("TEST");
  const param TYPE("TYPE");
  const param SCHEME("SCHEME");
  const param MAXSTEPSNBR("MAXSTEPSNBR");
  const param DTMIN("DTMIN");
  const param DTMAX("DTMAX");
  const param DTINIT("DTINIT");
  const param TINIT("TINIT");
  const param TFINAL("TFINAL");
  const param INFLU("INFLU");
  const param SILENT("SILENT");
  const param MINREF("MINREF");
  const param ZERO("ZERO");
  
}

using namespace keyFE2 ;



param::param(void) : name() {
  type=0;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=NULL;
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const param &pin) : name(pin) {
  type=pin.type;
  
  intData=pin.intData;
  doubleData=pin.doubleData;
  stringData=pin.stringData;
  boolData=pin.boolData;
  parentData=NULL;
  setParentData(pin.parentData);
  function1Data=pin.function1Data;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=pin.components[i];
}



param::param(const string &s) : name(s) {
  type=0;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=NULL;
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Int &iParam) : name(s) {
  type = 1;
  
  intData=iParam;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=NULL;
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Double &dParam) : name(s) {
  type=2;
  
  intData=-1;
  doubleData=dParam;
  stringData="";
  boolData=false;
  parentData=NULL;
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const string &sParam) : name(s) {
  type=3;
  
  intData=-1;
  doubleData=0.0;
  stringData=sParam;
  boolData=false;
  parentData=NULL;
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const char* cParam) : name(s) {
  type=3;
  
  intData=-1;
  doubleData=0.0;
  stringData=cParam;
  boolData=false;
  parentData=NULL;
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const bool &bParam) : name(s) {
  type=4;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=bParam;
  parentData=NULL;
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Array &aParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(aParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const GlobLoc &glParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(glParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const TensorCoordSys &tcsParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(tcsParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Domain &dParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(dParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Field &fParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(fParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Filter &fParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(fParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Integ &iParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(iParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Mesh &mParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(mParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Constr &cParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(cParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Problem &pParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(pParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const Result &rParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(rParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,const TimeDep &tdParam) : name(s) {
  type=5;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=dynamic_cast<parents*>(tdParam.getData());
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::param(const string &s,function1 &fParam) : name(s) {
  type=6;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  parentData=NULL;
  function1Data=&fParam;
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



param::~param(void) {
  type=0;
  
  intData=-1;
  doubleData=0.0;
  stringData="";
  boolData=false;
  setParentData(NULL);
  function1Data=NULL;
  
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=-1;
}



void param::setParentData(parents *p) {
  parentData=p;
}



Int param::getType(void) const {
  return type;
}



string param::getTypeName(void) const {
  switch(type) {
  case 0:
    return "none";
  case 1:
    return "Int";
  case 2:
    return "Double";
  case 3:
    return "string";
  case 4:
    return "bool";
  case 5:
    return "parents";
  case 6:
    return "function1";
  default:
    return "unknown";
  }
}



Int param::getIntData(void) const {
  return(intData);
}



Double param::getDoubleData(void) const {
  return(doubleData);
}



string param::getStringData(void) const {
  return(stringData);
}



bool param::getBoolData(void) const {
  return(boolData);
}



const Int* param::getComponentsData(void) const {
  return(components);
}



parents* param::getParentData(void) const {
  return(parentData);
}

  

Array param::getArrayData(void) const {
  Array A;
  A.setData(dynamic_cast<array*>(getParentData()));
  return A;
}



GlobLoc param::getGlobLocData(void) const {
  GlobLoc GL;
  GL.setData(dynamic_cast<globLoc*>(getParentData()));
  return GL;
}
      


TensorCoordSys param::getTensorCoordSysData(void) const {
  TensorCoordSys TCS;
  TCS.setData(dynamic_cast<tensorCoordSys*>(getParentData()));
  return TCS;  
}



Domain param::getDomainData(void) const {
  Domain D;
  D.setData(dynamic_cast<domain*>(getParentData()));
  return D;
}



Field param::getFieldData(void) const {
  Field F;
  F.setData(dynamic_cast<field*>(getParentData()));
  return F;
}



Integ param::getIntegData(void) const {
  Integ I;
  I.setData(dynamic_cast<integ*>(getParentData()));
  return I;
}
      


Mesh param::getMeshData(void) const {
  Mesh M;
  M.setData(dynamic_cast<mesh*>(getParentData()));
  return M;
}

      

Constr param::getConstrData(void) const {
  Constr C;
  C.setData(dynamic_cast<constr*>(getParentData()));
  return C;
}

      
      
Problem param::getProblemData(void) const {
  Problem P;
  P.setData(dynamic_cast<problem*>(getParentData()));
  return P;
}

      
      
Result param::getResultData(void) const {
  Result R;
  R.setData(dynamic_cast<result*>(getParentData()));
  return R;
}

      
      
TimeDep param::getTimeDepData(void) const {
  TimeDep TD;
  TD.setData(dynamic_cast<timeDep*>(getParentData()));
  return TD;
}

      
      
function1* param::getFunction1Data(void) const {
  return function1Data;
}

      
      
param param::operator=(Int iParam) const {
  param output(*this);
  output.intData=iParam;
  output.type=1;
  return(output);
}



param param::operator=(Double dParam) const {
  param output(*this);
  output.doubleData=dParam;
  output.type=2;
  return(output);
}



param param::operator=(const string &sParam) const {
  param output(*this);
  output.stringData=sParam;
  output.type=3;
  return(output);
}



param param::operator=(const char *cParam) const {
  param output(*this);
  output.stringData=cParam;
  output.type=3;
  return(output);
}



param param::operator=(const bool bParam) const {
  param output(*this);
  output.boolData=bParam;
  output.type=4;
  return(output);
}



param param::operator=(const Array &aParam) const {
  param output(*this);
  array *A=aParam.getData();
  output.setParentData(A);
  if (A!=NULL)
    output.intData=A->getCurrentComponent();
  output.type=5;
  return(output);
}


  
param param::operator=(const GlobLoc &gParam) const {
  param output(*this);
  output.setParentData(gParam.getData());
  output.type=5;
  return(output);
}


  
param param::operator=(const TensorCoordSys &tParam) const {
  param output(*this);
  output.setParentData(tParam.getData());
  output.type=5;
  return(output);
}


  
param param::operator=(const Domain &dParam) const {
  param output(*this);
  output.setParentData(dParam.getData());
  output.type=5;
  return(output);
}


  
param param::operator=(const Field &fParam) const {
  param output(*this);
  field *F=fParam.getData();
  output.setParentData(F);
  output.intData=F->getCurrentComponent();
  output.type=5;
  return(output);
}


  
param param::operator=(const Filter &fParam) const {
  param output(*this);
  filter *F=fParam.getData();
  output.setParentData(F);
  output.type=5;
  return(output);
}


  
param param::operator=(const Integ &iParam) const {
  param output(*this);
  output.setParentData(iParam.getData());
  output.type=5;
  return(output);
}



param param::operator=(const Mesh &mParam) const {
  param output(*this);
  output.setParentData(mParam.getData());
  output.type=5;
  return(output);
}



param param::operator=(const Constr &cParam) const {
  param output(*this);
  constr *C=cParam.getData();
  output.setParentData(C);
  output.type=5;
  return(output);
}


  
param param::operator=(const TimeDep &tdParam) const {
  param output(*this);
  timeDep *TD=tdParam.getData();
  output.setParentData(TD);
  output.type=5;
  return(output);
}


  
param param::operator=(const Problem &pParam) const {
  param output(*this);
  problem *P=pParam.getData();
  output.setParentData(P);
  output.type=5;
  return(output);
}


  
param param::operator=(const Result &rParam) const {
  param output(*this);
  result *R=rParam.getData();
  output.setParentData(R);
  output.type=5;
  return(output);
}


  
param param::operator=(function1 &fParam) const {
  param output(*this);
  output.function1Data=&fParam;
  output.type=6;
  return(output);
}


  
void param::operator=(const param &pin) {
  if (this==&pin)
    return;
  
  setName(pin.getName());
  intData=pin.intData;
  doubleData=pin.doubleData;
  stringData=pin.stringData;
  boolData=pin.boolData;
  setParentData(pin.parentData);
  function1Data=pin.function1Data;
  type=pin.type;
}  



void param::equals(const param &pin) {
  setName(pin.getName());
  intData=pin.intData;
  doubleData=pin.doubleData;
  stringData=pin.stringData;
  boolData=pin.boolData;
  parentData=NULL;
  setParentData(pin.parentData);
  function1Data=pin.function1Data;
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    components[i]=pin.components[i];
  type=pin.type;
}



param param::operator()(Int index0,Int index1,Int index2,Int index3,
                        Int index4,Int index5,Int index6,Int index7) const {
  param output(*this);
  
  output.components[0]=index0;
  output.components[1]=index1;
  output.components[2]=index2;
  output.components[3]=index3;
  output.components[4]=index4;
  output.components[5]=index5;
  output.components[6]=index6;
  output.components[7]=index7;
  
  return(output);
}



paramList param::operator,(const param &argParam) const {
  paramList PL;
  PL.pushBack(*this);
  PL.pushBack(argParam);
  return(PL);
}



void param::print(void) const {
  cerr << "param : " << getName() << endl
    << "   type = " << getTypeName() << endl 
    << "   intData = " << getIntData() << endl 
    << "   doubleData = " << getDoubleData() << endl 
    << "   stringData = " << getStringData() << endl 
    << "   boolData = " << getBoolData() << endl 
    << "   parentData = " << getParentData() << endl 
    << "   componentsData = " ;
  for (Int i=0;i<MAX_ORDER_ARRAY;i++)
    cerr << components[i] << " " ;
  cerr << endl ;
}


