#ifndef PARAM_H
#define PARAM_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "util/name.h"
#include "types/types.h"
#include "types/params.h"

namespace keyFE2 {
  
  class parents ;
  class Array ;
  class Constr ;
  class Filter ;
  class GlobLoc ;
  class TensorCoordSys ;
  class Domain ;
  class Field ;
  class Integ ;
  class Mesh ;
  class Result ;
  class Problem ;
  class TimeDep ;
  class paramList ;
  class function1 ;
  
  class param : public virtual name {
    
    private :
      
      Int type ;
      
      Int intData ;              // if type = 1
      Double doubleData ;        // if type = 2
      std::string stringData ;        // if type = 3
      bool boolData ;            // if type = 4
      parents *parentData ;      // if type = 5
      function1 *function1Data ; // if type = 6
      
      Int components[MAX_ORDER_ARRAY] ;
    
    public :
      
    // constructors and destructor :
      
      param(void) ;
      param(const param &) ;
      param(const std::string &s) ;
      
      param(const std::string &s,const Int &iParam) ;
      param(const std::string &s,const Double &dParam) ;
      param(const std::string &s,const std::string &sParam) ;
      param(const std::string &s,const char* cParam) ;
      param(const std::string &s,const bool &bParam) ;
      param(const std::string &s,const Array &aParam) ;
      param(const std::string &s,const GlobLoc &gParam) ;
      param(const std::string &s,const TensorCoordSys &tParam) ;
      param(const std::string &s,const Domain &dParam) ;
      param(const std::string &s,const Field &fParam) ;
      param(const std::string &s,const Filter &fParam) ;
      param(const std::string &s,const Integ &iParam) ;
      param(const std::string &s,const Mesh &mParam) ;
      param(const std::string &s,const Constr &cParam) ;
      param(const std::string &s,const Problem &pParam) ;
      param(const std::string &s,const Result &rParam) ;
      param(const std::string &s,const TimeDep &tdParam) ;
      param(const std::string &s,function1 &fParam) ;
      
      ~param(void) ;
      
    // Access to member data :
      
      void setParentData(parents *p) ;
      
      Int getType(void) const ;
      std::string getTypeName(void) const ;
      
      Int getIntData(void) const ;
      Double getDoubleData(void) const ;
      std::string getStringData(void) const ;
      bool getBoolData(void) const ;
      const Int* getComponentsData(void) const ;
      parents* getParentData(void) const ;
      Array getArrayData(void) const ;
      GlobLoc getGlobLocData(void) const ;
      TensorCoordSys getTensorCoordSysData(void) const ;
      Domain getDomainData(void) const ;
      Field getFieldData(void) const ;
      Integ getIntegData(void) const ;
      Mesh getMeshData(void) const ;
      Constr getConstrData(void) const ;
      Problem getProblemData(void) const ;
      Result getResultData(void) const ;
      TimeDep getTimeDepData(void) const ;
      function1* getFunction1Data(void) const ;
      
    // Operators :
      
      param operator=(Int iParam) const ;
      param operator=(Double dParam) const ;
      param operator=(const std::string &sParam) const ;
      param operator=(const char *cParam) const ;
      param operator=(const bool bParam) const ;
      param operator=(const Array &aParam) const ;
      param operator=(const GlobLoc &gParam) const ;
      param operator=(const TensorCoordSys &tParam) const ;
      param operator=(const Domain &dParam) const ;
      param operator=(const Field &fParam) const ;
      param operator=(const Filter &fParam) const ;
      param operator=(const Integ &iParam) const ;
      param operator=(const Mesh &mParam) const ;
      param operator=(const Constr &cParam) const ;
      param operator=(const Problem &pParam) const ;
      param operator=(const Result &rParam) const ;
      param operator=(const TimeDep &tdParam) const ;
      param operator=(function1 &fParam) const ;
      void operator=(const param &pin) ;
      void equals(const param &pin) ;
      
      param operator()(Int index0=-1,Int index1=-1,Int index2=-1,
		       Int index3=-1,Int index4=-1,Int index5=-1,
		       Int index6=-1,Int index7=-1) const ;
      
      paramList operator,(const param &argParam) const ;
      
      void print(void) const ;
      
  } ;
  
  extern const param ACTIVE ;
  extern const param ARRAY ;
  extern const param AUTOFIX ;
  extern const param COMPONENTNAME ;
  extern const param COMPONENT ;
  extern const param CONSTR ;
  extern const param CONVERGENCE ;
  extern const param COORD ;
  extern const param DELAY ;
  extern const param DERIV ;
  extern const param DIM ;
  extern const param DIVERGENCE ;
  extern const param DOM ;
  extern const param DTINIT ;
  extern const param DTMAX ;
  extern const param DTMIN ;
  extern const param EPSILON ;
  extern const param EQUATION ;
  extern const param EXPR ;
  extern const param FIELD ;
  extern const param FILTER ;
  extern const param GLTYPE ;
  extern const param GLOBLOC ;
  extern const param INFLU ;
  extern const param INTERPOLATION ;
  extern const param INTERPMESH ;
  extern const param LAYER ;
  extern const param LEFTDOM ;
  extern const param LEFTEQUATION ;
  extern const param LEFTTEST ;
  extern const param LINETYPE ;
  extern const param MAXSTEPSNBR ;
  extern const param MINREF ;
  extern const param NAME ;
  extern const param NBRCOMPONENTS ;
  extern const param NITMAX ;
  extern const param ORDER ;
  extern const param PERMUT ;
  extern const param PROBLEM ;
  extern const param RESULT ;
  extern const param RIGHTDOM ;
  extern const param RIGHTEQUATION ;
  extern const param RIGHTTEST ;
  extern const param RULE ;
  extern const param RULENAME ;
  extern const param SCHEME ;
  extern const param SILENT ;
  extern const param TCSTYPE ;
  extern const param TENSORCOORDSYS ;
  extern const param TEST ;
  extern const param TFINAL ;
  extern const param TINIT ;
  extern const param TYPE ;
  extern const param ZERO ;
  
}

#endif
