#ifndef PARAMLIST_H
#define PARAMLIST_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "param/param.h"
#include "types/types.h"

namespace keyFE2 {
  
  class paramList {
    
    private :
      
      Int paramSize;
      Int paramCapacity;
      param* params;
    
    public :
      
      paramList(void) ;
      paramList(const paramList &PL) ;
      ~paramList(void) ;
      
      void operator=(const paramList &PL) ;

      param operator[](Int index) const ;
      
      void pushBack(const param &p) ;
      paramList& operator,(const param &p) ;
      paramList& operator,(const paramList &pl) ;
      
      paramList getListOfType(const std::string &s) const ;
      paramList getListNotOfType(const std::string &s) const ;
      paramList getListOfType(const param &pp) const ;
      paramList getListNotOfType(const param &pp) const ;
      
      Int getNbrParams(void) const ;
      Int getNbrParamsNamed(const std::string &sId) const ;
      
      param extractParamNamed(const std::string &sId,bool exactlyOne=false) ;
      
      Int extractIntData(std::string sId,bool exactlyOne=false) ;
      Double extractDoubleData(std::string sId,bool exactlyOne=false) ;
      std::string extractStringData(std::string sId,bool exactlyOne=false) ;
      bool extractBoolData(std::string sId,bool exactlyOne=false) ;
      const Int* extractComponentsData(std::string sId,bool exactlyOne=false) ;
      parents* extractParentData(std::string sId,bool exactlyOne=false) ;
      Array extractArrayData(std::string sId,bool exactlyOne=false) ;
      GlobLoc extractGlobLocData(std::string sId,bool exactlyOne=false) ;
      TensorCoordSys extractTensorCoordSysData(std::string sId,bool exactlyOne=false) ;
      Domain extractDomainData(std::string sId,bool exactlyOne=false) ;
      Field extractFieldData(std::string sId,bool exactlyOne=false) ;
      Integ extractIntegData(std::string sId,bool exactlyOne=false) ;
      Mesh extractMeshData(std::string sId,bool exactlyOne=false) ;
      Constr extractConstrData(std::string sId,bool exactlyOne=false) ;
      TimeDep extractTimeDepData(std::string sId,bool exactlyOne=false) ;
      Problem extractProblemData(std::string sId,bool exactlyOne=false) ;
      Result extractResultData(std::string sId,bool exactlyOne=false) ;

      void print(void) const ;
      
  } ;
  
}

#endif
