#ifndef PLOT_BASE_H
#define PLOT_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "print/print.h"
#include "types/types.h"
#include "util/utils.h"

namespace keyFE2 {
  
  class domain ;
  class array ;
  class result ;
  
  class plot : public virtual name, public virtual parents,
	       public virtual print, public virtual newPoint {
				      
    private :
      
      std::string outputType ;
      domain *dom ;
      array *coordinates ;
    
      Int resultsCapacity ;
      Int resultsSize ;
      result **results ;
    
    public :
      
      plot(void) ;
      plot(domain *d,array *c) ;
      ~plot(void) ;
      
      void setType(std::string s) ;
      void setDomain(domain *d) ;
      void setCoordinates(array *c) ;
      void addResult(array *r) ;
      void addResult(result *r) ;
      
      void exec(std::string baseFileName) ;
      void exec_vigie2D(std::string baseFileName) ;
      void exec_openDX(std::string baseFileName) ;
      void exec_gmsh(std::string baseFileName) ;
      
      void setNewPoint(bool) ;
      void accessNbrInit(void) ;
      
      virtual printTree getPrintTreeData(void) const ;
      virtual printTree getPrintTreeValues(void) const ;
      virtual printTree getPrintTree(bool values) const ;
      
  } ;
  
}

#endif
