
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <cstring>
#include <stdio.h>

#include "printTree.h"
#include "err/err.h"
#include "types/types.h"

using namespace std ;
using namespace keyFE2 ;



printTree::printTree(void) {
  nbrStrings=0;
  strings=NULL;
}



printTree::printTree(const printTree &ptin) {
  try {
    Int i,length;
    char *str;
    
    nbrStrings=ptin.nbrStrings;
    if (nbrStrings>0)
      strings=new  char* [nbrStrings];
    else
      strings=NULL;
    for (i=0;i<nbrStrings;i++) {
      length=strlen(ptin.strings[i])+1;
      str=new char[length];
      strcpy(str,ptin.strings[i]);
      strings[i]=str;
    }
  }
  catchAll("printTree::printTree(const printTree &ptin)","")
}



printTree::~printTree(void) {
  try {
    Int i;
    for (i=0;i<nbrStrings;i++)
      delete[](strings[i]);
    delete[] strings;
    
    nbrStrings=0;
    strings=NULL;
  }
  catchAll("printTree::~printTree(void)","")
}



void printTree::operator=(const string &s) {
  try {
    Int i,length;
    char *str;
    
    for (i=0;i<nbrStrings;i++)
      delete[] strings[i];
    delete[] strings;
    
    length=strlen(s.c_str())+1;
    str=new char[length];
    strcpy(str,s.c_str());
    
    nbrStrings=1;
    strings=new char* [1];
    strings[0]=str;
  }
  catchAll("void printTree::operator=(const string &s)","")
}



void printTree::operator+=(const string &s) {
  try {
    Int i,length;
    char **newStrings,*str;
    
    newStrings=new char* [nbrStrings+1];
    for (i=0;i<nbrStrings;i++)
      newStrings[i]=strings[i];
    length=strlen(s.c_str())+1;
    str=new char[length];
    strcpy(str,s.c_str());
    newStrings[nbrStrings++]=str;
    
    delete[] strings;
    strings=newStrings;
  }
  catchAll("void printTree::operator+=(const string &s)","")
}



printTree printTree::operator+(const printTree &ptadd) const {
  try {
    Int i,nbr1,nbr2,length;
    char *str;
    printTree ptout;
    
    nbr1=nbrStrings;
    nbr2=ptadd.nbrStrings;
    
    ptout.nbrStrings=nbr1+nbr2;
    if (nbr1+nbr2>0)
      ptout.strings=new char* [nbr1+nbr2];
    else
      ptout.strings=NULL;
    for (i=0;i<nbr1;i++) {
      length=strlen(strings[i])+1;
      str=new char[length];
      strcpy(str,strings[i]);
      ptout.strings[i]=str;
    }
    for (i=0;i<nbr2;i++) {
      length=strlen(ptadd.strings[i])+1;
      str=new char[length];
      strcpy(str,ptadd.strings[i]);
      ptout.strings[nbr1+i]=str;
    }
    
    return(ptout);
  }
  catchAll("printTree printTree::operator+(const printTree &ptadd) const","")
}



void printTree::operator+=(const printTree &ptadd) {
  try {
    printTree ptout;
    
    ptout=(*this)+ptadd;
    
    (*this)=ptout;
  }
  catchAll("void printTree::operator+=(const printTree &ptadd)","")
}



void printTree::operator=(const printTree &ptin) {
  if (this==&ptin)
    return;
  
  try {
    Int i,length;
    char *str;
    
    for (i=0;i<nbrStrings;i++)
      delete[] strings[i];
    nbrStrings=0;
    delete[] strings;
    strings=NULL;
    
    nbrStrings=ptin.nbrStrings;
    if (nbrStrings>0)
      strings=new char* [nbrStrings];
    else
      strings=NULL;
    for (i=0;i<nbrStrings;i++) {
      length=strlen(ptin.strings[i])+1;
      str=new char[length];
      strcpy(str,ptin.strings[i]);
      strings[i]=str;
    }
  }
  catchAll("void printTree::operator=(const printTree &ptin)","")
}



void printTree::addMargin(Int dec) {
  try {
    Int i,j,length;
    char *str;
    string sbuf;
    
    for (i=0;i<nbrStrings;i++) {
      sbuf="";
      for (j=0;j<dec;j++)
	sbuf+=" ";
      sbuf+=strings[i];
      length=strlen(sbuf.c_str())+1;
      str=new char[length];
      strcpy(str,sbuf.c_str());
      delete[] strings[i];
      strings[i]=str;
    }
  }
  catchAll("void printTree::addMargin(Int dec)","")
}  



void printTree::write(ostream &os) const {
  try {
    Int i;
    for (i=0;i<nbrStrings;i++) {
      os << strings[i] << endl ;
    }
  }
  catchAll("void printTree::write(ostream &os) const","")
}



void printTree::prepareForArguments(void) {
  try {
    Int i;
    
    for (i=1;i<nbrStrings;i++)
      strings[i][1]='|';
  }
  catchAll("void printTree::prepareForArguments(void)","")
}



void printTree::prepareForAdd(string s) {
  try {
    Int i,length;
    char *str;
    
    if (strings!=NULL&&s!="") {
      s+=strings[0];
      delete[] strings[0];
      length=strlen(s.c_str())+1;
      strings[0]=new char[length];
      strcpy(strings[0],s.c_str());
    }
    
    for (i=0;i<nbrStrings;i++) {
      length=strlen(strings[i])+7;
      str=new char[length];
      if (i==0)
	sprintf(str," |--> %s",strings[i]);
      else
	sprintf(str," |    %s",strings[i]);
      delete[] strings[i];
      strings[i]=str;
    }
  }
  catchAll("void printTree::prepareForAdd(string s)","")
}



void printTree::prepareForAddLast(string s) {
  try {
    Int i,length;
    char *str;
    
    if (strings!=NULL&&s!="") {
      s+=strings[0];
      delete[] strings[0];
      length=strlen(s.c_str())+1;
      strings[0]=new char[length];
      strcpy(strings[0],s.c_str());
    }
    
    for (i=0;i<nbrStrings;i++) {
      length=strlen(strings[i])+7;
      str=new char[length];
      if (i==0)
	sprintf(str," |--> %s",strings[i]);
      else
	sprintf(str,"      %s",strings[i]);
      delete[] strings[i];
      strings[i]=str;
    }
  }
  catchAll("void printTree::prepareForAddLast(string s)","")
}


