
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <stdio.h>
#include <iostream>
#include <iomanip>
#include <typeinfo>

#include "result_base.h"
#include "array/array_base.h"
#include "err/err.h"
#include "domain/domain_base.h"
#include "field/field_base.h"
#include "print/printTree.h"

using namespace std ;
using namespace keyFE2 ;



result::result(void) : name(), parents() {
  try {
    A=NULL;
    dom=NULL;
    interpolationTypeName="";
    
    F=NULL;
  }
  catchAll("result::result(void)","")
}



result::result(array *inArray,domain *inDom,string interp) : 
     name(), parents() {
  try {
    A=NULL;
    setArray(inArray);
    dom=NULL;
    setDom(inDom);
    interpolationTypeName=interp;
    
    F=NULL;
  }
  catchAll("result::result(void)","")
}



result::result(const result &in) : name(in), parents() {
  try {
    A=in.A;
    dom=in.dom;
    interpolationTypeName=in.interpolationTypeName;
    
    F=NULL;
  }
  catchAll("result::result(const result &in)","")
}



result::~result(void) {
  try {
    setArray(NULL);
    setDom(NULL);
    setField(NULL);
    interpolationTypeName="";
  }
  catchAll("result::~result(void)","")
}



void result::setArray(array *inArray) {
  try {
    delinkAndDelete(A,&A);
    A=inArray;
    if (A!=NULL)
      A->addParent(&A);
  }
  catchAll("void result::setArray(array *inArray)","")
}



void result::setDom(domain *inDom) {
  try {
    delinkAndDelete(dom,&dom);
    dom=inDom;
    if (dom!=NULL)
      dom->addParent(&dom);
  }
  catchAll("void result::setDom(domain *inDom)","")
}



void result::setField(field *inField) {
  try {
    delinkAndDelete(F,&F);
    F=inField;
    if (F!=NULL)
      F->addParent(&F);
  }
  catchAll("void result::setField(field *inField)","")
}



void result::generateField(array *coordinates) {
  try {
    int nbrComponents=A->getNbrVal();
    field *tmpF=new field(dom,nbrComponents,interpolationTypeName);
    tmpF->interpolate(A,dom,coordinates);
    setField(tmpF);
  }
  catchAll("void result::generateField(array *coordinates)","")
}



void result::cleanField(void) {
  try {
    setField(NULL);
  }
  catchAll("void result::cleanField(void)","")
}



void result::setNewPoint(bool b) {
  A->setNewPoint(b);
  dom->setNewPoint(b);
  newPoint::setNewPoint(b);
}



void result::accessNbrInit(void) {
  A->accessNbrInit();
  dom->accessNbrInit();
  newPoint::accessNbrInit();
}



printTree result::getPrintTreeData(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt;
    
    if (getName()=="")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    sprintf(cbuf,"   interpolation type = %s",interpolationTypeName.c_str());
    pt+=cbuf;
    
    return(pt);
  }
  catchAll("result::getPrintTreeData","")
}



printTree result::getPrintTreeValues(void) const {
  try {
    printTree pt;
    return(pt);
  }
  catchAll("result::getPrintTreeValues","")
}



printTree result::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt,ptArg;
    
    sprintf(cbuf,"result %p",static_cast<const void*>(this));
    sbuf=cbuf;
    pt+=sbuf;
    
    pt+=getPrintTreeData();
    
    if (values)
      pt+=getPrintTreeValues();
    
    sbuf="   ";
    pt+=sbuf;
    
    pt.prepareForArguments();

    sprintf(cbuf,"Array");
    ptArg=A->print::getPrintTreeSummary(cbuf);
    ptArg.prepareForAdd();
    pt+=ptArg;
      
    sprintf(cbuf,"Domain");
    ptArg=dom->print::getPrintTreeSummary(cbuf);
    ptArg.prepareForAddLast();
    pt+=ptArg;
    
    return(pt);
  }
  catchAll("result::getPrintTree","")
}



printTree result::getPrintTreeSummary(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf,typeName;
    printTree pt;
    
    sprintf(cbuf,"result %p",static_cast<const void*>(this));
    sbuf=cbuf;
    pt+=sbuf;
    
    if (getName()=="")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    sbuf="   ";
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("result::getPrintTreeSummary","")
}

    
