#ifndef RESULT_BASE_H
#define RESULT_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "print/print.h"
#include "types/allTypes.h"
#include "util/utils.h"

namespace keyFE2 {
  
  class result : public virtual name, public virtual parents,
		 public virtual print {

    friend class array ;
    friend class domain ;
    friend class field ;
    friend class plot ;
		
    protected :
    
      array *A ;
      domain *dom ;
      std::string interpolationTypeName ;
      
      field *F ;
    
    public :
      
      result(void) ;
      result(array *inArray,domain *inDom,std::string interp) ;
      result(const result &in) ;
      virtual ~result(void) ; 
      
      void setArray(array *inArray) ;
      void setDom(domain *inDom) ;     
      void setField(field *inField) ;
      
      void generateField(array *coordinates) ;
      void cleanField(void) ;
      
      void setNewPoint(bool b) ;
      void accessNbrInit(void) ;
      
      printTree getPrintTreeData(void) const ;
      printTree getPrintTreeValues(void) const ;
      printTree getPrintTree(bool values) const ;
      printTree getPrintTreeSummary(void) const ;
      
  } ;
   
}

#endif
    
    
