#ifndef SCAL_H
#define SCAL_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <string>

#include "types/types.h"
#include "types/params.h"

#include "util/boolZero.h"
#include "util/utils.h"

#include "scal/scalAccel.h"

namespace keyFE2 {
  
  class arrayField ;
  class function1 ;
  
  class scal : public boolZero {
    
    friend class array ;
    friend class arrayComp ;
    friend class arrayField ;
    friend class arrayFilter ;
    friend class arrayOp1 ;
    friend class arrayOpN ;
    friend class field ;
    friend class step ;
    friend void exchange(scal &s1,scal &s2) ;
    
    private :
      
      static Int mapCapacity ;
      static Int mapSize ;
      static Int *map ;
      static bool white ;
      
      Double val ;
      
      Int ligCapacity ;
      Int ligSize ;
      Int ligIndex ;
      bool *ligBool ;
      Double *lig ;
      
      Int colCapacity ;
      Int colSize ;
      Int colIndex ;
      bool *colBool ;
      Double *col ;
      
      Double **mat ;
      
      // Data for combili calculation :
      
      static Int stackCapacity ;
      static Int stackSize ;
      static Double *dStack ;
      static scal **sStack ;
      
    public : 
      
      scal(void) ;
      scal(Double d) ;
      scal(const scal &) ;
      ~scal(void) ;
      
      Int* getMap(void) const ;
      Int getPosInMap(Int nbr) const ;
      
      static void setWhite(bool b) ;
      static bool getWhite(void) ;
      
      void setLigColCapacity(Int nbr1,Int nbr2) ;
      void setLigCapacity(Int nbr) ;
      void setColCapacity(Int nbr) ;
      void setLigColSize(Int nbr1,Int nbr2) ;
      void setLigSize(Int nbr) ;
      void setColSize(Int nbr) ;
      void setLigIndex(Int nbr) ;
      void setColIndex(Int nbr) ;
      void zeroInit(void) ;
      
      Double getVal(void) const ;
      
      Int getLigSize(void) const ;
      Int getLigIndex(void) const ;
      bool* getLigBool(void) const ;
      Double* getLig(void) const ;
      
      Int getColSize(void) const ;
      Int getColIndex(void) const ;
      bool* getColBool(void) const ;
      Double* getCol(void) const ;
      
      Double** getMat(void) const ;
      
      bool updateBoolZeroValue(void) ;
      
      void writeMat(std::ostream &myCout) const ;
      void writeCol(std::ostream &myCout) const ;
      void writeLig(std::ostream &myCout) const ;
      void writeVal(std::ostream &myCout) const ;
      void writeMat(std::string &fileName) const ;
      void writeCol(std::string &fileName) const ;
      void writeLig(std::string &fileName) const ;
      void writeVal(std::string &fileName) const ;
      
      // Various operators :
      
      void assignValuesWhite(const scal &) ;
      void assignValues(const scal &) ;
      void uminusAssignValues(const scal &) ;
      void multAssignValues(Double factor,const scal &) ;
      void multAddValues(Double factor,const scal &) ;
      void assignWhite(const scal &) ;
      void operator=(const scal &) ;
      void operator=(Double) ;
      void setToTransposedOf(const scal &) ;
      
      void mapMerge(const scal &,const scal &) ;
      void mapMerge(const scal &,const scal &,const scal &) ;
      
      void dyadicWhite(const scal &,const scal &) ;
      void sum(const scal &,const scal &) ;
      void diff(const scal &,const scal &) ;
      void mult(const scal &,const scal &) ;
      void uminusMult(const scal &,const scal &) ;
      void uminusAssign(const scal &) ;
      void div(const scal &,const scal &) ;
      void mult(const scal &,Double) ;
      void div(const scal &,Double) ;
      void doubleDiff(const Double &d,const scal &,const scal &) ;
      void MultDiffMult(const scal &,const scal &,const scal &,const scal &) ;
      void MultSumMult(const scal &,const scal &,const scal &,const scal &) ;
      
      void incWhite(const scal &) ;
      void operator+=(const scal &) ;
      void operator-=(const scal &) ;
      void operator*=(const scal &) ;
      void operator/=(const scal &) ;
      void operator*=(Double) ;
      void operator/=(Double) ;
      
      scal& operator-(const scal &) const ;
      scal& operator/(const scal &) const ;
      scal& operator*(Double) const ;
      scal& operator/(Double) const ;
      
      bool operator==(Double) const ;
      bool operator!=(Double) const ;
      bool operator<=(Double) const ;
      bool operator>=(Double) const ;
      bool operator<(Double) const ;
      bool operator>(Double) const ;
      
      void functionExec(scal *scalArg,function1 *fParam,
			bool makeValue=true,bool makeDeriv=true) ;
      
      // Member functions for stack management :
      
      static void stackClear(void) ;
      static void stackPushBack(Double dArg,scal &sArg) ;
      void stackCalculate(void) ;
      
      // Accelerated operators :
      
      void operator=(const SSAdd &tmp) {
	this->sum(tmp.s1,tmp.s2);
      }
      
      void operator=(const SSSAdd &tmp) {
	this->sum(tmp.s1,tmp.s2);
	(*this)+=tmp.s3;
      }
      
      void operator=(const SSDiff &tmp) {
	this->diff(tmp.s1,tmp.s2);
      }
      
      void operator=(const SSMult &tmp) {
	this->mult(tmp.s1,tmp.s2);
      }
      
      void operator=(const SUMinus &tmp) {
	this->uminusAssign(tmp.s);
      }
      
      void addDyadicWhite(const scal &left,const scal &right);
      void operator+=(const SSMult &tmp) ;    
      void operator-=(const SSMult &tmp) ;
      
      void operator=(const SSMultUMinus &tmp) {
	this->uminusMult(tmp.s1,tmp.s2);
      }
      
      void operator=(const SSDoubleDiff &tmp) {
	this->doubleDiff(tmp.d,tmp.s1,tmp.s2);
      }
      
      void operator=(const SSMSSMdiff &tmp) {
	this->MultDiffMult(tmp.ssm1.s1,tmp.ssm1.s2,tmp.ssm2.s1,tmp.ssm2.s2);
      }
      
      void operator=(const SSMSSMsum &tmp) {
	this->MultSumMult(tmp.ssm1.s1,tmp.ssm1.s2,tmp.ssm2.s1,tmp.ssm2.s2);
      }
      
  } ;
  
  void exchange(scal &s1,scal &s2) ;
  
}

#endif
