
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

namespace keyFE2 {
  
  class scal ;
  
  // Sum of two scal arguments :
  
  struct SSAdd {
    const scal &s1 ;
    const scal &s2 ;
    SSAdd(const scal &left,const scal &right) : s1(left), s2(right) {}
  } ;
  
  inline SSAdd operator+(const scal &left,const scal &right) {
    return SSAdd(left,right);
  }
  
  // Sum of three scal arguments :
  
  struct SSSAdd {
    const scal &s1 ;
    const scal &s2 ;
    const scal &s3 ;
    SSSAdd(const SSAdd &tmp,const scal &right) : s1(tmp.s1), s2(tmp.s2),
							     s3(right) {}
  } ;
  
  inline SSSAdd operator+(const SSAdd &tmp,const scal &right) {
    return SSSAdd(tmp,right);
  }
  
  // Difference of two scal arguments :
  
  struct SSDiff {
    const scal &s1 ;
    const scal &s2 ;
    SSDiff(const scal &left,const scal &right) : s1(left), s2(right) {}
  } ;
  
  inline SSDiff operator-(const scal &left,const scal &right) {
    return SSDiff(left,right);
  }
  
  // Double product of the difference of two scal arguments :
  
  struct SSDoubleDiff {
    const Double &d ;
    const scal &s1 ;
    const scal &s2 ;
    SSDoubleDiff(const Double &r,const scal &left,const scal &right) : d(r),
    s1(left), s2(right) {}
  } ;
  
  inline SSDoubleDiff operator*(const Double &left,const SSDiff &right) {
    return SSDoubleDiff(left,right.s1,right.s2);
  }
  
  inline SSDoubleDiff operator*(const SSDiff &right,const Double &left) {
    return SSDoubleDiff(left,right.s1,right.s2);
  }  
  
  // Product of two scal arguments :
  
  struct SSMult {
    const scal &s1 ;
    const scal &s2 ;
    SSMult(const scal &left,const scal &right) : s1(left), s2(right) {}
  } ;
  
  inline SSMult operator*(const scal &left,const scal &right) {
    return SSMult(left,right);
  }
  
  // Unary minus of a scal and derived :
  
  struct SUMinus {
    const scal &s ;
    SUMinus(const scal &arg) : s(arg) {}
  } ;  
  
  inline SUMinus operator-(const scal &arg) {
    return SUMinus(arg);
  }
  
  struct SSMultUMinus {
    const scal &s1 ;
    const scal &s2 ;
    SSMultUMinus(const SUMinus &left,const scal &right) : s1(left.s), s2(right) {}
								      SSMultUMinus(const scal &left,const SUMinus &right) : s1(left), s2(right.s) {}
  } ;
  
  inline SSMultUMinus operator*(const SUMinus &left,const scal &right) {
    return SSMultUMinus(left,right);
  }
  
  inline SSMultUMinus operator*(const scal &left,const SUMinus &right) {
    return SSMultUMinus(left,right);
  }
  
  // Difference of two SSMult arguments :
  
  struct SSMSSMdiff {
    const SSMult &ssm1 ;
    const SSMult &ssm2 ;
    SSMSSMdiff(const SSMult &left,const SSMult &right) : 
      ssm1(left), ssm2(right) {}
  } ;
  
  inline SSMSSMdiff operator-(const SSMult &left,const SSMult &right) {
    return SSMSSMdiff(left,right);
  }
  
  // Addition of two SSMult arguments :
  
  struct SSMSSMsum {
    const SSMult &ssm1 ;
    const SSMult &ssm2 ;
    SSMSSMsum(const SSMult &left,const SSMult &right) : 
      ssm1(left), ssm2(right) {}
  } ;
  
  inline SSMSSMsum operator+(const SSMult &left,const SSMult &right) {
    return SSMSSMsum(left,right);
  }
  
}
