#ifndef SOLVER_H
#define SOLVER_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>
#include <iostream>

#include "types/types.h"

namespace keyFE2 {
  
  class array ;
  class param ;
  class paramList ;
  class solver ;
  class Array ;
  class Constr ;
  
  class Solver {
    
    private :
      
      void *data ;
    
    public :
      
      Solver(void) ;
      Solver(const Solver &in) ;
      Solver(param P) ;
      Solver(paramList PL) ;
      ~Solver(void) ;
      
      void setData(solver*);
      solver* getData(void) const ;
      void setName(std::string s);
      std::string getName(void) const ;
      
      void write(void) const ;
      void writeValues(void) const ;
      void write(std::ostream &out) const ;
      void writeValues(std::ostream &out) const ;
      
      // = operator :
      
      void operator=(const Solver &);
      
      void setZero(Double) ;
      void addProblem(Array &A) ;
      void addConstraint(Constr &C) ;
      
      bool NewtonRaphson(void) ;
      
      Int getLastState(void) const ; // 1: converged, 3: diverged, 2: neither
      Int getLastNbrIterations(void) const ;
      Int getLastNbrDelayed(void) const ;
      Int getLastNbrAutoFix(void) const ;
      Int getLastNbrPermut(void) const ;
      
      bool check(std::ostream &os) ;
      
      
  } ;
  
}

#endif
