#ifndef STEP_H
#define STEP_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>
#include <iostream>

#include "scal/scal.h"
#include "types/params.h"
#include "types/types.h"

namespace keyFE2 {
  
  class array ;
  class arrayFilter ;
  class object ;
  
  class step {
    
    friend class stepList ;
    
    private :
      
      Double factor ;
      array *equation ;
      object *element ;
      Int direction ;
      
      std::set<arrayFilter*> AFL ;
    
      static scal *scalPtr ;
      static scal S ;
    
    public :
      
      step(void) ;
      step(array *eq,object *elem=NULL,Int direct=-1000) ;
      step(const step &in) ;
      ~step(void) ;
      
      void write(std::ostream &myCout) const ;
      
      void setEquation(array *arg);
      void setElement(object *arg);
      void setDirection(Int arg);
      void setFactor(Double x) ;
      void multFactor(Double x) ;
      Double getFactor(void) const ;
      array* getEquation(void) const ;
      object* getElement(void) const ;
      Int getDirection(void) const ;
      scal* getScalPtr(void) const ;
      void addArrayFilter(arrayFilter *A) ;
      
      void buildActiveFieldList(void) const ;
      void exec(Double *Coord=NULL) const ;
      
      // operators :
      
      step& operator=(const step &in) ;
      bool operator<(const step &arg2) const ;
      bool operator>(const step &arg2) const ;
      bool operator==(const step &arg2) const ;
      
  } ;
  
}

#endif
