
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <list>
#include <algorithm>

#include "stepList.h"
#include "step.h"
#include "err/err.h"
#include "object/object_base.h"

using namespace std ;
using namespace keyFE2 ;



stepList::stepList(Int initCapacity) {
  try {
    capacity=initCapacity;
    size=0;
    if (capacity>0)
      data=new step[capacity];
    else
      data=NULL;
  }
  catchAll("stepList::stepList(Int initCapacity)","")
}



stepList::stepList(const stepList &in) {
  try {
    capacity=in.size;
    size=in.size;
    if (capacity>0) {
      data=new step[capacity];
      for (Int i=0;i<size;i++)
	data[i]=in.data[i];
    }
    else
      data=NULL;
  }
  catchAll("stepList::stepList(const stepList &in)","")
}



stepList::~stepList(void) {
  try {
    capacity=0;
    size=0;
    delete[] data;
    data=NULL;
  }
  catchAll("stepList::~stepList(void)","")
}



void stepList::init(void) {
  size=0;
}



void stepList::clear(void) {
  try {
    capacity=0;
    size=0;
    delete[] data;
    data=NULL;
  }
  catchAll("void stepList::clear(void)","")
}



void stepList::pushBack(const step &in) {
  try {
    if (size<capacity) {
      data[size]=in;
      size++;
    }
    else {
      step *tmpData=data;
      capacity=2*(size+1);
      data=new step[capacity];
      for (Int i=0;i<size;i++)
	data[i]=tmpData[i];
      delete[] tmpData;
      data[size]=in;
      size++;
    }    
  }
  catchAll("void stepList::pushBack(const step &in)","")
}



inline void swap(step &arg1,step &arg2) {
  step tmp=arg1;
  arg1=arg2;
  arg2=tmp;
}



inline void swap(object* &arg1,object* &arg2) {
  object* tmp=arg1;
  arg1=arg2;
  arg2=tmp;
}



void stepList::sort(void) {
  try {
    // On se satisfait pour l'instant d'un tri  bulles :
    Int i,j;
    for (i=size-1;i>0;i--)
      for (j=0;j<i;j++)
	if (data[j]<data[j+1])
	  swap(data[j],data[j+1]);
  }
  catchAll("void stepList::sort(void)","")
}



void stepList::write(ostream &myCout) const {
  try {
    for (Int i=0;i<size;i++)
      data[i].write(myCout);
  }
  catchAll("void stepList::write(ostream &myCout) const","")
}



void stepList::setFactor(Double x) {
  try {
    for (Int i=0;i<size;i++)
      data[i].setFactor(x);
  }
  catchAll("void stepList::setFactor(Double x)","")
}



void stepList::multFactor(Double x) {
  try {
    for (Int i=0;i<size;i++)
      data[i].multFactor(x);
  }
  catchAll("void stepList::multFactor(Double x)","")
}



void stepList::operator=(const stepList &in) {
  if (this==&in)
    return;
  
  try {
    if (capacity<in.size) {
      delete[] data;
      capacity=2*in.size;
      if (capacity>0)
	data=new step[capacity];
      else
	data=NULL;
    }
    size=in.size;
    for (Int i=0;i<size;i++)
      data[i]=in.data[i];
  }
  catchAll("void stepList::operator=(const stepList &in)","")
}



void stepList::operator+=(const stepList &in) {
  try {
    if (capacity<size+in.size) {
      step *tmpData=data;
      capacity=size+in.size+10;
      data=new step[capacity];
      for (Int i=0;i<size;i++)
	data[i]=tmpData[i];
      delete[] tmpData;
    }
    for (Int i=0;i<in.size;i++)
      data[size+i]=in.data[i];
    size+=in.size;
  }
  catchAll("void stepList::operator+=(const stepList &in)","")
}



stepList stepList::operator+(const stepList &in) {
  try {
    stepList output;
    output.capacity=size+in.size+10;
    delete[] output.data;
    output.data=new step[output.capacity];
    output.size=size+in.size;
    for (Int i=0;i<size;i++)
      output.data[i]=data[i];
    for (Int i=0;i<in.size;i++)
      output.data[size+i]=in.data[i];
    return(output);
  }
  catchAll("stepList stepList::operator+(const stepList &in)","")
}



Int stepList::getSize(void) const {
  return(size);
}



step* stepList::operator[](Int index) const {
  try {
    if (index<0||index>size)
      errThrow("stepList::operator[]",string("invalid index :  ")
	       +strFromInt(index)+".");
    return(data+index);
  }
  catchAll("step* stepList::operator[](Int index) const","")
}



// classes used for sorting stepLists :
// ************************************


stepList::triplet::triplet(void) {
  currentStep=NULL;
  appearance=NULL;
  disappearance=NULL;
}
    


stepList::triplet::triplet(step *step1,object *obj1,object *obj2) {
  currentStep=step1;
  appearance=obj1;
  disappearance=obj2;
}



stepList::triplet::~triplet(void) {
  currentStep=NULL;
  appearance=NULL;
  disappearance=NULL;
}



void stepList::triplet::operator=(const triplet &tr) {
  currentStep=tr.currentStep;
  appearance=tr.appearance;
  disappearance=tr.disappearance;
}



bool stepList::tripletCmp::operator()(const triplet &tr1,const triplet &tr2) const {
  if (tr1.disappearance==NULL)
    return true;
  if (tr2.disappearance==NULL)
    return false;
  if (tr1.disappearance<tr2.disappearance)
    return true;
  if (tr1.disappearance>tr2.disappearance)
    return false;
  if (tr1.appearance<tr2.appearance)
    return true;
  if (tr1.appearance>tr2.appearance)
    return false;
  if (tr1.currentStep<tr2.currentStep)
    return true;
  return false;
}



void stepList::sortForFrontale(void) {
  try {
    Int i,j;
    static object **appearance,**disappearance;
    
    bool quickSort=true;
    
    if (size>0) {
      appearance=new object* [size];
      disappearance=new object* [size];
      for (i=0;i<size;i++) {
	if (data[i].element!=NULL) {
	  (data[i].element)->getPlusMinMax(appearance[i],disappearance[i]);
	}
	else {
	  appearance[i]=NULL;
	  disappearance[i]=NULL;
	}
      }
    }
    else {
      appearance=NULL;
      disappearance=NULL;
    }
    
    if (quickSort) {
      list<triplet> tripList;
      triplet tr;
      for (i=0;i<size;i++) {
	tr=triplet(data+i,appearance[i],disappearance[i]);
	tripList.push_back(tr);
      }
      tripList.sort(tripletCmp());
      
      i=0;
      step *oldData=data;
      data = new step[capacity];
      list<triplet>::iterator trit;
      for (trit=tripList.begin();trit!=tripList.end();trit++)
	data[i++]=*(trit->currentStep);
      delete[] oldData;
      if (i!=size)
	errThrow("stepList::sortForFrontale","Error in sorting!!!");
    }
    else {
      // On se contente d'un tri  bulles:
      for (i=size-1;i>0;i--) {
	for (j=0;j<i;j++) {
	  if ((disappearance[j]==NULL)
	      ||(disappearance[j]<disappearance[j+1])
	      ||(disappearance[j]==disappearance[j+1]
		 &&appearance[j]<appearance[j+1])) {
	    swap(data[j],data[j+1]);
	    swap(appearance[j],appearance[j+1]);
	    swap(disappearance[j],disappearance[j+1]);
	  }
	}
      }
    }
    
    delete[] appearance;
    delete[] disappearance;
  }
  catchAll("void stepList::sortForFrontale(void)","")
}




