#ifndef STEPLIST_H
#define STEPLIST_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>

#include "types/params.h"
#include "types/types.h"

namespace keyFE2 {
  
  class array ;
  class object ;
  class step ;
  
  class stepList {
    
    private :
      
      Int capacity ;
      Int size ;
      step *data ;
    
    public :
      
      stepList(int initCapacity=10) ;
      stepList(const stepList &in) ;
      ~stepList(void) ;
      
      void init(void) ;
      void clear(void) ;
      void pushBack(const step &in) ;
      void sort(void) ;
      void write(std::ostream &myCout) const ;
      
      void setFactor(Double x) ;
      void multFactor(Double x) ;
      
      // operators :
      
      void operator=(const stepList &in) ;
      void operator+=(const stepList &in) ;
      stepList operator+(const stepList &in) ;
      Int getSize(void) const ;
      step* operator[](Int index) const ;
      
      // Everything for the sorting operations :
      
      class triplet {
	public:
	  step *currentStep ;
	object *appearance ;
	object *disappearance ;
	triplet(void);
	triplet(step *step1,object *obj1,object *obj2) ;
	~triplet(void) ;
	void operator=(const triplet &tr) ;
      } ;
      
      class tripletCmp {
	public:
	  bool operator()(const triplet &,const triplet &) const ;
      } ;

      void sortForFrontale(void) ;
      
  } ;
  
}

#endif
