
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "array/ArrayOperators.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "mesh/Mesh.h"
#include "param/paramList.h"
#include "scal/scal.h"
#include "step/step.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  void Array_test1(void) {
    Mesh MESH;
    MESH.read("brol.msh","rsMesh");
    
    Domain Dom1=MESH.getDomain(1);
    Domain Dom2=MESH.getDefDomain();
    Dom1.setName("first Domain");
    
    Field coord((DOM=Dom1,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates field",INTERPMESH=MESH));
    coord.write();
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de coordonnes",
              DERIV=false,COMPONENT(0)=coord.component(0),
	      COMPONENT(1)=coord.component(1)));
    A1.write();
    
    Array A3(A1);
  }
  
  
  
  void Array_test2(void) {
    Int i,j;
    
    Mesh MESH;
    MESH.read("oneQuad.msh","rsMesh");
    
    Domain Dom=MESH.getDefDomain();
    Dom.setName("first Domain");
    
    Field coordinates((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		       COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		       NAME="coordinates field",INTERPMESH=MESH));
    Field velo((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    
    velo.interpolate(coordinates,Dom,coordinates);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de coordonnes",
	      DERIV=false,COMPONENT(0)=coordinates.component(0),
	      COMPONENT(1)=coordinates.component(1)));
    
    Array A3(A1);
    A3.setName("copy of coordinates arrayField");
    
    Array A2((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      COMPONENT(0)=velo.component(0),COMPONENT(1)=velo.component(1)));
    
    Object unic=Dom.getObject(0);
    
    array *tmpArray;
    Double coord[3];
    //  ofstream outfile("Array2.lst");
    //  outfile.setf(ios::scientific);
    //  outfile << setprecision(7);
    scal *s1,s2;
    int istep=1;
    
    step tmpStep(A2.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (i=-10;i<=10;i+=istep) {
      for (j=-10;j<=10;j+=istep) {
	
	coord[0]=0.1*i;
	coord[1]=0.1*j;
	coord[2]=0.0;
	
	tmpStep.exec(coord);
	
	tmpArray=A2.getData();
	s1=tmpArray->getComponentValues();
	s2=(*s1)+1.0;
	
	//      outfile << setw(16) << coord[0] << setw(16) << coord[1] ;
	//      outfile << setw(16) << s1->getVal() << setw(16) << s2.getVal() ;
	//      s1->writeLig(outfile);
	//      outfile << endl ;
      }
      //    outfile << endl ;
    }
    //  outfile.close();
  }
  
  
  
  void Array_test3(void) {
    Mesh MESH;
    MESH.read("oneQuad.msh","rsMesh");
    
    Domain Dom=MESH.getDefDomain();
    Dom.setName("definition Domain");
    
    Field coord((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates field",INTERPMESH=MESH));
    Field velo((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    velo.interpolate(coord,Dom,coord);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de coordonnes",
	      DERIV=false,COMPONENT(0)=coord.component(0),
	      COMPONENT(1)=coord.component(1)));
    
    Array A3(A1);
    A3.setName("copy of coordinates arrayField");
    
    Array A2((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      COMPONENT(0)=velo.component(0),COMPONENT(1)=velo.component(1)));
    
    Array A4;
    A4=exp(3.0*A2/2.0);
    A4.setName("the exponential arrayOp1");
    A4.write();
    
    Object unic=Dom.getObject(0);
    Double coordpoint[3]={0.7,0.2,0};
    Int istep=1;
    
    step tmpStep(A4.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (Int i=-1;i<=1;i+=istep) {
      for (Int j=-1;j<=1;j+=istep) {
	coordpoint[0]=0.1*i;
	coordpoint[1]=0.1*j;
	tmpStep.exec(coordpoint);
      }
    }
  }
  
}

