
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <cmath>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "array/ArrayOperators.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "mesh/Mesh.h"
#include "param/paramList.h"
#include "scal/scal.h"
#include "step/step.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  using ::sqrt ;

  void arrayOp2_test1(void) {
    int k;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parentQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field temp((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		COMPONENTNAME(0)="p",NAME="temperature"));
    
    Array A1((TYPE="arrayField",ORDER=0,NAME="Champ de temperature",
	      COMPONENT()=temp.component(0)));
    Array A2((TYPE="arrayTest",TEST=A1,NAME="equation d'energie"));
    
    Array A4;
    A4=contractedMultiplication(makeCorlocDeriv(A2,2),0,
				makeCorlocDeriv(A1,2),0);
    A4.write();
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    s2=0.0;
    step tmpStep(A4.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    for (k=0;k<4;k++) { 
      switch(k) {
      case 0 :
	coordpoint[0]=r;
	coordpoint[1]=r;
	break;
      case 1 :
	coordpoint[0]=-r;
	coordpoint[1]=r;
	break;
      case 2 :
	coordpoint[0]=r;
	coordpoint[1]=-r;
	break;
      case 3 :
	coordpoint[0]=-r;
	coordpoint[1]=-r;
	break;
      }
      tmpStep.exec(coordpoint);
      tmpArray=A4.getData();
      s1=tmpArray->getComponentValues();
      s2+=(*s1);
    }
    //  s2.writeMat(cerr);
  }
  
  
  
  void arrayOp2_test2(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parentQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field temp((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		COMPONENTNAME(0)="p",NAME="temperature"));
    
    Array A1((TYPE="arrayField",ORDER=0,NAME="Champ de temperature",
	      COMPONENT()=temp.component(0)));
    Array A2((TYPE="arrayTest",TEST=A1,NAME="equation d'energie"));
    
    Array A4;
    A4=contractedMultiplication(makeCorlocDeriv(A2,2),0,
				makeCorlocDeriv(A1,2),0);
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    step tmpStep(A4.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<1;l++) {
      s2=0.0;
      for (k=0;k<4;k++) { 
	switch(k) {
	case 0 :
	  coordpoint[0]=r;
	  coordpoint[1]=r;
	  break;
	case 1 :
	  coordpoint[0]=-r;
	  coordpoint[1]=r;
	  break;
	case 2 :
	  coordpoint[0]=r;
	  coordpoint[1]=-r;
	  break;
	case 3 :
	  coordpoint[0]=-r;
	  coordpoint[1]=-r;
	  break;
	}  
	tmpStep.exec(coordpoint);
	tmpArray=A4.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  s2.writeMat(cerr);
  }
  
  
  
  void arrayOp2_test3(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field temp((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		COMPONENTNAME(0)="p",NAME="temperature"));
    
    Array A1((TYPE="arrayField",ORDER=0,NAME="Champ de temperature",
	      COMPONENT()=temp.component(0)));
    Array A2((TYPE="arrayTest",TEST=A1,NAME="equation d'energie"));
    Array C1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de coordonnes",
	      DERIV=false,COMPONENT(0)=coord.component(0),
	      COMPONENT(1)=coord.component(1)));
    
    Array A3,A4,B1,B2,C2,C3,E1;
    A3=makeCorlocDeriv(A1,2);
    A4=makeCorlocDeriv(A2,2);
    C2=makeCorlocDeriv(C1,2);
    C3=inverse(C2);
    B1=contractedMultiplication(A3,0,C3,0);
    B2=contractedMultiplication(A4,0,C3,0);
    E1=contractedMultiplication(B1,0,B2,0);
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    step tmpStep(E1.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<1;l++) {
      s2=0.0;
      for (k=0;k<4;k++) { 
	switch(k) {
	case 0 :
	  coordpoint[0]=r;
	  coordpoint[1]=r;
	  break;
	case 1 :
	  coordpoint[0]=-r;
	  coordpoint[1]=r;
	  break;
	case 2 :
	  coordpoint[0]=r;
	  coordpoint[1]=-r;
	  break;
	case 3 :
	  coordpoint[0]=-r;
	  coordpoint[1]=-r;
	  break;
	}  
	
	tmpStep.exec(coordpoint);
	
	tmpArray=E1.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  s2.writeMat(cerr);
  }
  
  
  
  void arrayOp2_test4(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field temp((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		COMPONENTNAME(0)="p",NAME="temperature"));
    Field coord((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates"));
    
    Array A1((TYPE="arrayField",ORDER=0,NAME="Champ de temperature",
	      COMPONENT()=temp.component(0)));
    Array A2((TYPE="arrayTest",TEST=A1,NAME="equation d'energie"));
    Array C1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de coordonnes",
	      DERIV=false,COMPONENT(0)=coord.component(0),
	      COMPONENT(1)=coord.component(1)));
    
    Array A3,A4,B1,B2,C2,C3,E1;
    A3=makeCorlocDeriv(A1,2);
    A4=makeCorlocDeriv(A2,2);
    C2=makeCorlocDeriv(C1,2);
    C3=inverse(C2);
    B1=contractedMultiplication(A3,0,C3,0);
    B2=contractedMultiplication(A4,0,C3,0);
    E1=contractedMultiplication(B1,0,B2,0);
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    step tmpStep(E1.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<1;l++) {
      s2=0.0;
      for (k=0;k<4;k++) { 
	switch(k) {
	case 0 :
	  coordpoint[0]=r;
	  coordpoint[1]=r;
	  break;
	case 1 :
	  coordpoint[0]=-r;
	  coordpoint[1]=r;
	  break;
	case 2 :
	  coordpoint[0]=r;
	  coordpoint[1]=-r;
	  break;
	case 3 :
	  coordpoint[0]=-r;
	  coordpoint[1]=-r;
	  break;
	}  
	
	tmpStep.exec(coordpoint);
	
	tmpArray=E1.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  s2.writeMat(cerr);
  }
  
  
  
  void arrayOp2_test5(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field pressure((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor"));
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    Array Jacob=makeCorlocDeriv(ACoord,2);
    Array InvJacob=inverse(Jacob);  
    Array gradVelo=
      contractedMultiplication(makeCorlocDeriv(Avelo,2),1,InvJacob,0);
    Array gradMvt=
      contractedMultiplication(makeCorlocDeriv(TestMvt,2),1,InvJacob,0);
    Array divV=contract(gradVelo,0,1);
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array tmp=contractedMultiplication(gradMvt,0,D+Apressure*Unit,0);
    Array Total=contract(tmp,0,1)+testDiv*divV;
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    step tmpStep(Total.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<1;l++) {
      s2=0.0;
      for (k=0;k<9;k++) { 
	switch(k/3) {
	case 0 : coordpoint[0]=-r;
	  break;
	case 1 : coordpoint[0]=0;
	  break;
	case 2 : coordpoint[0]=r;
	  break;
	}
	switch(k%3) {
	case 0 : coordpoint[1]=-r;
	  break;
	case 1 : coordpoint[1]=0;
	  break;
	case 2 : coordpoint[1]=r;
	  break;
	}
	
	tmpStep.exec(coordpoint);
	
	tmpArray=Total.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  s2.writeMat(cerr);
  }
  
  
  
  void arrayOp2_test6(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field pressure((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor"));
    Array testDiv((TYPE="arrayTest",TEST=Apressure,NAME="Equations de continuit"));
    
    Array Jacob=makeCorlocDeriv(ACoord,2);
    Array InvJacob=inverse(Jacob);  
    Array gradVelo=
      contractedMultiplication(makeCorlocDeriv(Avelo,2),1,InvJacob,0);
    Array gradMvt=
      contractedMultiplication(makeCorlocDeriv(TestMvt,2),1,InvJacob,0);
    Array divV=contract(gradVelo,0,1);  
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    Array Total=accumulation(gradMvt,T,2)+testDiv*divV;
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    step tmpStep(Total.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<1;l++) {
      s2=0.0;
      for (k=0;k<9;k++) { 
	switch(k/3) {
	case 0 : coordpoint[0]=-r;
	  break;
	case 1 : coordpoint[0]=0;
	  break;
	case 2 : coordpoint[0]=r;
	  break;
	}
	switch(k%3) {
	case 0 : coordpoint[1]=-r;
	  break;
	case 1 : coordpoint[1]=0;
	  break;
	case 2 : coordpoint[1]=r;
	  break;
	}
	
	tmpStep.exec(coordpoint);
	
	tmpArray=Total.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  s2.writeMat(cerr);
  }
  
  
  
  void arrayOp2_test7(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field pressure((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor"));
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    Array Jacob=makeCorlocDeriv(ACoord,2);
    Array InvJacob=inverse(Jacob);  
    Array gradVelo=
      contractedMultiplication(makeCorlocDeriv(Avelo,2),1,InvJacob,0);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=
      contractedMultiplication(makeCorlocDeriv(TestMvt,2),1,InvJacob,0);
    gradMvt.setName("gradMvt");
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");  
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    step tmpStep(Total.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<1;l++) {
      s2=0.0;
      for (k=0;k<9;k++) { 
	switch(k/3) {
	case 0 : coordpoint[0]=-r;
	  break;
	case 1 : coordpoint[0]=0;
	  break;
	case 2 : coordpoint[0]=r;
	  break;
	}
	switch(k%3) {
	case 0 : coordpoint[1]=-r;
	  break;
	case 1 : coordpoint[1]=0;
	  break;
	case 2 : coordpoint[1]=r;
	  break;
	}
	
	tmpStep.exec(coordpoint);
	
	tmpArray=Total.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  s2.writeMat(cerr);
  }
  
}

