
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "array/ArrayOperators.h"
#include "array/arrayField.h"
#include "array/arrayInteg.h"
#include "constr/Constr.h"
#include "constr/constr_base.h"
#include "constr/constr1.h"
#include "coordSys/GlobLoc.h"
#include "coordSys/TensorCoordSys.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "field/field_base.h"
#include "integ/Integ.h"
#include "mesh/Mesh.h"
#include "object/object_base.h"
#include "param/param.h"
#include "param/paramList.h"
#include "plot/Plot.h"
#include "scal/scal.h"
#include "solver/Solver.h"
#include "step/step.h"
#include "step/stepList.h"

using namespace std ;
using namespace keyFE2 ;


namespace keyFE2 {
  
  void constr1_test1(void) {
    Mesh MESH;
    Domain Dom,Bnd;
    
    MESH.read("brol.msh","rsMesh");
    Dom=MESH.getDomain(1);
    Dom.setName("work Domain");
    Bnd=MESH.getDomain(2);
    Bnd.setName("boundary Domain");
    Bnd.write(cerr);
    
    Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord2((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		  COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		  NAME="coordinates (2)",INTERPMESH=MESH));
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",TENSORCOORDSYS=TCS));
    
    Array Force((TYPE="array",ORDER=1,DIM(0)=2,NAME="Champ de force",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=Double(1.0),
                 COMPONENT(1)=Double(2.0)));
    
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    Array TestPenalty((TYPE="arrayTest",TEST=Avelo,
		       NAME="Equations de pnalit"));  
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));
    
    // Navier-Stokes equations :
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+0.001*accumulation(TestMvt,Convect,1)
      +testDiv*divV+accumulation(TestMvt,Force,1);
    Total.setName("Equations de Navier-Stokes");
    
    Array Final1=integrate(Total,Dom,INTG);
    
    // Penalty equation :
    
    Array P=1.0e8*accumulation(TestPenalty,Avelo,1);
    P.setName("Problme de pnalit");
    Array Final2=integrate(P,Bnd,INTG2); 
    
    // Constr1 :
    
    //  for (Int i=0;i<100;i++) {
    Constr C1((NAME="brol",DOM=Bnd,TEST=TestMvt,EQUATION=Avelo,
	       GLOBLOC=GL2,TENSORCOORDSYS=TCS,TYPE="usual"));
    //  constr1 *c1=dynamic_cast<constr1*>(C1.getData());
    //  c1->generateExprCollect();
    //  c1->EC.solve();
    //  c1->EC.write(cerr);
    //  }
    //  return;
    
    C1.write(cerr);
    
    // problme total :
    
    Array Final=Final1+Final2;
    
    Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure));
    output.write(cerr);
    output.writeValues(cerr);
    
    Solver tmp((PROBLEM=Final,CONVERGENCE=Double(1.0e-4),
                DIVERGENCE=Double(1.0e4),NITMAX=6,
		CONSTR=C1));
    tmp.NewtonRaphson();
    tmp.write(cerr);
    tmp.writeValues(cerr);
    
    output.exec("solver1"); 
  }
  
}

