
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "array/ArrayOperators.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "mesh/Mesh.h"
#include "param/paramList.h"
#include "scal/scal.h"
#include "step/step.h"

using namespace std ;
using namespace keyFE2 ;


namespace keyFE2 {
  
  void explicitDeriv_test1(void) {
    Mesh MESH;
    Domain dom;
    
    MESH.read("oneQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field velo((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    
    velo.interpolate(coord,dom,coord);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de coordonnes",
	      DERIV=false,COMPONENT(0)=coord.component(0),
	      COMPONENT(1)=coord.component(1)));
    Array A2((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      COMPONENT(0)=velo.component(0),COMPONENT(1)=velo.component(1)));
    Array A3(A1);
    Array A4=exp(A2);
    
    A4=exp(3.0*A2/2.0);
    A4.write();
    
    A2=makeCorlocDeriv(A4,3);
    A2.write();
    
    Object unic=dom.getObject(0);
    Double coordpoint[3]={0.7,0.2,0};
    step tmpStep(A2.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    int step=7;
    for (int i=-10;i<=10;i+=step) {
      for (int j=-10;j<=10;j+=step) {
	coordpoint[0]=0.1*i;
	coordpoint[1]=0.1*j;
	tmpStep.exec(coordpoint);
      }
    }
  }
  
  
  
  void explicitDeriv_test2(void) {
    Mesh MESH;
    Domain dom;
    
    MESH.read("oneQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    
    Array ACoordField((TYPE="arrayField",ORDER=1,DIM(0)=2,
		       NAME="Champ de coordonnes",DERIV=false,
		       COMPONENT(0)=coord.component(0),
		       COMPONENT(1)=coord.component(1)));
    
    Array CartesianMetric((TYPE="array",ORDER=2,DIM(0)=3,DIM(1)=3,
			   NAME="Metric tensor in cartesian coordinates",
			   COMPONENT(0,0)=Double(1.0),
                           COMPONENT(1,1)=Double(1.0),
			   COMPONENT(2,2)=Double(1.0)));
    
    Array CartesianChrist2((TYPE="array",ORDER=3,DIM(0)=3,DIM(1)=3,
			    DIM(2)=3,
			    NAME="Second Christoffel symbol in cartesian coordinates"));
    
    Array CylindricalCoord((TYPE="array",ORDER=1,DIM(0)=3,
			    NAME="3D cylindrical coordinates",
			    COMPONENT(0)=Double(2.0),
                            COMPONENT(1)=Double(1.0),
			    COMPONENT(2)=Double(1.0)));
    
    Array R((TYPE="arrayComp",ORDER=0,NAME="r",
	     COMPONENT()=CylindricalCoord.component(0)));
    Array Theta((TYPE="arrayComp",ORDER=0,NAME="theta",
		 COMPONENT()=CylindricalCoord.component(1)));
    Array Z((TYPE="arrayComp",ORDER=0,NAME="z",
	     COMPONENT()=CylindricalCoord.component(2)));
    
    Array X0=R*cos(Theta);
    X0.setName("x");
    Array X1=R*sin(Theta);
    X1.setName("y");
    Array X2=Z;
    X2.setName("z");
    
    Array CartesianCoord((TYPE="arrayComp",ORDER=1,DIM(0)=3,
			  NAME="3D cartesian coordinates",
			  COMPONENT(0)=X0.component(),
			  COMPONENT(1)=X1.component(),
			  COMPONENT(2)=X2.component()));
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.7,0.2,0};
    
    step tmpStep1(CartesianCoord.getData(),unic.getData(),unic.getDirect());
    tmpStep1.buildActiveFieldList();
    tmpStep1.exec(coordpoint);
    
    int nbrScal=3;
    scal **scalList=new scal* [nbrScal];
    {
      array *data=CylindricalCoord.getData();
      scal *tmpScal=data->getComponentValues();
      scalList[0]=tmpScal;
      scalList[1]=tmpScal+1;
      scalList[2]=tmpScal+2;
    }
    
    Array Jacob1=CartesianCoord.makeExplicitDeriv(nbrScal,scalList);
    Jacob1.setName("Jacobienne directe");
    step tmpStep2(Jacob1.getData(),unic.getData(),unic.getDirect());
    tmpStep2.buildActiveFieldList();
    tmpStep2.exec(coordpoint);
    
    Array Jacob2=inverse(Jacob1);
    Jacob2.setName("Jacobienne inverse");
    step tmpStep3(Jacob2.getData(),unic.getData(),unic.getDirect());
    tmpStep3.buildActiveFieldList();
    tmpStep3.exec(coordpoint);
    
    Array CylindricalMetric;
    {
      Array tmp=contractedMultiplication(Jacob1,0,CartesianMetric,0);
      CylindricalMetric=contractedMultiplication(tmp,1,Jacob1,0);
    }
    CylindricalMetric.setName("Metric tensor in cylindrical coordinates");
    step tmpStep4(CylindricalMetric.getData(),unic.getData(),unic.getDirect());
    tmpStep4.buildActiveFieldList();
    tmpStep4.exec(coordpoint);
    
    Array CylindricalChrist2;
    {
      Array tmpG1=contractedMultiplication(CartesianChrist2,0,Jacob1,0);
      Array tmpG2=contractedMultiplication(tmpG1,1,Jacob1,0);
      Array tmpG3=contractedMultiplication(tmpG2,2,Jacob2,0);
      
      Array tmpD1=Jacob1.makeExplicitDeriv(nbrScal,scalList);
      Array tmpD2=contractedMultiplication(tmpD1,0,Jacob2,1);
      
      CylindricalChrist2=tmpG3+tmpD2;
    }
    CylindricalChrist2.setName("Metric tensor in cylindrical coordinates");
    step tmpStep5(CylindricalChrist2.getData(),unic.getData(),unic.getDirect());
    tmpStep5.buildActiveFieldList();
    tmpStep5.exec(coordpoint);
    CylindricalChrist2.write();
    
    delete[] scalList;
    scalList=NULL;
  }
  
}

