
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <cmath>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "array/ArrayOperators.h"
#include "array/arrayInteg.h"
#include "coordSys/GlobLoc.h"
#include "coordSys/TensorCoordSys.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "field/field_base.h"
#include "integ/Integ.h"
#include "mesh/Mesh.h"
#include "object/object_base.h"
#include "param/param.h"
#include "param/paramList.h"
#include "scal/scal.h"
#include "step/step.h"
#include "step/stepList.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  using ::sqrt ;
  
  void globloc_test1(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field pressure((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor"));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    Array Jacob=makeCorlocDeriv(ACoord,2);
    Jacob.setName("Jacobienne");
    Array InvJacob=inverse(Jacob);
    InvJacob.setName("Jacobienne Inverse");
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    
    Array gradVelo=Avelo.makeCoordDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makeCoordDeriv(GL);
    gradMvt.setName("gradMvt");
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=sqrt(0.6);
    array *tmpArray;
    
    for (l=0;l<1;l++) {
      //    cerr << "execution " << l << "." << endl ;  
      s2=0.0;
      for (k=0;k<9;k++) { 
	switch(k/3) {
	case 0 : coordpoint[0]=-r;
	  break;
	case 1 : coordpoint[0]=0;
	  break;
	case 2 : coordpoint[0]=r;
	  break;
	}
	switch(k%3) {
	case 0 : coordpoint[1]=-r;
	  break;
	case 1 : coordpoint[1]=0;
	  break;
	case 2 : coordpoint[1]=r;
	  break;
	}
	tmpArray=Total.getData();
	step tmpStep(tmpArray,unic.getData(),unic.getDirect());
	tmpStep.buildActiveFieldList();
	tmpStep.exec(coordpoint);
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  ofstream fout("globloc1.lst");
    //  s2.writeMat(fout);
  }
  
  
  
  void globloc_test2(void) {
    Int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field pressure((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));  
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
                TENSORCOORDSYS=TCS));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=sqrt(0.6);
    array *tmpArray;
    
    for (l=0;l<1;l++) {
      //    if (l%100==0)
      //       cerr << "execution " << l << "." << endl ;
      s2=0.0;
      
      for (k=0;k<9;k++) { 
	switch(k/3) {
	case 0 : coordpoint[0]=-r;
	  break;
	case 1 : coordpoint[0]=0;
	  break;
	case 2 : coordpoint[0]=r;
	  break;
	}
	switch(k%3) {
	case 0 : coordpoint[1]=-r;
	  break;
	case 1 : coordpoint[1]=0;
	  break;
	case 2 : coordpoint[1]=r;
	  break;
	}
	tmpArray=Total.getData();
	step tmpStep(tmpArray,unic.getData(),unic.getDirect());
	tmpStep.buildActiveFieldList();
	tmpStep.exec(coordpoint);
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  ofstream fout("globloc2.lst");
    //  s2.writeMat(fout);
  }
  
  
  
  void globloc_test3(void) {
    Int l;
    Mesh MESH;
    Domain Dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    Dom=MESH.getDefDomain();
    Dom.setName("the domain");
    
    Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
                TENSORCOORDSYS=TCS));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Array Final=integrate(Total,Dom,INTG);
    
    Object unic=Dom.getObject(0);
    
    array *tmpArray=NULL;
    for (l=0;l<1;l++) {
      //    if (l%100==0)
      //       cerr << "execution " << l << "." << endl ;
      tmpArray=Final.getData();
      step tmpStep(tmpArray,unic.getData(),unic.getDirect());
      tmpStep.buildActiveFieldList();
      tmpStep.exec();
    }
    //  ofstream fout("globloc3.lst");  
    //  scal *s1=tmpArray->getComponentValues();
    //  s1->writeMat(fout);
  }
  
  
  
  void globloc_test4(void) {
    Mesh MESH;
    Domain Dom;
    
    MESH.read("brol.msh","rsMesh");
    Dom=MESH.getDomain(1);
    Dom.setName("the domain");
    
    Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));  
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
                TENSORCOORDSYS=TCS));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Array Final=integrate(Total,Dom,INTG);
    
    Int objectsNbr=Dom.getObjectsNbr();
    Object unic;
    
    array *tmpArray=NULL;
    scal *s1=NULL;
    
    static Int counter=0;
    
    //  ofstream fout("globloc4.lst");  
    for (Int l=0;l<1;l++) {
      for (Int k=0;k<objectsNbr;k++) {
	counter++;
	//      if (counter%100==0)
	//        cerr << "object counter " << counter << "." << endl ;
	tmpArray=Final.getData();
	unic=Dom.getObject(k);
	step tmpStep(tmpArray,unic.getData(),unic.getDirect());
	tmpStep.buildActiveFieldList();
	tmpStep.exec();
	s1=tmpArray->getComponentValues();
	//      s1->writeMat(fout);
	//      fout << endl ;
	//      fout << endl ;
      }
    }
  }
  
  
  
  void globloc_test5(void) {
    Mesh MESH;
    Domain Dom;
    
    MESH.read("brol.msh","rsMesh");
    Dom=MESH.getDomain(0);
    Dom.setName("the domain");
    
    Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord2((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		  COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		  NAME="coordinates",INTERPMESH=MESH));
    Field coord=MESH.getCoordinateField(Dom);
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
                TENSORCOORDSYS=TCS));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");  
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Array Final=integrate(Total,Dom,INTG);
    
    Int objectsNbr=Dom.getObjectsNbr();
    Object unic;
    
    array *tmpArray=NULL;
    //  scal *s1=NULL;
    
    static int counter=0;
    //  ofstream fout("globloc5.lst");
    for (Int l=0;l<1;l++) {
      for (Int k=0;k<objectsNbr;k++) {
	counter++;
	//      if (counter%100==0)
	//        cerr << "object counter " << counter << "." << endl ;
	tmpArray=Final.getData();
	unic=Dom.getObject(k);
	step tmpStep(tmpArray,unic.getData(),unic.getDirect());
	tmpStep.buildActiveFieldList();
	tmpStep.exec();
	//      s1=tmpArray->getComponentValues();
	//      s1->writeMat(fout);
	//      fout << endl ;
	//      fout << endl ;
      }
    }
  }
  
}
