
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "array/ArrayOperators.h"
#include "array/arrayInteg.h"
#include "coordSys/GlobLoc.h"
#include "coordSys/TensorCoordSys.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "field/field_base.h"
#include "integ/Integ.h"
#include "mesh/Mesh.h"
#include "object/object_base.h"
#include "param/param.h"
#include "param/paramList.h"
#include "plot/Plot.h"
#include "scal/scal.h"
#include "solver/Solver.h"
#include "step/step.h"
#include "step/stepList.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  void solver_test1(void) {
    Mesh MESH;
    Domain Dom;
    
    MESH.read("brol.msh","rsMesh");
    Dom=MESH.getDomain(1);
    Dom.setName("the domain");
    
    Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord2((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		  COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		  NAME="coordinates",INTERPMESH=MESH));
    Field coord=MESH.getCoordinateField(Dom);
    
    pressure.write(cerr);
    pressure.writeValues(cerr);
    coord.write(cerr);
    coord.writeValues(cerr);
    coord2.write(cerr);
    coord2.writeValues(cerr);
    velocities.write(cerr);
    velocities.writeValues(cerr);
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
                TENSORCOORDSYS=TCS));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");  
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Array Final=integrate(Total,Dom,INTG);
    
    Solver tmp((PROBLEM=Final,CONVERGENCE=Double(1.0e-4),
                DIVERGENCE=Double(1.0e4),NITMAX=6));
  }
  
  
  
  void solver_test2(void) {
    Mesh MESH;
    Domain Dom,Bnd;
    
    MESH.read("brol.msh","rsMesh");
    Dom=MESH.getDomain(1);
    Dom.setName("work domain");
    Bnd=MESH.getDomain(2);
    Bnd.setName("boundary domain");
    Bnd.write(cerr);
    
    Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord2((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		  COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		  NAME="coordinates (2)",INTERPMESH=MESH));
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="Champ de pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
                TENSORCOORDSYS=TCS));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    Array TestPenalty((TYPE="arrayTest",TEST=Avelo,
		       NAME="Equations de pnalit"));  
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));
    
    // Navier-Stokes equations :
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");  
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+accumulation(TestMvt,Convect,1)
      +testDiv*divV;
    Total.setName("Equations de Navier-Stokes");
    
    Array Final1=integrate(Total,Dom,INTG);
    
    // Penalty equation :
    
    Array P=1.0e8*accumulation(TestPenalty,Avelo,1);
    P.setName("Problme de pnalit");
    Array Final2=integrate(P,Bnd,INTG2); 
    
    // problme total :
    
    Array Final=Final1+Final2;
    
    Solver tmp((PROBLEM=Final,CONVERGENCE=Double(1.0e-4),
                DIVERGENCE=Double(1.0e4),NITMAX=6));
  }
  
  
  
  void solver_test3(void) {
    Mesh MESH;
    Domain Dom,Bnd;
    
    MESH.read("brol.msh","rsMesh");
    Dom=MESH.getDomain(1);
    Dom.setName("work domain");
    Bnd=MESH.getDomain(2);
    Bnd.setName("boundary domain");
    Bnd.write(cerr);
    
    Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		    COMPONENTNAME(0)="p",NAME="pressure"));
    Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		      COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		      NAME="velocities"));
    Field coord2((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		  COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		  NAME="coordinates (2)",INTERPMESH=MESH));
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Avelo((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="vitesses",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
		 COMPONENT(1)=velocities.component(1)));
    Array Apressure((TYPE="arrayField",ORDER=0,NAME="pression",
		     TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
    Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
		COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
                TENSORCOORDSYS=TCS));
    
    Array Force((TYPE="array",ORDER=1,DIM(0)=2,NAME="Champ de force",
		 TENSORCOORDSYS=TCS,
                 COMPONENT(0)=Double(1.0),COMPONENT(1)=Double(2.0)));
    
    Array TestMvt((TYPE="arrayTest",TEST=Avelo,
		   NAME="Equations de mouvement"));  
    Array testDiv((TYPE="arrayTest",TEST=Apressure,
		   NAME="Equations de continuit"));
    Array TestPenalty((TYPE="arrayTest",TEST=Avelo,
		       NAME="Equations de pnalit"));  
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));
    
    // Navier-Stokes equations :
    
    Array gradVelo=Avelo.makePhysicalDeriv(GL);
    gradVelo.setName("Champ gradients de vitesses");
    Array Convect=contractedMultiplication(Avelo,0,gradVelo,1);
    Convect.setName("Termes convectifs");
    Array gradMvt=TestMvt.makePhysicalDeriv(GL);
    gradMvt.setName("gradMvt");
    
    Array divV=contract(gradVelo,0,1);
    divV.setName("divergence des vitesses");
    
    Array D=0.5*(gradVelo+transpose(gradVelo,0,1));
    Array T=D+Apressure*Unit;
    T.setName("Tenseur des tensions de Cauchy");
    Array Total=accumulation(gradMvt,T,2)+0.001*accumulation(TestMvt,Convect,1)
      +testDiv*divV+accumulation(TestMvt,Force,1);
    Total.setName("Equations de Navier-Stokes");
    
    Array Final1=integrate(Total,Dom,INTG);
    
    // Penalty equation :
    
    Array P=1.0e8*accumulation(TestPenalty,Avelo,1);
    P.setName("Problme de pnalit");
    Array Final2=integrate(P,Bnd,INTG2); 
    
    // problme total :
    
    Array Final=Final1+Final2;
    
    Solver tmp((PROBLEM=Final,CONVERGENCE=Double(1.0e-4),
                DIVERGENCE=Double(1.0e4),NITMAX=6));
    tmp.NewtonRaphson();
    
    Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure));
    output.exec("solver1"); 
  }
  
  
  void solver_test4(void) {
    Mesh MESH;
    Domain Dom,Bnd;
    
    MESH.read("carre.msh","rsMesh");
    Dom=MESH.getDomain(0);
    Dom.setName("work domain");
    Bnd=MESH.getDomain(1);
    Bnd.setName("boundary domain");
    Bnd.write(cerr);
    
    Field temperature((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		       COMPONENTNAME(0)="T",NAME="temperature"));
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    
    TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));
    
    Array Atemp((TYPE="arrayField",ORDER=0,NAME="Champ de temprature",
		 TENSORCOORDSYS=TCS,COMPONENT()=temperature.component(0)));
    Array Source((TYPE="array",ORDER=0,NAME="source d'nergie",
		  TENSORCOORDSYS=TCS,COMPONENT(0)=Double(1.0)));
    
    Array Energie((TYPE="arrayTest",TEST=Atemp,NAME="Equation d'nergie"));  
    Array TestPenalty((TYPE="arrayTest",TEST=Atemp,
		       NAME="Equation de pnalit"));  
    
    GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
    GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
    
    Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
    Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));
    
    // Equations d'energie :
    
    Array gradTemp=Atemp.makePhysicalDeriv(GL);
    gradTemp.setName("Champ gradients de temprature");
    Array gradEnerg=Energie.makePhysicalDeriv(GL);
    gradEnerg.setName("gradEnerg");
    
    Array Total=accumulation(gradEnerg,gradTemp,1)+Energie*Source;
    Total.setName("Equation de Poisson");
    
    Array Final1=integrate(Total,Dom,INTG);
    
    // Penalty equation :
    
    Array P=1.0e8*Atemp*TestPenalty;
    P.setName("Problme de pnalit");
    Array Final2=integrate(P,Bnd,INTG2); 
    
    // problme total :
    
    Array Final=Final1+Final2;
  }
  
}

