#ifndef TIMEDEP2_H
#define TIMEDEP2_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>
#include <iostream>

#include "types/types.h"
  
namespace keyFE2 {
  
  class timeDep ;
  class Field ;
  class Solver ;
  class param ;
  class paramList ;
  
  class TimeDep {
    
    private :
      
      void *data ;
    
    public :
      
      TimeDep(void) ;
      TimeDep(param P) ;
      TimeDep(paramList PL) ;
      TimeDep(const TimeDep &) ;
      ~TimeDep(void) ;
      
      void setData(timeDep*) ;
      timeDep* getData(void) const ;
      void setName(std::string s) ;
      std::string getName(void) const ;
      
      void addActive(Field &) ;
      void addInflu(Field &) ;
      void addInflu(Solver &) ;
      
      void write(void) const ;
      void writeValues(void) const ;
      void write(std::ostream &out) const ;
      void writeValues(std::ostream &out) const ;
      
      TimeDep operator=(const TimeDep &) ;
      
      bool isToBeContinued(void) ;
      
      Double getThetaDT(void) const ;
      Int getCurrentStepIndex(void) const ;
      Double getCurrentTimeStep(void) const ;
      Double getCurrentTime(void) const ;
      Double getPreviousTime(void) const ;
    
  } ;
    
}

#endif



