#ifndef TIMEDEP_H
#define TIMEDEP_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "types/allTypes.h"
#include "util/utils.h"
#include "print/print.h"

namespace keyFE2 {
  
  class field ;
  class solver ;
  
  class timeDep : public virtual name, public virtual parents,
		  public virtual print, public virtual newPoint {
					 
    private :
      
      // Initial data :
      
      Int timeScheme ;
    
      Int maxStepsNbr ;
    
      Double epsilon ;
    
      Double dtMin ;
      Double dtMax ;
      Double dtInit ;
      Double tInit ;
      Double tFinal ;
      
      Int influSolversNbr ;
      solver **influSolvers ;
      
      Int activeFieldsNbr ;
      field **activeFields ; // Zp=Z[n+1]
      bool *activeInfluence ;
      field **previousFields ; // Zr
      
      // Working data :
      
      Int currentState ; // -1 : none, 0 : failed, 1 : continue
      Int currentStep ;
      Double t ;
      
      Double theta ;
      Double lastDt[5] ;
      
      field **workingFields ; // dZ[n]/dt, dZ[n-1]/dt for active fields
      Int allFieldsNbr ;
      field *allFieldsN ; // Z[n] for all fields
      
    public :
      
      timeDep(void) ;
      timeDep(const timeDep &in) ;
      ~timeDep(void) ;
    
      void operator=(const timeDep &in) ;
    
      void setTimeScheme(std::string timeSchemeName) ;
      void setTimeScheme(Int timeSchemeNbr) ;
      std::string getTimeSchemeName(void) const ;
      Int getTimeScheme(void) const ;
      
      void setMaxStepsNbr(Int nbr) ;
      void setEpsilon(Double e) ;
      void setDtMin(Double dt) ;
      void setDtMax(Double dt) ;
      void setDtInit(Double dt) ;
      void setTInit(Double t) ;
      void setTFinal(Double t) ;
      
      void addInfluField(field *F) ;
      field* addActiveField(field *F,Int influ=-1) ;
      void addInfluSolver(solver *S) ;
      
      // Execution :
      
      void init(void) ;
      bool solversCheck(void) const ;
      void solversReinit(void) const ;
      void nextStep(void) ;
      void redoStep(void) ;
      void makePrediction(void) ;
      void clear(void) ;
      bool isToBeContinued(void) ;
      
      bool isActive(void) const ;
      
      Double getThetaDT(void) const ;
      Int getCurrentStepIndex(void) const ;
      Double getCurrentTimeStep(void) const ;
      Double getCurrentTime(void) const ;
      Double getPreviousTime(void) const ;

      void setNewPoint(bool) ;
      void accessNbrInit(void) ;
      
      virtual printTree getPrintTreeData(void) const ;
      virtual printTree getPrintTreeValues(void) const ;
      virtual printTree getPrintTree(bool values) const ;
      
  } ;
  
}

#endif

