
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <typeinfo>

#include "parents.h"
#include "err/err.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  license parents::magic;
  
  
  Int delinkAndDelete(parents *child,void *parent) {
    try {
      Int nbr=0;
      if (child!=NULL) {
	nbr=child->supParent(parent);
	if (nbr==0) {
	  delete child;
	}
      }
      return(nbr);
    }
    catchAll("delinkAndDelete","")
  }
  
}



using namespace keyFE2 ;



parents::parents(void) {
  parentsSize=0;
  parentsCapacity=0;
  Parents=NULL;
}



parents::~parents(void) {
  try {
    parentsSize=0;
    parentsCapacity=0;
    delete[] Parents;
    Parents=NULL;
  }
  catchAll("parents::~parents(void)","")
}



Int parents::addParent(void *parent) {
  try {
    if (this==NULL)
      return(0);
    
    license brol;
    brol.touch();
    
    Int i;
    // One checks whether the parent is in the list :
    for (i=0;i<parentsSize;i++)
      if (parent==Parents[i])
	errThrow("parents::addParent","parent already in the list.");
    
    // If the list is to be resized, one does it :
    if (parentsSize==parentsCapacity) {
      void **newParents=new void* [++parentsCapacity];
      for (i=0;i<parentsSize;i++)
	newParents[i]=Parents[i];
      delete[] Parents;
      Parents=newParents;
    }
    
    // The parent is added to the list :
    Parents[parentsSize]=parent;
    parentsSize++;
    
    return(parentsSize);
  }
  catchAll("Int parents::addParent(void *parent)","")
}



Int parents::supParent(void *parent) {
  try {
    Int i,index=-1;
    bool inTheList=false;
    
    // One checks whether the parent is in the list :
    for (i=0;i<parentsSize;i++) {
      if (parent==Parents[i]) {
	inTheList=true;
	index=i;
      }
    }
    if (!inTheList) {
      //using namespace std;
      //using std::name;
      //cerr << typeid(*this).name() << endl ;
      //cerr << typeid(*parent).name() << endl ;
      errThrow("parents::supParent","parent not in the list.");
    }
    
    // One deletes the parent :
    parentsSize--;
    for (i=index;i<parentsSize;i++)
      Parents[i]=Parents[i+1];
    
    return(parentsSize);
  }
  catchAll("Int parents::supParent(void *parent)","")
}



void parents::initParents(void) {
  try {
    parentsSize=0;
    parentsCapacity=0;
    if (Parents!=NULL)
      delete[] Parents;
    Parents=NULL;
  }
  catchAll("void parents::initParents(void)","")
}



Int parents::getParentsNbr(void) const {
  return(parentsSize);
}



void* parents::getParent(Int index) const {
  try {
    if (index>parentsSize||index<0)
      errThrow("parents::getParent",string("invalid parent index : ")
	       +strFromInt(index)+".");
    return(Parents[index]);
  }
  catchAll("void* parents::getParent(Int index) const","")
}


